@@ -12,9 +12,9 @@ package Image::ExifTool::APP12;
 
 use strict;
 use vars qw($VERSION);
-use Image::ExifTool qw(:DataAccess);
+use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.08';
+$VERSION = '1.12';
 
 sub ProcessAPP12($$$);
 sub ProcessDucky($$$);
@@ -24,6 +24,7 @@ sub WriteDucky($$$);
 %Image::ExifTool::APP12::PictureInfo = (
     PROCESS_PROC => \&ProcessAPP12,
     GROUPS => { 0 => 'APP12', 1 => 'PictureInfo', 2 => 'Image' },
+    PRIORITY => 0,
     NOTES => q{
         The JPEG APP12 "Picture Info" segment was used by some older cameras, and
         contains ASCII-based meta information.  Below are some tags which have been
@@ -133,14 +134,12 @@ sub WriteDucky($$$);
 # Returns: New directory data or undefined on error
 sub WriteDucky($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
-    $exifTool or return 1;    # allow dummy access to autoload this package
+    my ($et, $dirInfo, $tagTablePtr) = @_;
+    $et or return 1;    # allow dummy access to autoload this package
     my $dataPt = $$dirInfo{DataPt};
     my $pos = $$dirInfo{DirStart};
-    my $newTags = $exifTool->GetNewTagInfoHash($tagTablePtr);
+    my $newTags = $et->GetNewTagInfoHash($tagTablePtr);
     my @addTags = sort { $a <=> $b } keys(%$newTags);
-    my $verbose = $exifTool->Options('Verbose');
-    my $out = $exifTool->Options('TextOut');
     my ($dirEnd, %doneTags);
     if ($dataPt) {
         $dirEnd = $pos + $$dirInfo{DirLen};
@@ -159,7 +158,7 @@ sub WriteDucky($$$)
             $len = Get16u($dataPt, $pos + 2);
             $pos += 4;
             if ($pos + $len > $dirEnd) {
-                $exifTool->Warn('Invalid Ducky block length');
+                $et->Warn('Invalid Ducky block length');
                 return undef;
             }
             $val = substr($$dataPt, $pos, $len);
@@ -172,22 +171,22 @@ sub WriteDucky($$$)
         $doneTags{$tag} = 1;
         my $tagInfo = $$newTags{$tag};
         if ($tagInfo) {
-            my $nvHash = $exifTool->GetNewValueHash($tagInfo);
+            my $nvHash = $et->GetNewValueHash($tagInfo);
             my $isNew;
             if (defined $val) {
-                if (Image::ExifTool::IsOverwriting($nvHash, $val)) {
-                    $exifTool->VerboseValue("- Ducky:$$tagInfo{Name}", $val);
+                if ($et->IsOverwriting($nvHash, $val)) {
+                    $et->VerboseValue("- Ducky:$$tagInfo{Name}", $val);
                     $isNew = 1;
                 }
             } else {
-                next unless Image::ExifTool::IsCreating($nvHash);
+                next unless $$nvHash{IsCreating};
                 $isNew = 1;
             }
             if ($isNew) {
-                $val = Image::ExifTool::GetNewValues($nvHash);
-                ++$exifTool->{CHANGED};
+                $val = $et->GetNewValues($nvHash);
+                ++$$et{CHANGED};
                 next unless defined $val;   # next if tag is being deleted
-                $exifTool->VerboseValue("+ Ducky:$$tagInfo{Name}", $val);
+                $et->VerboseValue("+ Ducky:$$tagInfo{Name}", $val);
             }
         }
         $newData .= pack('nn', $tag, length $val) . $val;
@@ -208,7 +207,7 @@ sub WriteDucky($$$)
 #      c) N bytes: block data
 sub ProcessDucky($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $pos = $$dirInfo{DirStart};
     my $dirEnd = $pos + $$dirInfo{DirLen};
@@ -220,11 +219,11 @@ sub ProcessDucky($$$)
         my $len = Get16u($dataPt, $pos + 2);
         $pos += 4;
         if ($pos + $len > $dirEnd) {
-            $exifTool->Warn('Invalid Ducky block length');
+            $et->Warn('Invalid Ducky block length');
             last;
         }
         my $val = substr($$dataPt, $pos, $len);
-        $exifTool->HandleTag($tagTablePtr, $tag, $val,
+        $et->HandleTag($tagTablePtr, $tag, $val,
             DataPt => $dataPt,
             DataPos => $$dirInfo{DataPos},
             Start => $pos,
@@ -241,7 +240,7 @@ sub ProcessDucky($$$)
 # Returns: 1 on success, 0 if this wasn't a recognized APP12
 sub ProcessAPP12($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $dirStart = $$dirInfo{DirStart} || 0;
     my $dirLen = $$dirInfo{DirLen} || (length($$dataPt) - $dirStart);
@@ -251,7 +250,7 @@ sub ProcessAPP12($$$)
     } else {
         pos($$dataPt) = $$dirInfo{DirStart};
     }
-    my $verbose = $exifTool->Options('Verbose');
+    my $verbose = $et->Options('Verbose');
     my $success = 0;
     my $section = '';
     pos($$dataPt) = 0;
@@ -264,24 +263,24 @@ sub ProcessAPP12($$$)
         my $token = $1;
         # was this a section name?
         if ($token =~ /^\[(.*)\]/) {
-            $exifTool->VerboseDir($1) if $verbose;
+            $et->VerboseDir($1) if $verbose;
             $section = ($token =~ /\[(\S+) ?Info\]/i) ? $1 : '';
             $success = 1;
             next;
         }
-        $exifTool->VerboseDir($$dirInfo{DirName}) if $verbose and not $success;
+        $et->VerboseDir($$dirInfo{DirName}) if $verbose and not $success;
         $success = 1;
         my ($tag, $val) = ($token =~ /(\S+)=(.+)/);
-        my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tag);
-        $verbose and $exifTool->VerboseInfo($tag, $tagInfo, Value => $val);
+        my $tagInfo = $et->GetTagInfo($tagTablePtr, $tag);
+        $verbose and $et->VerboseInfo($tag, $tagInfo, Value => $val);
         unless ($tagInfo) {
             # add new tag to table
             $tagInfo = { Name => ucfirst $tag };
             # put in Camera group if information in "Camera" section
             $$tagInfo{Groups} = { 2 => 'Camera' } if $section =~ /camera/i;
-            Image::ExifTool::AddTagToTable($tagTablePtr, $tag, $tagInfo);
+            AddTagToTable($tagTablePtr, $tag, $tagInfo);
         }
-        $exifTool->FoundTag($tagInfo, $val);
+        $et->FoundTag($tagInfo, $val);
     }
     return $success;
 }
@@ -306,7 +305,7 @@ APP12 meta information.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
