@@ -19,7 +19,7 @@ use Image::ExifTool qw(:DataAccess);
 use Image::ExifTool::Canon;
 use Image::ExifTool::Exif;
 
-$VERSION = '1.40';
+$VERSION = '1.46';
 
 sub ProcessCanonCustom($$$);
 sub ProcessCanonCustom2($$$);
@@ -1273,10 +1273,7 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
         {
             Name => 'ISOExpansion',
             Notes => 'other models',
-            PrintConv => {
-                0 => 'Off',
-                1 => 'On',
-            },
+            PrintConv => \%offOn,
         },
     ],
     0x0104 => {
@@ -1295,7 +1292,7 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     0x0106 => {
         Name => 'AEBShotCount',
         Count => -1,
-        Notes => '1 value for the 1DmkIII, 2 for the 1DmkIV',
+        Notes => '1 value for the 1DmkIII and 5DmkIII, 2 for the 1DmkIV',
         PrintConv => [{
             0 => 3,
             1 => 2,
@@ -1437,17 +1434,16 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
         },
         {
             Name => 'FlashSyncSpeedAv',
-            Condition => '$$self{Model} =~ /\b(5D Mark II|500D|T1i|Kiss X3|550D|T2i|Kiss X4)\b/',
-            Notes => '5D Mark II, 500D and 550D',
+            Condition => '$$self{Model} =~ /\bEOS-1Ds? Mark III\b/',
+            Notes => '1D Mark III and 1Ds Mark III',
             PrintConv => {
                 0 => 'Auto',
-                1 => '1/200-1/60 Auto',
-                2 => '1/200 Fixed',
+                1 => '1/300 Fixed',
             },
         },
         {
             Name => 'FlashSyncSpeedAv',
-            Condition => '$$self{Model} =~ /\b(EOS-1D Mark IV)\b/',
+            Condition => '$$self{Model} =~ /\bEOS-1D Mark IV\b/',
             Notes => '1D Mark IV',
             PrintConv => {
                 0 => 'Auto',
@@ -1457,10 +1453,11 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
         },
         {
             Name => 'FlashSyncSpeedAv',
-            Notes => '1D Mark III',
+            Notes => '5D Mark II, 5D Mark III, 500D, 550D, 600D and 1100D',
             PrintConv => {
                 0 => 'Auto',
-                1 => '1/300 Fixed',
+                1 => '1/200-1/60 Auto',
+                2 => '1/200 Fixed',
             },
         },
     ],
@@ -1486,8 +1483,11 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     0x0202 => [
         {
             Name => 'HighISONoiseReduction',
-            Condition => '$$self{Model} =~ /\b(50D|60D|5D Mark II|7D|500D|T1i|Kiss X3|550D|T2i|Kiss X4)\b/',
-            Notes => '50D, 60D, 500D, 550D, 5DmkII and 7D',
+            Condition => q{
+                $$self{Model} =~ /\b(50D|60D|5D Mark II|7D|500D|T1i|Kiss X3|550D|T2i|Kiss X4)\b/ or
+                $$self{Model} =~ /\b(600D|T3i|Kiss X5|1100D|T3|Kiss X50)\b/
+            },
+            Notes => '50D, 60D, 500D, 550D, 600D, 1100D, 5DmkII and 7D',
             PrintConv => {
                 0 => 'Standard',
                 1 => 'Low',
@@ -1575,6 +1575,23 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
             },
         },
     ],
+    0x040a => { # new for 5DmkIII
+        Name => 'ViewfinderWarnings',
+        PrintConv => { BITMASK => { # (NC)
+            0 => 'Monochrome',              # (have seen for: 5DmkII, 6D)
+            1 => 'WB corrected',            # (have seen for: 5DmkII, 6D)
+            2 => 'One-touch image quality', # (have seen for: 5DmkII; doesn't exist for 6D)
+            3 => 'ISO expansion',           # (have seen for: 5DmkII)
+            4 => 'Spot metering',           # (have seen for: 5DmkII, 6D)
+        }},
+    },
+    0x040b => { # new for 5DmkIII
+        Name => 'LVShootingAreaDisplay',
+        PrintConv => {
+            0 => 'Masked',
+            1 => 'Outlined',
+        },
+    },
     #### 3a) Auto focus
     0x0501 => {
         Name => 'USMLensElectronicMF',
@@ -1587,11 +1604,11 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     0x0502 => {
         Name => 'AIServoTrackingSensitivity',
         PrintConv => {
-           -2 => 'Slow',
-           -1 => 'Medium Slow',
+           -2 => 'Slow',        # (Locked on -2 for 6D)
+           -1 => 'Medium Slow', # (Locked on -1 for 6D)
             0 => 'Standard',
-            1 => 'Medium Fast',
-            2 => 'Fast',
+            1 => 'Medium Fast', # (Responsive +1 for 6D)
+            2 => 'Fast',        # (Responsive +2 for 6D)
         },
     },
     0x0503 => {
@@ -1732,8 +1749,8 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     0x050e => [
         {
             Name => 'AFAssistBeam',
-            Condition => '$$self{Model} =~ /\b1D Mark IV\b/',
-            Notes => '1D Mark IV',
+            Condition => '$$self{Model} =~ /\b(1D Mark IV|6D)\b/',
+            Notes => '1D Mark IV and 6D',
             PrintConv => {
                 0 => 'Emits',
                 1 => 'Does not emit',
@@ -1846,6 +1863,25 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
             1 => 'AF point selection',
         },
     },
+    0x0518 => { # new for 6D
+        Name => 'AccelerationTracking',
+    },
+    0x0519 => { # new for 6D
+        Name => 'AIServoFirstImagePriority',
+        PrintConv => { # (NC)
+            -1 => 'Release priority',
+            0 => 'Equal priority',
+            1 => 'Focus priority',
+        },
+    },
+    0x051a => { # new for 6D
+        Name => 'AIServoSecondImagePriority',
+        PrintConv => { # (NC)
+            -1 => 'Shooting speed priority',
+            0 => 'Equal priority',
+            1 => 'Focus priority',
+        },
+    },
     #### 3b) Drive
     0x060f => {
         Name => 'MirrorLockup',
@@ -1886,8 +1922,11 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     0x0701 => [
         {
             Name => 'Shutter-AELock',
-            Condition => '$$self{Model} =~ /\b(1000D|XS|Kiss F|500D|T1i|Kiss X3|550D|T2i|Kiss X4)\b/',
-            Notes => '500D, 550D and 1000D',
+            Condition => q{
+                $$self{Model} =~ /\b(1000D|XS|Kiss F|500D|T1i|Kiss X3|550D|T2i|Kiss X4)\b/ or
+                $$self{Model} =~ /\b(600D|T3i|Kiss X5|1100D|T3|Kiss X50)\b/
+            },
+            Notes => '500D, 550D, 600D, 1000D and 1100D',
             PrintConv => {
                 0 => 'AF/AE lock',
                 1 => 'AE lock/AF',
@@ -1963,8 +2002,8 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
         },
         {
             Name => 'SetButtonWhenShooting',
-            Condition => '$$self{Model} =~ /\b(450D|XSi|Kiss X2|550D|T2i|Kiss X4)\b/',
-            Notes => '450D and 550D; value of 5 is new for 550D',
+            Condition => '$$self{Model} =~ /\b(450D|XSi|Kiss X2|550D|T2i|Kiss X4|600D|T3i|Kiss X5)\b/',
+            Notes => '450D, 550D and 600D; value of 5 is new for 550D',
             PrintConv => {
                 0 => 'Normal (disabled)',
                 1 => 'Image quality',
@@ -1974,6 +2013,19 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
                 5 => 'ISO speed',
             },
         },
+        {
+            Name => 'SetButtonWhenShooting',
+            Condition => '$$self{Model} =~ /\b(1100D|T3|Kiss X50)\b/',
+            Notes => '1100D',
+            PrintConv => {
+                0 => 'Normal (disabled)',
+                1 => 'Image quality',
+                2 => 'Flash exposure compensation',
+                3 => 'LCD monitor On/Off',
+                4 => 'Menu display',
+                5 => 'Depth-of-field preview',
+            },
+        },
         {
             Name => 'SetButtonWhenShooting',
             Condition => '$$self{Model} =~ /\b(1000D|XS|Kiss F)\b/',
@@ -2042,12 +2094,16 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     },
     0x0709 => {
         Name => 'LockMicrophoneButton',
-        PrintConv => {
+        PrintConv => [{
             # called "sound" in 1DmkIII manual, and "memo" in 1DmkIV manual
             0 => 'Protect (hold:record memo)',
             1 => 'Record memo (protect:disable)',
             2 => 'Play memo (hold:record memo)', # new with 1DmkIV
-        },
+            3 => 'Rating (protect/memo:disable)', # new with 1DX
+        }],
+        # (not sure what the 2nd number is -- new for 1DX.  Seen a value of 31.
+        # Memo quality may be set to 48kHz or 8kHz through another option that
+        # doesn't seem to be stored separately -- is this it?)
     },
     0x070a => {
         Name => 'ButtonFunctionControlOff',
@@ -2077,6 +2133,24 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
             1 => 'Quick start (FEL button)',
         },
     },
+    0x070e => { # new for 1100D
+        Name => 'FlashButtonFunction',
+        PrintConv => {
+            0 => 'Raise built-in flash',
+            1 => 'ISO speed',
+        },
+    },
+    0x070f => { # new for 5DmkIII
+        Name => 'MultiFunctionLock',
+        PrintConv => [
+            \%offOn, # (NC)
+            { BITMASK => { # (NC)
+                0 => 'Main dial',
+                1 => 'Quick control dial',
+                2 => 'Multi-controller',
+            }},
+        ],
+    },
     #### 4b) Others
     0x080b => [
         {
@@ -2099,6 +2173,25 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
                 2 => 'Eg-S',
             },
         },
+        {
+            Name => 'FocusingScreen',
+            Condition => '$$self{Model} =~ /\b6D\b/',
+            Notes => '6D',
+            PrintConv => {
+                0 => 'Eg-A II',
+                1 => 'Eg-D',
+                2 => 'Eg-S',
+            },
+        },
+        {
+            Name => 'FocusingScreen',
+            Condition => '$$self{Model} =~ /\b1D X\b/',
+            Notes => '1DX',
+            PrintConv => {
+                0 => 'Ec-CV',
+                1 => 'Ec-A,B,D,H,I,L',
+            },
+        },
         {
             Name => 'FocusingScreen',
             Notes => '1DmkIII and 1DmkIV',
@@ -2160,6 +2253,13 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
             1 => 'Retain power off status',
         },
     },
+    0x0813 => { # new for 5DmkIII
+        Name => 'DefaultEraseOption',
+        PrintConv => {
+            0 => 'Cancel selected',
+            1 => 'Erase selected',
+        },
+    },
 );
 
 #------------------------------------------------------------------------------
@@ -2185,27 +2285,27 @@ sub ConvertPfnInv($)
 # Returns: 1 on success
 sub ProcessCanonCustom2($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $offset = $$dirInfo{DirStart};
     my $size = $$dirInfo{DirLen};
     my $write = $$dirInfo{Write};
-    my $verbose = $exifTool->Options('Verbose');
+    my $verbose = $et->Options('Verbose');
     my $newTags;
 
     # first entry in array must be the size
     my $len = Get16u($dataPt, $offset);
     unless ($len == $size and $len >= 8) {
-        $exifTool->Warn("Invalid CanonCustom2 data");
+        $et->Warn("Invalid CanonCustom2 data");
         return 0;
     }
     # get group count
     my $count = Get32u($dataPt, $offset + 4);
     if ($write) {
-        $newTags = $exifTool->GetNewTagInfoHash($tagTablePtr);
-        $exifTool->VPrint(0, "  Rewriting CanonCustom2\n");
+        $newTags = $et->GetNewTagInfoHash($tagTablePtr);
+        $et->VPrint(0, "  Rewriting CanonCustom2\n");
     } elsif ($verbose) {
-        $exifTool->VerboseDir('CanonCustom2', $count);
+        $et->VerboseDir('CanonCustom2', $count);
     }
     my $pos = $offset + 8;
     my $end = $offset + $size;
@@ -2220,7 +2320,7 @@ sub ProcessCanonCustom2($$$)
         my $recPos = $pos;
         my $recEnd = $pos + $recLen;
         if ($verbose and not $write) {
-            $exifTool->VerboseDir("CanonCustom2 group $recNum", $recCount);
+            $et->VerboseDir("CanonCustom2 group $recNum", $recCount);
         }
         my ($i, $num, $tag);
         for ($i=0; $recPos + 8 < $recEnd; ++$i, $recPos+=4*$num) {
@@ -2233,18 +2333,18 @@ sub ProcessCanonCustom2($$$)
                 # write new value
                 my $tagInfo = $$newTags{$tag};
                 next unless $tagInfo;
-                my $nvHash = $exifTool->GetNewValueHash($tagInfo);
-                next unless Image::ExifTool::IsOverwriting($nvHash, $val);
-                my $newVal = Image::ExifTool::GetNewValues($nvHash);
+                my $nvHash = $et->GetNewValueHash($tagInfo);
+                next unless $et->IsOverwriting($nvHash, $val);
+                my $newVal = $et->GetNewValues($nvHash);
                 next unless defined $newVal;    # can't delete from a custom table
                 WriteValue($newVal, 'int32s', $num, $dataPt, $recPos);
-                $exifTool->VerboseValue("- CanonCustom:$$tagInfo{Name}", $val);
-                $exifTool->VerboseValue("+ CanonCustom:$$tagInfo{Name}", $newVal);
-                ++$exifTool->{CHANGED};
+                $et->VerboseValue("- CanonCustom:$$tagInfo{Name}", $val);
+                $et->VerboseValue("+ CanonCustom:$$tagInfo{Name}", $newVal);
+                ++$$et{CHANGED};
             } else {
                 # save extracted tag
                 my $oldInfo = $$tagTablePtr{$tag};
-                $exifTool->HandleTag($tagTablePtr, $tag, $val,
+                $et->HandleTag($tagTablePtr, $tag, $val,
                     Index  => $i,
                     Format => 'int32u',
                     Count  => $num,
@@ -2261,7 +2361,7 @@ sub ProcessCanonCustom2($$$)
         $pos += $recLen - 8;
     }
     if ($pos != $end) {
-        $exifTool->Warn('Possibly corrupted CanonCustom2 data');
+        $et->Warn('Possibly corrupted CanonCustom2 data');
         return 0;
     }
     return 1;
@@ -2273,8 +2373,8 @@ sub ProcessCanonCustom2($$$)
 # Returns: New custom data block or undefined on error
 sub WriteCanonCustom2($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
-    $exifTool or return 1;    # allow dummy access to autoload this package
+    my ($et, $dirInfo, $tagTablePtr) = @_;
+    $et or return 1;    # allow dummy access to autoload this package
     my $dataPt = $$dirInfo{DataPt};
     # edit a copy of the custom function 2 data
     my $buff = substr($$dataPt, $$dirInfo{DirStart}, $$dirInfo{DirLen});
@@ -2284,7 +2384,7 @@ sub WriteCanonCustom2($$$)
         DirLen   => $$dirInfo{DirLen},
         Write    => 1,
     );
-    ProcessCanonCustom2($exifTool, \%dirInfo, $tagTablePtr) or return undef;
+    ProcessCanonCustom2($et, \%dirInfo, $tagTablePtr) or return undef;
     return $buff;
 }
 
@@ -2294,26 +2394,26 @@ sub WriteCanonCustom2($$$)
 # Returns: 1 on success
 sub ProcessCanonCustom($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $offset = $$dirInfo{DirStart};
     my $size = $$dirInfo{DirLen};
-    my $verbose = $exifTool->Options('Verbose');
+    my $verbose = $et->Options('Verbose');
 
     # first entry in array must be the size
     my $len = Get16u($dataPt,$offset);
-    unless ($len == $size or ($$exifTool{Model}=~/\bD60\b/ and $len+2 == $size)) {
-        $exifTool->Warn("Invalid CanonCustom data");
+    unless ($len == $size or ($$et{Model}=~/\bD60\b/ and $len+2 == $size)) {
+        $et->Warn("Invalid CanonCustom data");
         return 0;
     }
-    $verbose and $exifTool->VerboseDir('CanonCustom', $size/2-1);
+    $verbose and $et->VerboseDir('CanonCustom', $size/2-1);
     my $pos;
     for ($pos=2; $pos<$size; $pos+=2) {
         # ($pos is position within custom directory)
         my $val = Get16u($dataPt,$offset+$pos);
         my $tag = ($val >> 8);
         $val = ($val & 0xff);
-        $exifTool->HandleTag($tagTablePtr, $tag, $val,
+        $et->HandleTag($tagTablePtr, $tag, $val,
             Index  => $pos/2-1,
             Format => 'int8u',
             Count  => 1,
@@ -2329,7 +2429,7 @@ sub ProcessCanonCustom($$$)
 # Returns: error string or undef (and may modify value) on success
 sub CheckCanonCustom($$$)
 {
-    my ($exifTool, $tagInfo, $valPtr) = @_;
+    my ($et, $tagInfo, $valPtr) = @_;
     return Image::ExifTool::CheckValue($valPtr, 'int8u');
 }
 
@@ -2339,8 +2439,8 @@ sub CheckCanonCustom($$$)
 # Returns: New custom data block or undefined on error
 sub WriteCanonCustom($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
-    $exifTool or return 1;    # allow dummy access to autoload this package
+    my ($et, $dirInfo, $tagTablePtr) = @_;
+    $et or return 1;    # allow dummy access to autoload this package
     my $dataPt = $$dirInfo{DataPt};
     my $dirStart = $$dirInfo{DirStart} || 0;
     my $dirLen = $$dirInfo{DirLen} || length($$dataPt) - $dirStart;
@@ -2350,26 +2450,26 @@ sub WriteCanonCustom($$$)
 
     # first entry in array must be the size
     my $len = Get16u($dataPt, 0);
-    unless ($len == $dirLen or ($$exifTool{Model}=~/\bD60\b/ and $len+2 == $dirLen)) {
-        $exifTool->Warn("Invalid CanonCustom data");
+    unless ($len == $dirLen or ($$et{Model}=~/\bD60\b/ and $len+2 == $dirLen)) {
+        $et->Warn("Invalid CanonCustom data");
         return undef;
     }
-    my $newTags = $exifTool->GetNewTagInfoHash($tagTablePtr);
+    my $newTags = $et->GetNewTagInfoHash($tagTablePtr);
     my $pos;
     for ($pos=2; $pos<$dirLen; $pos+=2) {
         my $val = Get16u($dataPt, $pos);
         my $tag = ($val >> 8);
         my $tagInfo = $$newTags{$tag};
         next unless $tagInfo;
-        my $nvHash = $exifTool->GetNewValueHash($tagInfo);
+        my $nvHash = $et->GetNewValueHash($tagInfo);
         $val = ($val & 0xff);
-        next unless Image::ExifTool::IsOverwriting($nvHash, $val);
-        my $newVal = Image::ExifTool::GetNewValues($nvHash);
+        next unless $et->IsOverwriting($nvHash, $val);
+        my $newVal = $et->GetNewValues($nvHash);
         next unless defined $newVal;    # can't delete from a custom table
         Set16u(($newVal & 0xff) + ($tag << 8), $dataPt, $pos);
-        $exifTool->VerboseValue("- $dirName:$$tagInfo{Name}", $val);
-        $exifTool->VerboseValue("+ $dirName:$$tagInfo{Name}", $newVal);
-        ++$exifTool->{CHANGED};
+        $et->VerboseValue("- $dirName:$$tagInfo{Name}", $val);
+        $et->VerboseValue("+ $dirName:$$tagInfo{Name}", $newVal);
+        ++$$et{CHANGED};
     }
     return $newData;
 }
@@ -2396,7 +2496,7 @@ Image::ExifTool to read this information.
 
 =head1 AUTHOR
 
-Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
