@@ -6,22 +6,22 @@
 # Revisions:    11/25/2003 - P. Harvey Created
 #               12/02/2003 - P. Harvey Completely reworked and figured out many
 #                            more tags
-#               01/19/2004 - P. Harvey Added CleanRaw()
 #
 # References:   1) http://www.cybercom.net/~dcoffin/dcraw/
 #               2) http://www.wonderland.org/crw/
 #               3) http://xyrion.org/ciff/CIFFspecV1R04.pdf
+#               4) Dave Nicholson private communication (PowerShot S30)
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::CanonRaw;
 
 use strict;
 use vars qw($VERSION $AUTOLOAD %crwTagFormat);
-use Image::ExifTool qw(:DataAccess);
+use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 use Image::ExifTool::Canon;
 
-$VERSION = '1.39';
+$VERSION = '1.56';
 
 sub WriteCRW($$);
 sub ProcessCanonRaw($$$);
@@ -52,7 +52,7 @@ sub BuildMakerNotes($$$$$$);
     WRITE_PROC => \&WriteCanonRaw,
     CHECK_PROC => \&CheckCanonRaw,
     WRITABLE => 1,
-    0x0000 => 'NullRecord', #3
+    0x0000 => { Name => 'NullRecord', Writable => 'undef' }, #3
     0x0001 => { #3
         Name => 'FreeBytes',
         Format => 'undef',
@@ -64,9 +64,11 @@ sub BuildMakerNotes($$$$$$);
         {
             Condition => '$self->{DIR_NAME} eq "ImageDescription"',
             Name => 'CanonFileDescription',
+            Writable => 'string[32]',
         },
         {
             Name => 'UserComment',
+            Writable => 'string[256]',
         },
     ],
     0x080a => {
@@ -76,18 +78,16 @@ sub BuildMakerNotes($$$$$$);
             TagTable => 'Image::ExifTool::CanonRaw::MakeModel',
         },
     },
-    0x080b => 'CanonFirmwareVersion',
-    0x080c => 'ComponentVersion', #3
-    0x080d => 'ROMOperationMode', #3
-    0x0810 => {
-        Name => 'OwnerName',
-        Description => "Owner's Name",
-    },
-    0x0815 => 'CanonImageType',
-    0x0816 => 'OriginalFileName',
-    0x0817 => 'ThumbnailFileName',
+    0x080b => { Name => 'CanonFirmwareVersion', Writable => 'string[32]' },
+    0x080c => { Name => 'ComponentVersion',     Writable => 'string'     }, #3
+    0x080d => { Name => 'ROMOperationMode',     Writable => 'string[8]'  }, #3
+    0x0810 => { Name => 'OwnerName',            Writable => 'string[32]' },
+    0x0815 => { Name => 'CanonImageType',       Writable => 'string[32]' },
+    0x0816 => { Name => 'OriginalFileName',     Writable => 'string[32]' },
+    0x0817 => { Name => 'ThumbnailFileName',    Writable => 'string[32]' },
     0x100a => { #3
         Name => 'TargetImageType',
+        Writable => 'int16u',
         PrintConv => {
             0 => 'Real-world Subject',
             1 => 'Written Document',
@@ -95,6 +95,7 @@ sub BuildMakerNotes($$$$$$);
     },
     0x1010 => { #3
         Name => 'ShutterReleaseMethod',
+        Writable => 'int16u',
         PrintConv => {
             0 => 'Single Shot',
             2 => 'Continuous Shooting',
@@ -102,13 +103,14 @@ sub BuildMakerNotes($$$$$$);
     },
     0x1011 => { #3
         Name => 'ShutterReleaseTiming',
+        Writable => 'int16u',
         PrintConv => {
             0 => 'Priority on shutter',
             1 => 'Priority on focus',
         },
     },
-    0x1016 => 'ReleaseSetting', #3
-    0x101c => 'BaseISO', #3
+    0x1016 => { Name => 'ReleaseSetting',       Writable => 'int16u' }, #3
+    0x101c => { Name => 'BaseISO',              Writable => 'int16u' }, #3
     0x1028=> { #PH
         Name => 'CanonFlashInfo',
         Writable => 'int16u',
@@ -129,7 +131,14 @@ sub BuildMakerNotes($$$$$$);
             TagTable => 'Image::ExifTool::Canon::ShotInfo',
         },
     },
-    0x102c => { Name => 'CanonColorInfo2', Writable => 0 },
+    0x102c => {
+        Name => 'CanonColorInfo2',
+        Writable => 0,
+        # for the S30, the following information has been decoded: (ref 4)
+        # offset 66: int32u - shutter half press time in ms
+        # offset 70: int32u - image capture time in ms
+        # offset 74: int16u - custom white balance flag (0=Off, 512=On)
+    },
     0x102d => {
         Name => 'CanonCameraSettings',
         Writable => 0,
@@ -137,6 +146,14 @@ sub BuildMakerNotes($$$$$$);
             TagTable => 'Image::ExifTool::Canon::CameraSettings',
         },
     },
+    0x1030 => { #4
+        Name => 'WhiteSample',
+        Writable => 0,
+        SubDirectory => {
+            Validate => 'Image::ExifTool::Canon::Validate($dirData,$subdirStart,$size)',
+            TagTable => 'Image::ExifTool::CanonRaw::WhiteSample',
+        },
+    },
     0x1031 => {
         Name => 'SensorInfo',
         Writable => 0,
@@ -149,7 +166,7 @@ sub BuildMakerNotes($$$$$$);
     0x1033 => [
         {
             Name => 'CustomFunctions10D',
-            Condition => '$self->{CameraModel} =~ /EOS 10D/',
+            Condition => '$self->{Model} =~ /EOS 10D/',
             SubDirectory => {
                 Validate => 'Image::ExifTool::Canon::Validate($dirData,$subdirStart,$size)',
                 TagTable => 'Image::ExifTool::CanonCustom::Functions10D',
@@ -157,7 +174,7 @@ sub BuildMakerNotes($$$$$$);
         },
         {
             Name => 'CustomFunctionsD30',
-            Condition => '$self->{CameraModel} =~ /EOS D30\b/',
+            Condition => '$self->{Model} =~ /EOS D30\b/',
             SubDirectory => {
                 Validate => 'Image::ExifTool::Canon::Validate($dirData,$subdirStart,$size)',
                 TagTable => 'Image::ExifTool::CanonCustom::FunctionsD30',
@@ -165,7 +182,7 @@ sub BuildMakerNotes($$$$$$);
         },
         {
             Name => 'CustomFunctionsD60',
-            Condition => '$self->{CameraModel} =~ /EOS D60\b/',
+            Condition => '$self->{Model} =~ /EOS D60\b/',
             SubDirectory => {
                 # the stored size in the D60 apparently doesn't include the size word:
                 Validate => 'Image::ExifTool::Canon::Validate($dirData,$subdirStart,$size-2,$size)',
@@ -182,10 +199,10 @@ sub BuildMakerNotes($$$$$$);
         },
     ],
     0x1038 => {
-        Name => 'CanonPictureInfo',
+        Name => 'CanonAFInfo',
         Writable => 0,
         SubDirectory => {
-            TagTable => 'Image::ExifTool::Canon::PictureInfo',
+            TagTable => 'Image::ExifTool::Canon::AFInfo',
         },
     },
     0x1093 => {
@@ -212,9 +229,13 @@ sub BuildMakerNotes($$$$$$);
             TagTable => 'Image::ExifTool::CanonRaw::RawJpgInfo',
         },
     },
-    0x10ae => 'ColorTemperature',
+    0x10ae => {
+        Name => 'ColorTemperature',
+        Writable => 'int16u',
+    },
     0x10b4 => {
         Name => 'ColorSpace',
+        Writable => 'int16u',
         PrintConv => {
             1 => 'sRGB',
             2 => 'Adobe RGB',
@@ -228,26 +249,44 @@ sub BuildMakerNotes($$$$$$);
             TagTable => 'Image::ExifTool::CanonRaw::ImageFormat',
         },
     },
-    0x1804 => 'RecordID', #3
+    0x1804 => { Name => 'RecordID', Writable => 'int32u' }, #3
     0x1806 => { #3
         Name => 'SelfTimerTime',
+        Writable => 'int32u',
         ValueConv => '$val / 1000',
         ValueConvInv => '$val * 1000',
-        PrintConv => '"$val sec"',
-        PrintConvInv => '$val=~s/\s*sec.*//;$val',
+        PrintConv => '"$val s"',
+        PrintConvInv => '$val=~s/\s*s.*//;$val',
     },
     0x1807 => {
         Name => 'TargetDistanceSetting',
         Format => 'float',
         PrintConv => '"$val mm"',
-        PrintConvInv => '$val=~s/\s*mm//;$val',
-    },
-    0x180b => {
-        Name => 'SerialNumber',
-        Description => 'Camera Body No.',
-        PrintConv => 'sprintf("%.10d",$val)',
-        PrintConvInv => '$val',
+        PrintConvInv => '$val=~s/\s*mm$//;$val',
     },
+    0x180b => [
+        {
+            # D30
+            Name => 'SerialNumber',
+            Condition => '$$self{Model} =~ /EOS D30\b/',
+            Writable => 'int32u',
+            PrintConv => 'sprintf("%x-%.5d",$val>>16,$val&0xffff)',
+            PrintConvInv => '$val=~/(.*)-(\d+)/ ? (hex($1)<<16)+$2 : undef',
+        },
+        {
+            # all EOS models (D30, 10D, 300D)
+            Name => 'SerialNumber',
+            Condition => '$$self{Model} =~ /EOS/',
+            Writable => 'int32u',
+            PrintConv => 'sprintf("%.10d",$val)',
+            PrintConvInv => '$val',
+        },
+        {
+            # this is not SerialNumber for PowerShot models (but what is it?) - PH
+            Name => 'UnknownNumber',
+            Unknown => 1,
+        },
+    ],
     0x180e => {
         Name => 'TimeStamp',
         Writable => 0,
@@ -272,19 +311,24 @@ sub BuildMakerNotes($$$$$$);
     0x1814 => { #3
         Name => 'MeasuredEV',
         Notes => q{
-            this the Canon name for what should properly be called MeasuredLV, and is
-            offset by about -5 EV from the calculated LV for most models
+            this is the Canon name for what could better be called MeasuredLV, and
+            should be close to the calculated LightValue for a proper exposure with most
+            models
         },
         Format => 'float',
+        ValueConv => '$val + 5',
+        ValueConvInv => '$val - 5',
     },
     0x1817 => {
         Name => 'FileNumber',
+        Writable => 'int32u',
         Groups => { 2 => 'Image' },
         PrintConv => '$_=$val;s/(\d+)(\d{4})/$1-$2/;$_',
         PrintConvInv => '$_=$val;s/-//;$_',
     },
     0x1818 => { #3
         Name => 'ExposureInfo',
+        Groups => { 1 => 'CIFF' }, # (only so CIFF shows up in group lists)
         Writable => 0,
         SubDirectory => {
             TagTable => 'Image::ExifTool::CanonRaw::ExposureInfo',
@@ -292,17 +336,21 @@ sub BuildMakerNotes($$$$$$);
     },
     0x1834 => { #PH
         Name => 'CanonModelID',
-        Writable => 0,
+        Writable => 'int32u',
         PrintHex => 1,
         Notes => q{
             this is the complete list of model ID numbers, but note that many of these
             models do not produce CRW images
         },
+        SeparateTable => 'Canon CanonModelID',
         PrintConv => \%Image::ExifTool::Canon::canonModelID,
     },
     0x1835 => {
         Name => 'DecoderTable',
         Writable => 0,
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::CanonRaw::DecoderTable',
+        },
     },
     0x183b => { #PH
         # display format for serial number
@@ -323,16 +371,14 @@ sub BuildMakerNotes($$$$$$);
         Name => 'JpgFromRaw',
         Writable => 'resize',  # 'resize' allows this value to change size
         Permanent => 0,
-        ValueConv => '$self->ValidateImage(\$val,$tag)',
-        ValueConvInv => '$val',
+        RawConv => '$self->ValidateImage(\$val,$tag)',
     },
     0x2008 => {
         Name => 'ThumbnailImage',
         Writable => 'resize',  # 'resize' allows this value to change size
         WriteCheck => '$self->CheckImage(\$val)',
         Permanent => 0,
-        ValueConv => '$self->ValidateImage(\$val,$tag)',
-        ValueConvInv => '$val',
+        RawConv => '$self->ValidateImage(\$val,$tag)',
     },
     # the following entries are subdirectories
     # (any 0x28 and 0x30 tag types are handled automatically by the decoding logic)
@@ -387,13 +433,15 @@ sub BuildMakerNotes($$$$$$);
     0 => {
         Name => 'Make',
         Format => 'string[6]',  # "Canon\0"
-        RawConv => '$self->{CameraMake} = $val',
+        DataMember => 'Make',
+        RawConv => '$self->{Make} = $val',
     },
     6 => {
         Name => 'Model',
-        Format => 'string[$size-6]',
+        Format => 'string', # no size = to the end of the data
         Description => 'Camera Model Name',
-        RawConv => '$self->{CameraModel} = $val',
+        DataMember => 'Model',
+        RawConv => '$self->{Model} = $val',
     },
 );
 
@@ -412,7 +460,7 @@ sub BuildMakerNotes($$$$$$);
         ValueConv => 'ConvertUnixTime($val)',
         ValueConvInv => 'GetUnixTime($val)',
         PrintConv => '$self->ConvertDateTime($val)',
-        PrintConvInv => '$val',
+        PrintConvInv => '$self->InverseDateTime($val)',
     },
     1 => { #3
         Name => 'TimeZoneCode',
@@ -501,8 +549,20 @@ sub BuildMakerNotes($$$$$$);
     FIRST_ENTRY => 0,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
     0 => 'ExposureCompensation',
-    1 => 'TvValue',
-    2 => 'AvValue',
+    1 => {
+        Name => 'ShutterSpeedValue',
+        ValueConv => 'abs($val)<100 ? 1/(2**$val) : 0',
+        ValueConvInv => '$val>0 ? -log($val)/log(2) : -100',
+        PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
+        PrintConvInv => 'Image::ExifTool::Exif::ConvertFraction($val)',
+    },
+    2 => {
+        Name => 'ApertureValue',
+        ValueConv => '2 ** ($val / 2)',
+        ValueConvInv => '$val>0 ? 2*log($val)/log(2) : 0',
+        PrintConv => 'sprintf("%.1f",$val)',
+        PrintConvInv => '$val',
+    },
 );
 
 %Image::ExifTool::CanonRaw::ImageInfo = (
@@ -510,7 +570,7 @@ sub BuildMakerNotes($$$$$$);
     WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
     CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
     FORMAT => 'int32u',
-    FIRST_ENTRY => 1,
+    FIRST_ENTRY => 0,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
     # Note: Don't make these writable (except rotation) because it confuses
     # Canon decoding software if the are changed
@@ -530,6 +590,35 @@ sub BuildMakerNotes($$$$$$);
     6 => 'ColorBW', #3
 );
 
+# ref 4
+%Image::ExifTool::CanonRaw::DecoderTable = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    FORMAT => 'int32u',
+    FIRST_ENTRY => 0,
+    0 => 'DecoderTableNumber',
+    2 => 'CompressedDataOffset',
+    3 => 'CompressedDataLength',
+);
+
+# ref 1/4
+%Image::ExifTool::CanonRaw::WhiteSample = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    FORMAT => 'int16u',
+    FIRST_ENTRY => 1,
+    1 => 'WhiteSampleWidth',
+    2 => 'WhiteSampleHeight',
+    3 => 'WhiteSampleLeftBorder',
+    4 => 'WhiteSampleTopBorder',
+    5 => 'WhiteSampleBits',
+    # this is followed by the encrypted white sample values (ref 1)
+);
+
 #------------------------------------------------------------------------------
 # AutoLoad our writer routines when necessary
 #
@@ -545,13 +634,13 @@ sub AUTOLOAD
 # Returns: 1 on success
 sub ProcessCanonRaw($$$)
 {
-    my ($exifTool, $dirInfo, $rawTagTable) = @_;
+    my ($et, $dirInfo, $rawTagTable) = @_;
     my $blockStart = $$dirInfo{DirStart};
     my $blockSize = $$dirInfo{DirLen};
     my $raf = $$dirInfo{RAF} or return 0;
     my $buff;
-    my $verbose = $exifTool->Options('Verbose');
-    my $buildMakerNotes = $exifTool->Options('MakerNotes');
+    my $verbose = $et->Options('Verbose');
+    my $buildMakerNotes = $et->Options('MakerNotes');
 
     # 4 bytes at end of block give directory position within block
     $raf->Seek($blockStart+$blockSize-4, 0) or return 0;
@@ -563,7 +652,7 @@ sub ProcessCanonRaw($$$)
     # read the directory (10 bytes per entry)
     $raf->Read($buff, 10 * $entries) == 10 * $entries or return 0;
 
-    $verbose and $exifTool->VerboseDir('Raw', $entries);
+    $verbose and $et->VerboseDir('CIFF', $entries);
     my $index;
     for ($index=0; $index<$entries; ++$index) {
         my $pt = 10 * $index;
@@ -572,13 +661,13 @@ sub ProcessCanonRaw($$$)
         my $valuePtr = Get32u(\$buff, $pt+6);
         my $ptr = $valuePtr + $blockStart;  # all pointers relative to block start
         if ($tag & 0x8000) {
-            $exifTool->Warn('Bad CRW directory entry');
+            $et->Warn('Bad CRW directory entry');
             return 1;
         }
         my $tagID = $tag & 0x3fff;          # get tag ID
         my $tagType = ($tag >> 8) & 0x38;   # get tag type
         my $valueInDir = ($tag & 0x4000);   # flag for value in directory
-        my $tagInfo = $exifTool->GetTagInfo($rawTagTable, $tagID);
+        my $tagInfo = $et->GetTagInfo($rawTagTable, $tagID);
         if (($tagType==0x28 or $tagType==0x30) and not $valueInDir) {
             # this type of tag specifies a raw subdirectory
             my $name;
@@ -595,13 +684,13 @@ sub ProcessCanonRaw($$$)
             );
             if ($verbose) {
                 my $fakeInfo = { Name => $name, SubDirectory => { } };
-                $exifTool->VerboseInfo($tagID, $fakeInfo,
+                $et->VerboseInfo($tagID, $fakeInfo,
                     'Index'  => $index,
                     'Size'   => $size,
                     'Start'  => $ptr,
                 );
             }
-            $exifTool->ProcessDirectory(\%subdirInfo, $rawTagTable);
+            $et->ProcessDirectory(\%subdirInfo, $rawTagTable);
             next;
         }
         my ($valueDataPos, $count, $subdir);
@@ -625,21 +714,21 @@ sub ProcessCanonRaw($$$)
             $valueDataPos = $ptr;
             if ($size <= 512 or ($verbose > 2 and $size <= 65536)
                 or ($tagInfo and ($$tagInfo{SubDirectory}
-                or grep(/^$$tagInfo{Name}$/i, $exifTool->GetRequestedTags()) )))
+                or grep(/^$$tagInfo{Name}$/i, $et->GetRequestedTags()) )))
             {
                 # read value if size is small or specifically requested
                 # or if this is a SubDirectory
                 unless ($raf->Seek($ptr, 0) and $raf->Read($value, $size) == $size) {
-                    $exifTool->Warn(sprintf("Error reading %d bytes from 0x%x",$size,$ptr));
+                    $et->Warn(sprintf("Error reading %d bytes from 0x%x",$size,$ptr));
                     next;
                 }
             } else {
                 $value = "Binary data $size bytes";
                 if ($tagInfo) {
-                    if ($exifTool->Options('Binary')) {
+                    if ($et->Options('Binary') or $verbose) {
                         # read the value anyway
                         unless ($raf->Seek($ptr, 0) and $raf->Read($value, $size) == $size) {
-                            $exifTool->Warn(sprintf("Error reading %d bytes from 0x%x",$size,$ptr));
+                            $et->Warn(sprintf("Error reading %d bytes from 0x%x",$size,$ptr));
                             next;
                         }
                     }
@@ -661,7 +750,7 @@ sub ProcessCanonRaw($$$)
         if ($verbose) {
             my $val = $value;
             $format and $val = ReadValue(\$val, 0, $format, $count, $size);
-            $exifTool->VerboseInfo($tagID, $tagInfo,
+            $et->VerboseInfo($tagID, $tagInfo,
                 Table   => $rawTagTable,
                 Index   => $index,
                 Value   => $val,
@@ -674,7 +763,7 @@ sub ProcessCanonRaw($$$)
         }
         if ($buildMakerNotes) {
             # build maker notes information if requested
-            BuildMakerNotes($exifTool, $tagID, $tagInfo, \$value, $format, $count);
+            BuildMakerNotes($et, $tagID, $tagInfo, \$value, $format, $count);
         }
         next unless defined $tagInfo;
 
@@ -682,7 +771,7 @@ sub ProcessCanonRaw($$$)
             my $name = $$tagInfo{Name};
             my $newTagTable;
             if ($$subdir{TagTable}) {
-                $newTagTable = Image::ExifTool::GetTagTable($$subdir{TagTable});
+                $newTagTable = GetTagTable($$subdir{TagTable});
                 unless ($newTagTable) {
                     warn "Unknown tag table $$subdir{TagTable}\n";
                     next;
@@ -704,19 +793,20 @@ sub ProcessCanonRaw($$$)
                 DirLen   => $size - $subdirStart,
                 Nesting  => $$dirInfo{Nesting} + 1,
                 RAF      => $raf,
+                DirName  => $name,
                 Parent   => $$dirInfo{DirName},
             );
             #### eval Validate ($dirData, $subdirStart, $size)
             if (defined $$subdir{Validate} and not eval $$subdir{Validate}) {
-                $exifTool->Warn("Invalid $name data");
+                $et->Warn("Invalid $name data");
             } else {
-                $exifTool->ProcessDirectory(\%subdirInfo, $newTagTable, $$subdir{ProcessProc});
+                $et->ProcessDirectory(\%subdirInfo, $newTagTable, $$subdir{ProcessProc});
             }
         } else {
             # convert to specified format if necessary
             $format and $value = ReadValue(\$value, 0, $format, $count, $size);
             # save the information
-            $exifTool->FoundTag($tagInfo, $value);
+            $et->FoundTag($tagInfo, $value);
             delete $$tagInfo{RawConv} if $delRawConv;
         }
     }
@@ -729,10 +819,10 @@ sub ProcessCanonRaw($$$)
 # Returns: 1 if this was a valid Canon RAW file
 sub ProcessCRW($$)
 {
-    my ($exifTool, $dirInfo) = @_;
+    my ($et, $dirInfo) = @_;
     my ($buff, $sig);
     my $raf = $$dirInfo{RAF};
-    my $buildMakerNotes = $exifTool->Options('MakerNotes');
+    my $buildMakerNotes = $et->Options('MakerNotes');
 
     $raf->Read($buff,2) == 2      or return 0;
     SetByteOrder($buff)           or return 0;
@@ -745,10 +835,10 @@ sub ProcessCRW($$)
     my $filesize = $raf->Tell()   or return 0;
 
     # initialize maker note data if building maker notes
-    $buildMakerNotes and InitMakerNotes($exifTool);
+    $buildMakerNotes and InitMakerNotes($et);
 
     # set the FileType tag unless already done (ie. APP0 CIFF record in JPEG image)
-    $exifTool->SetFileType() unless $exifTool->GetValue('FileType');
+    $et->SetFileType();
 
     # build directory information for main raw directory
     my %dirInfo = (
@@ -761,17 +851,22 @@ sub ProcessCRW($$)
     );
 
     # process the raw directory
-    my $rawTagTable = Image::ExifTool::GetTagTable('Image::ExifTool::CanonRaw::Main');
-    unless ($exifTool->ProcessDirectory(\%dirInfo, $rawTagTable)) {
-        $exifTool->Warn('CRW file format error');
+    my $rawTagTable = GetTagTable('Image::ExifTool::CanonRaw::Main');
+    my $oldIndent = $$et{INDENT};
+    $$et{INDENT} .= '| ';
+    unless (ProcessCanonRaw($et, \%dirInfo, $rawTagTable)) {
+        $et->Warn('CRW file format error');
     }
+    $$et{INDENT} = $oldIndent;
 
     # finish building maker notes if necessary
-    $buildMakerNotes and SaveMakerNotes($exifTool);
+    $buildMakerNotes and SaveMakerNotes($et);
 
-    # process trailers if they exist
-    my $trailInfo = Image::ExifTool::IdentifyTrailer($raf);
-    $exifTool->ProcessTrailers($trailInfo) if $trailInfo;
+    # process trailers if they exist in CRW file (not in CIFF information!)
+    if ($$et{FILE_TYPE} eq 'CRW') {
+        my $trailInfo = Image::ExifTool::IdentifyTrailer($raf);
+        $et->ProcessTrailers($trailInfo) if $trailInfo;
+    }
 
     return 1;
 }
@@ -803,7 +898,7 @@ tags.)
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -822,6 +917,10 @@ under the same terms as Perl itself.
 
 =back
 
+=head1 ACKNOWLEDGEMENTS
+
+Thanks to Dave Nicholson for decoding a number of new tags.
+
 =head1 SEE ALSO
 
 L<Image::ExifTool::TagNames/CanonRaw Tags>,
