@@ -20,7 +20,7 @@ use Image::ExifTool::PostScript;
 use Image::ExifTool::XMP qw(EscapeXML UnescapeXML);
 require Exporter;
 
-$VERSION = '1.11';
+$VERSION = '1.14';
 @ISA = qw(Exporter);
 @EXPORT_OK = qw(EscapeHTML UnescapeHTML);
 
@@ -160,6 +160,7 @@ my %entityName; # look up entity names by number (built as necessary)
     abstract        => { },
     author          => { },
     classification  => { },
+   'content-language'=>{ Name => 'ContentLanguage' },
     copyright       => { },
     description     => { },
     distribution    => { },
@@ -168,6 +169,7 @@ my %entityName; # look up entity names by number (built as necessary)
    'doc-type'       => { Name => 'DocType' },
     formatter       => { },
     generator       => { },
+    generatorversion=> { Name => 'GeneratorVersion' },
     googlebot       => { Name => 'GoogleBot' },
     keywords        => { List => 1 },
     mssmarttagspreventparsing => { Name => 'NoMSSmartTags' },
@@ -334,8 +336,8 @@ my %entityName; # look up entity names by number (built as necessary)
 # Returns: original string
 sub SetHTMLCharset($$)
 {
-    my ($val, $exifTool) = @_;
-    $$exifTool{HTMLCharset} = $htmlCharset{lc $1} if $val =~ /charset=['"]?([-\w]+)/;
+    my ($val, $et) = @_;
+    $$et{HTMLCharset} = $htmlCharset{lc $1} if $val =~ /charset=['"]?([-\w]+)/;
     return $val;
 }
 
@@ -401,20 +403,20 @@ sub UnescapeHTML($)
 # Returns: 1 on success, 0 if this wasn't a valid HTML file
 sub ProcessHTML($$)
 {
-    my ($exifTool, $dirInfo) = @_;
+    my ($et, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
     my $buff;
 
     # validate HTML or XHTML file
     $raf->Read($buff, 256) or return 0;
-    $buff =~ /^<(!DOCTYPE\s+HTML|HTML|\?xml)/i or return 0;
+    $buff =~ /^\s*<(!DOCTYPE\s+HTML|HTML|\?xml)/i or return 0;
     $buff =~ /<(!DOCTYPE\s+)?HTML/i or return 0 if $1 eq '?xml';
-    $exifTool->SetFileType();
+    $et->SetFileType();
 
-    $raf->Seek(0,0) or $exifTool->Warn('Seek error'), return 1;
+    $raf->Seek(0,0) or $et->Warn('Seek error'), return 1;
 
     local $/ = Image::ExifTool::PostScript::GetInputRecordSeparator($raf);
-    $/ or $exifTool->Warn('Invalid HTML data'), return 1;
+    $/ or $et->Warn('Invalid HTML data'), return 1;
 
     # extract header information
     my $doc;
@@ -437,7 +439,9 @@ sub ProcessHTML($$)
         my ($tagName, $attrs) = ($1, $2);
         my $tag = lc($tagName);
         my ($val, $grp);
-        unless ($attrs =~ m{/$}) {  # self-contained XHTML tags end in '/>'
+        if ($attrs =~ m{/$}) {  # self-contained XHTML tags end in '/>'
+            $val = '';
+        } else {
             # look for element close
             my $pos = pos($doc);
             my $close = "</$tagName>";
@@ -449,6 +453,7 @@ sub ProcessHTML($$)
             } else {
                 pos($doc) = $pos;
                 next unless $tag eq 'meta'; # META tags don't need to be closed
+                $val = '';
             }
         }
         my $table = $tagTablePtr;
@@ -456,21 +461,26 @@ sub ProcessHTML($$)
             # parse HTML META element
             undef $tag;
             # tag name is in NAME or HTTP-EQUIV attribute
-            if ($attrs =~ /name=['"]?([\w:.-]+)/si) {
+            if ($attrs =~ /\bname\s*=\s*['"]?([\w:.-]+)/si) {
                 $tagName = $1;
-            } elsif ($attrs =~ /http-equiv=['"]?([\w:.-]+)/si) {
+            } elsif ($attrs =~ /\bhttp-equiv\s*=\s*['"]?([\w:.-]+)/si) {
                 $tagName = "HTTP-equiv.$1";
             } else {
                 next;   # no name
             }
-            $tag = lc($tagName);
+            $tag = lc($tagName) or next;
             # tag value is in CONTENT attribute
-            $val = $2 if $attrs =~ /content=(['"])(.*?)\1/si;
-            next unless $tag and defined $val;
+            if ($attrs =~ /\bcontent\s*=\s*(['"])(.*?)\1/si or
+                $attrs =~ /\bcontent\s*=\s*(['"]?)([\w:.-]+)/si)
+            {
+                $val = $2;
+            } else {
+                next unless length $val;
+            }
             # isolate group name (separator is '.' in HTML, but ':' in ref 2)
             if ($tag =~ /^([\w-]+)[:.]([\w-]+)/) {
                 ($grp, $tag) = ($1, $2);
-                my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $grp);
+                my $tagInfo = $et->GetTagInfo($tagTablePtr, $grp);
                 if ($tagInfo and $$tagInfo{SubDirectory}) {
                     $table = GetTagTable($tagInfo->{SubDirectory}->{TagTable});
                 } else {
@@ -478,12 +488,12 @@ sub ProcessHTML($$)
                 }
             }
         } elsif ($tag eq 'xml') {
-            $exifTool->VPrint(0, "Parsing XML\n");
+            $et->VPrint(0, "Parsing XML\n");
             # parse XML tags (quick-and-dirty)
             my $xml = $val;
             while ($xml =~ /<([\w-]+):([\w-]+)(\s.*?)?>([^<]*?)<\/\1:\2>/g) {
                 ($grp, $tag, $val) = ($1, $2, $4);
-                my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $grp);
+                my $tagInfo = $et->GetTagInfo($tagTablePtr, $grp);
                 next unless $tagInfo and $$tagInfo{SubDirectory};
                 $table = GetTagTable($tagInfo->{SubDirectory}->{TagTable});
                 unless ($$table{$tag}) {
@@ -491,11 +501,11 @@ sub ProcessHTML($$)
                     $name =~ s/_x([0-9a-f]{4})_/chr(hex($1))/gie; # convert hex codes
                     $name =~ s/\s(.)/\U$1/g;     # capitalize all words in tag name
                     $name =~ tr/-_a-zA-Z0-9//dc; # remove illegal characters (also hex code wide chars)
-                    Image::ExifTool::AddTagToTable($table, $tag, { Name => $name });
-                    $exifTool->VPrint(0, "  [adding $tag '$name']\n");
+                    AddTagToTable($table, $tag, { Name => $name });
+                    $et->VPrint(0, "  [adding $tag '$name']\n");
                 }
-                $val = $exifTool->Decode($val, $$exifTool{HTMLCharset}) if $$exifTool{HTMLCharset};
-                $exifTool->HandleTag($table, $tag, UnescapeXML($val));
+                $val = $et->Decode($val, $$et{HTMLCharset}) if $$et{HTMLCharset};
+                $et->HandleTag($table, $tag, UnescapeXML($val));
             }
             next;
         } else {
@@ -507,14 +517,14 @@ sub ProcessHTML($$)
             $name =~ s/\W+(\w)/\u$1/sg;
             my $info = { Name => $name, Groups => { 0 => 'HTML' } };
             $info->{Groups}->{1} = ($grp eq 'http-equiv' ? 'HTTP-equiv' : "HTML-$grp") if $grp;
-            Image::ExifTool::AddTagToTable($table, $tag, $info);
-            $exifTool->VPrint(0, "  [adding $tag '$tagName']\n");
+            AddTagToTable($table, $tag, $info);
+            $et->VPrint(0, "  [adding $tag '$tagName']\n");
         }
         # recode if necessary
-        $val = $exifTool->Decode($val, $$exifTool{HTMLCharset}) if $$exifTool{HTMLCharset};
+        $val = $et->Decode($val, $$et{HTMLCharset}) if $$et{HTMLCharset};
         $val =~ s{\s*$/\s*}{ }sg;   # replace linefeeds and indenting spaces
         $val = UnescapeHTML($val);  # unescape HTML character references
-        $exifTool->HandleTag($table, $tag, $val);
+        $et->HandleTag($table, $tag, $val);
     }
     return 1;
 }
@@ -538,7 +548,7 @@ meta information from HTML documents.
 
 =head1 AUTHOR
 
-Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
