@@ -3,11 +3,13 @@
 #
 # Description:  Read ICC Profile meta information
 #
-# Revisions:    11/16/04 - P. Harvey Created
+# Revisions:    11/16/2004 - P. Harvey Created
 #
 # References:   1) http://www.color.org/icc_specs2.html (ICC.1:2003-09)
 #               2) http://www.color.org/icc_specs2.html (ICC.1:2001-04)
 #               3) http://developer.apple.com/documentation/GraphicsImaging/Reference/ColorSync_Manager/ColorSync_Manager.pdf
+#               4) http://www.color.org/privatetag2007-01.pdf
+#               5) http://www.color.org/icc_specs2.xalter (approved revisions, 2010-07-16)
 #
 # Notes:        The ICC profile information is different: the format of each
 #               tag is embedded in the information instead of in the directory
@@ -21,11 +23,13 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.08';
+$VERSION = '1.27';
 
 sub ProcessICC($$);
 sub ProcessICC_Profile($$$);
 sub WriteICC_Profile($$;$);
+sub ProcessMetadata($$$);
+sub ValidateICC($);
 
 # illuminant type definitions
 my %illuminantType = (
@@ -46,6 +50,7 @@ my %profileClass = (
     spac => 'ColorSpace Conversion Profile',
     abst => 'Abstract Profile',
     nmcl => 'NamedColor Profile',
+    nkpf => 'Nikon Input Device Profile (NON-STANDARD!)', # (written by Nikon utilities)
 );
 
 # ICC_Profile tag table
@@ -55,10 +60,12 @@ my %profileClass = (
     WRITE_PROC => \&WriteICC_Profile,
     NOTES => q{
         ICC profile information is used in many different file types including JPEG,
-        TIFF, PDF, PostScript, Photoshop, PNG, MIFF, PICT, QuickTime and some RAW
-        formats.  While the tags listed below are not individually writable, the
+        TIFF, PDF, PostScript, Photoshop, PNG, MIFF, PICT, QuickTime, XCF and some
+        RAW formats.  While the tags listed below are not individually writable, the
         entire profile itself can be accessed via the extra 'ICC_Profile' tag, but
-        this tag is neither extracted nor written unless specified explicitly.
+        this tag is neither extracted nor written unless specified explicitly.  See
+        L<http://www.color.org/icc_specs2.xalter> for the official ICC
+        specification.
     },
     A2B0 => 'AToB0',
     A2B1 => 'AToB1',
@@ -76,10 +83,14 @@ my %profileClass = (
         Groups => { 2 => 'Time' },
         PrintConv => '$self->ConvertDateTime($val)',
     },
-    targ => 'CharTarget',
+    targ => {
+        Name => 'CharTarget',
+        ValueConv => 'length $val > 128 ? \$val : $val',
+    },
     chad => 'ChromaticAdaptation',
     chrm => {
         Name => 'Chromaticity',
+        Groups => { 1 => 'ICC_Profile#' }, #(just for the group list)
         SubDirectory => {
             TagTable => 'Image::ExifTool::ICC_Profile::Chromaticity',
             Validate => '$type eq "chrm"',
@@ -93,7 +104,14 @@ my %profileClass = (
             Validate => '$type eq "clrt"',
         },
     },
-    cprt => 'ProfileCopyright',
+    clot => { # new in version 4.2
+        Name => 'ColorantTableOut',
+        Binary => 1,
+    },
+    cprt => {
+        Name => 'ProfileCopyright',
+        ValueConv => '$val=~s/\0.*//; $val', # may be null terminated
+    },
     crdi => 'CRDInfo', #2
     dmnd => {
         Name => 'DeviceMfgDesc',
@@ -139,7 +157,7 @@ my %profileClass = (
     psd1 => 'PostScript2CRD1', #2
     psd2 => 'PostScript2CRD2', #2
     ps2s => 'PostScript2CSA', #2
-    ps2i => 'PS2RenteringIntent', #2
+    ps2i => 'PS2RenderingIntent', #2
     rXYZ => 'RedMatrixColumn', # (called RedColorant in ref 2)
     rTRC => {
         Name => 'RedTRC',
@@ -149,7 +167,7 @@ my %profileClass = (
     scrn => 'Screening',
    'bfd '=> {
         Name => 'UCRBG',
-        Description => 'Under Color Removal & Black Gen.',
+        Description => 'Under Color Removal and Black Gen.',
     },
     tech => {
         Name => 'Technology',
@@ -176,6 +194,10 @@ my %profileClass = (
             offs => 'Offset Lithography',
             silk => 'Silkscreen',
             flex => 'Flexography',
+            mpfs => 'Motion Picture Film Scanner', #5
+            mpfr => 'Motion Picture Film Recorder', #5
+            dmpc => 'Digital Motion Picture Camera', #5
+            dcpj => 'Digital Cinema Projector', #5
         },
     },
     vued => 'ViewingCondDesc',
@@ -186,6 +208,44 @@ my %profileClass = (
             Validate => '$type eq "view"',
         },
     },
+    ciis => 'ColorimetricIntentImageState', #5
+    scoe => 'SceneColorimetryEstimates', #5
+    sape => 'SceneAppearanceEstimates', #5
+    fpce => 'FocalPlaneColorimetryEstimates', #5
+    rhoc => 'ReflectionHardcopyOrigColorimetry', #5
+    rpoc => 'ReflectionPrintOutputColorimetry', #5
+    psid => { #5
+        Name => 'ProfileSequenceIdentifier',
+        Binary => 1,
+    },
+    B2D0 => { Name => 'BToD0', Binary => 1 }, #5
+    B2D1 => { Name => 'BToD1', Binary => 1 }, #5
+    B2D2 => { Name => 'BToD2', Binary => 1 }, #5
+    B2D3 => { Name => 'BToD3', Binary => 1 }, #5
+    D2B0 => { Name => 'DToB0', Binary => 1 }, #5
+    D2B1 => { Name => 'DToB1', Binary => 1 }, #5
+    D2B2 => { Name => 'DToB2', Binary => 1 }, #5
+    D2B3 => { Name => 'DToB3', Binary => 1 }, #5
+    rig0 => { #5
+        Name => 'PerceptualRenderingIntentGamut',
+        PrintConv => {
+            prmg => 'Perceptual Reference Medium Gamut',
+        },
+    },
+    rig2 => { #5
+        Name => 'SaturationRenderingIntentGamut',
+        PrintConv => {
+            prmg => 'Perceptual Reference Medium Gamut',
+        },
+    },
+    meta => { #5 (EVENTUALLY DECODE THIS ONCE WE HAVE A SAMPLE!!)
+        Name => 'Metadata',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::ICC_Profile::Metadata',
+            Validate => '$type eq "meta"',
+        },
+    },
+
     # ColorSync custom tags (ref 3)
     psvm => 'PS2CRDVMSize',
     vcgt => 'VideoCardGamma',
@@ -193,6 +253,9 @@ my %profileClass = (
     dscm => 'ProfileDescriptionML',
     ndin => 'NativeDisplayInfo',
 
+    # Microsoft custom tags (ref http://msdn2.microsoft.com/en-us/library/ms536870.aspx)
+    MS00 => 'WCSProfiles',
+
     # the following entry represents the ICC profile header, and doesn't
     # exist as a tag in the directory.  It is only in this table to provide
     # a link so ExifTool can locate the header tags
@@ -263,19 +326,22 @@ my %profileClass = (
     48 => {
         Name => 'DeviceManufacturer',
         Format => 'string[4]',
+        # KODA = Kodak
     },
     52 => {
         Name => 'DeviceModel',
         Format => 'string[4]',
+        # ROMM = Refrence Output Medium Metric
     },
     56 => {
         Name => 'DeviceAttributes',
-        Format => 'int32u',
+        Format => 'int32u[2]',
         PrintConv => q[
-            ($val & 0x01 ? "Transparency, " : "Reflective, ") .
-            ($val & 0x02 ? "Matte, " : "Glossy, ") .
-            ($val & 0x04 ? "Negative, " : "Positive, ") .
-            ($val & 0x08 ? "B&W" : "Color")
+            my @v = split ' ', $val;
+            ($v[1] & 0x01 ? "Transparency, " : "Reflective, ") .
+            ($v[1] & 0x02 ? "Matte, " : "Glossy, ") .
+            ($v[1] & 0x04 ? "Negative, " : "Positive, ") .
+            ($v[1] & 0x08 ? "B&W" : "Color");
         ],
     },
     64 => {
@@ -295,6 +361,7 @@ my %profileClass = (
     80 => {
         Name => 'ProfileCreator',
         Format => 'string[4]',
+        # KODA = Kodak
     },
     84 => {
         Name => 'ProfileID',
@@ -349,7 +416,7 @@ my %profileClass = (
     28 => {
         Name => 'MeasurementFlare',
         Format => 'fixed32u',
-        PrintConv => '$val*100 . " %"',  # change into a percent
+        PrintConv => '$val*100 . "%"',  # change into a percent
     },
     32 => {
         Name => 'MeasurementIlluminant',
@@ -434,6 +501,19 @@ my %profileClass = (
     },
 );
 
+# metadata (meta) tags
+%Image::ExifTool::ICC_Profile::Metadata = (
+    PROCESS_PROC => \&ProcessMetadata,
+    GROUPS => { 0 => 'ICC_Profile', 1 => 'ICC-meta', 2 => 'Image' },
+    VARS => { NO_ID => 1 },
+    NOTES => q{
+        Only these few tags have been pre-defined, but ExifTool will extract any
+        Metadata tags that exist.
+    },
+    ManufacturerName => { },
+    MediaColor       => { },
+    MediaWeight      => { },
+);
 
 #------------------------------------------------------------------------------
 # print ICC Profile ID in hex
@@ -456,9 +536,10 @@ sub HexID($)
 # Returns: Formatted value or undefined if format not supported
 # Notes: The following types are handled by BinaryTables:
 #  chromaticityType, colorantTableType, measurementType, viewingConditionsType
-# The following types are not currently handled (most are large tables):
+# The following types are currently not handled (most are large tables):
 #  curveType, lut16Type, lut8Type, lutAtoBType, lutBtoAType, namedColor2Type,
 #  parametricCurveType, profileSeqDescType, responseCurveSet16Type
+# The multiLocalizedUnicodeType must be handled by the calling routine.
 sub FormatICCTag($$$)
 {
     my ($dataPt, $offset, $size) = @_;
@@ -487,22 +568,11 @@ sub FormatICCTag($$$)
     }
     # dateTimeType
     if ($type eq 'dtim' and $size >= 20) {
-        return sprintf("%.4d:$.2d:%.2d %.2d:%.2d:%.2d",
+        return sprintf("%.4d:%.2d:%.2d %.2d:%.2d:%.2d",
                Get16u($dataPt, $offset+8),  Get16u($dataPt, $offset+10),
                Get16u($dataPt, $offset+12), Get16u($dataPt, $offset+14),
                Get16u($dataPt, $offset+16), Get16u($dataPt, $offset+18));
     }
-    # multiLocalizedUnicodeType (replaces textDescriptionType of ref 2)
-    if ($type eq 'mluc' and $size >= 28) {
-        # take first language in list (pray that it is ascii)
-        my $len = Get32u($dataPt, $offset + 20);
-        my $pos = Get32u($dataPt, $offset + 24);
-        if ($size >= $pos + $len) {
-            my $str = substr($$dataPt, $offset + $pos, $len);
-            $str =~ tr/\x00-\x1f\x80-\xff//d; # remove control characters and non-ascii
-            return $str;
-        }
-    }
     # s15Fixed16ArrayType
     if ($type eq 'sf32') {
         return ReadValue($dataPt,$offset+8,'fixed32s',($size-8)/4,$size-8);
@@ -551,6 +621,64 @@ sub FormatICCTag($$$)
     return undef;   # data type is not supported
 }
 
+#------------------------------------------------------------------------------
+# Process ICC metadata record (ref 5) (UNTESTED!)
+# Inputs: 0) ExifTool ref, 1) dirInfo ref, 2) tag table ref
+# Returns: 1 on success
+sub ProcessMetadata($$$)
+{
+    my ($et, $dirInfo, $tagTablePtr) = @_;
+    my $dataPt = $$dirInfo{DataPt};
+    my $dirStart = $$dirInfo{DirStart};
+    my $dirLen = $$dirInfo{DirLen};
+    my $dirEnd = $dirStart + $dirLen;
+    
+    if ($dirLen < 16 or substr($$dataPt, $dirStart, 4) ne 'dict') {
+        $et->Warn('Invalid ICC meta dictionary');
+        return 0;
+    }
+    my $num = Get32u($dataPt, $dirStart + 8);
+    $et->VerboseDir('Metadata', $num);
+    my $size = Get32u($dataPt, $dirStart + 12);
+    $size < 16 and $et->Warn('Invalid ICC meta record size'), return 0;
+    # NOTE: In the example the minimum offset is 20,
+    # but this doesn't jive with the table (both in ref 5)
+    my $minPtr = 16 + $size * $num;
+    my $index;
+    for ($index=0; $index<$num; ++$index) {
+        my $entry = $dirStart + 16 + $size * $index;
+        if ($entry + $size > $dirEnd) {
+            $et->Warn('Truncated ICC meta dictionary');
+            last;
+        }
+        my $namePtr = Get32u($dataPt, $entry);
+        my $nameLen = Get32u($dataPt, $entry + 4);
+        my $valuePtr = Get32u($dataPt, $entry + 8);
+        my $valueLen = Get32u($dataPt, $entry + 16);
+        next unless $namePtr and $valuePtr;   # ignore if offsets are zero
+        if ($namePtr < $minPtr or $namePtr + $nameLen > $dirLen or
+            $valuePtr < $minPtr or $valuePtr + $valueLen > $dirLen)
+        {
+            $et->Warn('Corrupted ICC meta dictionary');
+            last;
+        }
+        my $tag = substr($dataPt, $dirStart + $namePtr, $nameLen);
+        my $val = substr($dataPt, $dirStart + $valuePtr, $valueLen);
+        $tag = $et->Decode($tag, 'UTF16', 'MM', 'UTF8');
+        $val = $et->Decode($val, 'UTF16', 'MM');
+        # generate tagInfo if it doesn't exist
+        unless ($$tagTablePtr{$tag}) {
+            my $name = ucfirst $tag;
+            $name =~ s/\s+(.)/\u$1/g;
+            $name =~ tr/-_a-zA-Z0-9//dc;
+            next unless length $name;
+            AddTagToTable($tagTablePtr, $tag, { Name => $name });
+        }
+        $et->HandleTag($tagTablePtr, $tag, $val);
+    }
+    return 1;
+}
+
 #------------------------------------------------------------------------------
 # Write ICC profile file
 # Inputs: 0) ExifTool object reference, 1) Reference to directory information
@@ -558,31 +686,39 @@ sub FormatICCTag($$$)
 #          or -1 if a write error occurred
 sub WriteICC($$)
 {
-    my ($exifTool, $dirInfo) = @_;
-    my $buff = WriteICC_Profile($exifTool, $dirInfo);
+    my ($et, $dirInfo) = @_;
+    # first make sure this is a valid ICC file (or no file at all)
+    my $raf = $$dirInfo{RAF};
+    my $buff;
+    return 0 if $raf->Read($buff, 24) and ValidateICC(\$buff);
+    # now write the new ICC
+    $buff = WriteICC_Profile($et, $dirInfo);
     if (defined $buff and length $buff) {
         Write($$dirInfo{OutFile}, $buff) or return -1;
     } else {
-        $exifTool->Error('No ICC information to write');
+        $et->Error('No ICC information to write');
     }
     return 1;
 }
 
 #------------------------------------------------------------------------------
-# Write ICC data record
+# Write ICC data as a block
 # Inputs: 0) ExifTool object reference, 1) source dirInfo reference,
 #         2) tag table reference
 # Returns: ICC data block (may be empty if no ICC data)
 # Notes: Increments ExifTool CHANGED flag if changed
 sub WriteICC_Profile($$;$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
-    $exifTool or return 1;    # allow dummy access
-    my $newValueHash = $exifTool->GetNewValueHash($Image::ExifTool::Extra{ICC_Profile});
-    return undef unless Image::ExifTool::IsOverwriting($newValueHash);
-    my $val = Image::ExifTool::GetNewValues($newValueHash);
+    my ($et, $dirInfo, $tagTablePtr) = @_;
+    $et or return 1;    # allow dummy access
+    my $dirName = $$dirInfo{DirName} || 'ICC_Profile';
+    # (don't write AsShotICCProfile or CurrentICCProfile here)
+    return undef unless $dirName eq 'ICC_Profile';
+    my $nvHash = $et->GetNewValueHash($Image::ExifTool::Extra{$dirName});
+    return undef unless $et->IsOverwriting($nvHash);
+    my $val = $et->GetNewValues($nvHash);
     $val = '' unless defined $val;
-    ++$exifTool->{CHANGED};
+    ++$$et{CHANGED};
     return $val;
 }
 
@@ -610,22 +746,22 @@ sub ValidateICC($)
 # Returns: 1 if this was an ICC file
 sub ProcessICC($$)
 {
-    my ($exifTool, $dirInfo) = @_;
+    my ($et, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
     my $buff;
     $raf->Read($buff, 24) == 24 or return 0;
     # check to see if this is a valid ICC profile file
     return 0 if ValidateICC(\$buff);
-    $exifTool->SetFileType();
+    $et->SetFileType();
     # read the profile
     my $size = unpack('N', $buff);
     if ($size < 128 or $size & 0x80000000) {
-        $exifTool->Error("Bad ICC Profile length ($size)");
+        $et->Error("Bad ICC Profile length ($size)");
         return 1;
     }
     $raf->Seek(0, 0);
     unless ($raf->Read($buff, $size)) {
-        $exifTool->Error('Truncated ICC profile');
+        $et->Error('Truncated ICC profile');
         return 1;
     }
     my %dirInfo = (
@@ -635,7 +771,7 @@ sub ProcessICC($$)
         DirLen => $size,
     );
     my $tagTablePtr = GetTagTable('Image::ExifTool::ICC_Profile::Main');
-    return ProcessICC_Profile($exifTool, \%dirInfo, $tagTablePtr);
+    return ProcessICC_Profile($et, \%dirInfo, $tagTablePtr);
 }
 
 #------------------------------------------------------------------------------
@@ -645,19 +781,29 @@ sub ProcessICC($$)
 # Returns: 1 on success
 sub ProcessICC_Profile($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
-    my $dirStart = $$dirInfo{DirStart};
+    my $dirStart = $$dirInfo{DirStart} || 0;
     my $dirLen = $$dirInfo{DirLen};
-    my $verbose = $exifTool->Options('Verbose');
-    my $raf;
+    my $verbose = $et->Options('Verbose');
+
+    return 0 if $dirLen < 4;
+
+    # extract binary ICC_Profile data block if binary mode or requested
+    if ((($$et{TAGS_FROM_FILE} and not $$et{EXCL_TAG_LOOKUP}{icc_profile}) or
+        $$et{REQ_TAG_LOOKUP}{icc_profile}) and
+        # (don't extract from AsShotICCProfile or CurrentICCProfile)
+        (not $$dirInfo{Name} or $$dirInfo{Name} eq 'ICC_Profile'))
+    {
+        $et->FoundTag('ICC_Profile', substr($$dataPt, $dirStart, $dirLen));
+    }
 
     SetByteOrder('MM');     # ICC_Profile is always big-endian
 
     # check length of table
     my $len = Get32u($dataPt, $dirStart);
     if ($len != $dirLen or $len < 128) {
-        $exifTool->Warn("Bad length ICC_Profile (length $len)");
+        $et->Warn("Bad length ICC_Profile (length $len)");
         return 0 if $len < 128 or $dirLen < $len;
     }
     my $pos = $dirStart + 128;  # position at start of table
@@ -665,22 +811,18 @@ sub ProcessICC_Profile($$$)
     if ($numEntries < 1 or $numEntries >= 0x100
         or $numEntries * 12 + 132 > $dirLen)
     {
-        $exifTool->Warn("Bad ICC_Profile table ($numEntries entries)");
+        $et->Warn("Bad ICC_Profile table ($numEntries entries)");
         return 0;
     }
-    # extract binary ICC_Profile data block if binary mode or requested
-    if ($exifTool->{OPTIONS}->{Binary} or $exifTool->{REQ_TAG_LOOKUP}->{icc_profile}) {
-        $exifTool->FoundTag('ICC_Profile', substr($$dataPt, $dirStart, $dirLen));
-    }
 
     if ($verbose) {
-        $exifTool->VerboseDir('ICC_Profile', $numEntries, $dirLen);
+        $et->VerboseDir('ICC_Profile', $numEntries, $dirLen);
         my $fakeInfo = { Name=>'ProfileHeader', SubDirectory => { } };
-        $exifTool->VerboseInfo(undef, $fakeInfo);
+        $et->VerboseInfo(undef, $fakeInfo);
     }
     # increment ICC dir count
-    my $dirCount = $exifTool->{DIR_COUNT}->{ICC} = ($exifTool->{DIR_COUNT}->{ICC} || 0) + 1;
-    $exifTool->{SET_GROUP1} = '+' . $dirCount if $dirCount > 1;
+    my $dirCount = $$et{DIR_COUNT}{ICC} = ($$et{DIR_COUNT}{ICC} || 0) + 1;
+    $$et{SET_GROUP1} = '+' . $dirCount if $dirCount > 1;
     # process the header block
     my %subdirInfo = (
         Name     => 'ProfileHeader',
@@ -689,9 +831,10 @@ sub ProcessICC_Profile($$$)
         DirStart => $dirStart,
         DirLen   => 128,
         Parent   => $$dirInfo{DirName},
+        DirName  => 'Header',
     );
     my $newTagTable = GetTagTable('Image::ExifTool::ICC_Profile::Header');
-    $exifTool->ProcessDirectory(\%subdirInfo, $newTagTable);
+    $et->ProcessDirectory(\%subdirInfo, $newTagTable);
 
     $pos += 4;    # skip item count
     my $index;
@@ -700,32 +843,73 @@ sub ProcessICC_Profile($$$)
         my $offset = Get32u($dataPt, $pos + 4);
         my $size   = Get32u($dataPt, $pos + 8);
         $pos += 12;
-        my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tagID);
+        my $tagInfo = $et->GetTagInfo($tagTablePtr, $tagID);
         # unknown tags aren't generated automatically by GetTagInfo()
         # if the tagID's aren't numeric, so we must do this manually:
-        if (not $tagInfo and $exifTool->{OPTIONS}->{Unknown}) {
+        if (not $tagInfo and $$et{OPTIONS}{Unknown}) {
             $tagInfo = { Unknown => 1 };
-            Image::ExifTool::AddTagToTable($tagTablePtr, $tagID, $tagInfo);
+            AddTagToTable($tagTablePtr, $tagID, $tagInfo);
         }
         next unless defined $tagInfo;
 
         if ($offset + $size > $dirLen) {
-            $exifTool->Warn("Bad ICC_Profile table (truncated)");
+            $et->Warn("Bad ICC_Profile table (truncated)");
             last;
         }
         my $valuePtr = $dirStart + $offset;
 
         my $subdir = $$tagInfo{SubDirectory};
         # format the value unless this is a subdirectory
-        my $value;
+        my ($value, $fmt);
+        if ($size > 4) {
+            $fmt = substr($$dataPt, $valuePtr, 4);
+            # handle multiLocalizedUnicodeType
+            if ($fmt eq 'mluc' and not $subdir) {
+                next if $size < 28;
+                my $count = Get32u($dataPt, $valuePtr + 8);
+                my $recLen = Get32u($dataPt, $valuePtr + 12);
+                next if $recLen < 12;
+                my $i;
+                for ($i=0; $i<$count; ++$i) {
+                    my $recPos = $valuePtr + 16 + $i * $recLen;
+                    last if $recPos + $recLen > $valuePtr + $size;
+                    my $lang = substr($$dataPt, $recPos, 4);
+                    my $langInfo;
+                    # validate language code and change to standard case (just in case)
+                    if ($lang =~ s/^([a-z]{2})([A-Z]{2})$/\L$1-\U$2/i and $lang ne 'en-US') {
+                        $langInfo = Image::ExifTool::GetLangInfo($tagInfo, $lang);
+                    }
+                    my $strLen = Get32u($dataPt, $recPos + 4);
+                    my $strPos = Get32u($dataPt, $recPos + 8);
+                    last if $strPos + $strLen > $size; 
+                    my $str = substr($$dataPt, $valuePtr + $strPos, $strLen);
+                    $str = $et->Decode($str, 'UTF16');
+                    $et->HandleTag($tagTablePtr, $tagID, $str,
+                        TagInfo => $langInfo || $tagInfo,
+                        Table  => $tagTablePtr,
+                        Index  => $index,
+                        Value  => $str,
+                        DataPt => $dataPt,
+                        Size   => $strLen,
+                        Start  => $valuePtr + $strPos,
+                        Format => "type '$fmt'",
+                    );
+                }
+                $et->Warn("Corrupted $$tagInfo{Name} data") if $i < $count;
+                next;
+            }
+        } else {
+            $fmt = 'err ';
+        }
         $value = FormatICCTag($dataPt, $valuePtr, $size) unless $subdir;
-        $verbose and $exifTool->VerboseInfo($tagID, $tagInfo,
-            'Table'  => $tagTablePtr,
-            'Index'  => $index,
-            'Value'  => $value,
-            'DataPt' => $dataPt,
-            'Size'   => $size,
-            'Start'  => $valuePtr,
+        $verbose and $et->VerboseInfo($tagID, $tagInfo,
+            Table  => $tagTablePtr,
+            Index  => $index,
+            Value  => $value,
+            DataPt => $dataPt,
+            Size   => $size,
+            Start  => $valuePtr,
+            Format => "type '$fmt'",
         );
         if ($subdir) {
             my $name = $$tagInfo{Name};
@@ -747,26 +931,26 @@ sub ProcessICC_Profile($$$)
                 DataLen  => $$dirInfo{DataLen},
                 DirStart => $valuePtr,
                 DirLen   => $size,
+                DirName  => $name,
                 Parent   => $$dirInfo{DirName},
             );
             my $type = substr($$dataPt, $valuePtr, 4);
             #### eval Validate ($type)
             if (defined $$subdir{Validate} and not eval $$subdir{Validate}) {
-                $exifTool->Warn("Invalid $name data");
+                $et->Warn("Invalid $name data");
             } else {
-                $exifTool->ProcessDirectory(\%subdirInfo, $newTagTable, $$subdir{ProcessProc});
+                $et->ProcessDirectory(\%subdirInfo, $newTagTable, $$subdir{ProcessProc});
             }
         } elsif (defined $value) {
-            $exifTool->FoundTag($tagInfo, $value);
+            $et->FoundTag($tagInfo, $value);
         } else {
             $value = substr($$dataPt, $valuePtr, $size);
             # treat unsupported formats as binary data
-            my $oldValueConv = $$tagInfo{ValueConv};
             $$tagInfo{ValueConv} = '\$val' unless defined $$tagInfo{ValueConv};
-            $exifTool->FoundTag($tagInfo, $value);
+            $et->FoundTag($tagInfo, $value);
         }
     }
-    delete $exifTool->{SET_GROUP1};
+    delete $$et{SET_GROUP1};
     return 1;
 }
 
@@ -792,7 +976,7 @@ data created on one device into another device's native color space.
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
