@@ -13,8 +13,9 @@ package Image::ExifTool::IPTC;
 
 use strict;
 use vars qw($VERSION $AUTOLOAD %iptcCharset);
+use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.42';
+$VERSION = '1.49';
 
 %iptcCharset = (
     "\x1b%G"  => 'UTF8',
@@ -32,6 +33,17 @@ sub CheckIPTC($$$);
 sub PrintCodedCharset($);
 sub PrintInvCodedCharset($);
 
+# standard IPTC locations
+# (MWG specifies locations only for JPEG, TIFF and PSD -- the rest are ExifTool-defined)
+my %isStandardIPTC = (
+    'JPEG-APP13-Photoshop-IPTC' => 1,
+    'TIFF-IFD0-IPTC'            => 1,
+    'PSD-IPTC'                  => 1,
+    'MIE-IPTC'                  => 1,
+    'EPS-Photoshop-IPTC'        => 1,
+    'PS-Photoshop-IPTC'         => 1,
+);
+
 my %fileFormat = (
     0 => 'No ObjectData',
     1 => 'IPTC-NAA Digital Newsphoto Parameter Record',
@@ -181,6 +193,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
     },
     80 => {
         Name => 'TimeSent',
@@ -189,6 +202,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
     },
     90 => {
         Name => 'CodedCharacterSet',
@@ -316,6 +330,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
     },
     35 => {
         Name => 'ReleaseTime',
@@ -324,6 +339,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
     },
     37 => {
         Name => 'ExpirationDate',
@@ -332,6 +348,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
     },
     38 => {
         Name => 'ExpirationTime',
@@ -340,6 +357,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
     },
     40 => {
         Name => 'SpecialInstructions',
@@ -352,7 +370,7 @@ my %fileFormat = (
             '' => '',
             '01' => 'Object Kill',
             '02' => 'Object Replace',
-            '03' => 'Ojbect Append',
+            '03' => 'Object Append',
             '04' => 'Object Reference',
         },
     },
@@ -369,6 +387,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
     },
     50 => {
         Name => 'ReferenceNumber',
@@ -382,6 +401,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
     },
     60 => {
         Name => 'TimeCreated',
@@ -390,6 +410,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
     },
     62 => {
         Name => 'DigitalCreationDate',
@@ -398,6 +419,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
     },
     63 => {
         Name => 'DigitalCreationTime',
@@ -406,6 +428,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
     },
     65 => {
         Name => 'OriginatingProgram',
@@ -953,18 +976,18 @@ sub PrintCodedCharset($)
 # Returns: IPTC character set if translation required (or 'bad' if unknown)
 sub HandleCodedCharset($$)
 {
-    my ($exifTool, $val) = @_;
+    my ($et, $val) = @_;
     my $xlat = $iptcCharset{$val};
     unless ($xlat) {
         if ($val =~ /^\x1b\x25/) {
             # some unknown character set invoked
             $xlat = 'bad';  # flag unsupported coding
         } else {
-            $xlat = $exifTool->Options('CharsetIPTC');
+            $xlat = $et->Options('CharsetIPTC');
         }
     }
     # no need to translate if Charset is the same
-    undef $xlat if $xlat eq $exifTool->Options('Charset');
+    undef $xlat if $xlat eq $et->Options('Charset');
     return $xlat;
 }
 
@@ -975,17 +998,17 @@ sub HandleCodedCharset($$)
 # Updates value on return
 sub TranslateCodedString($$$$)
 {
-    my ($exifTool, $valPtr, $xlatPtr, $read) = @_;
+    my ($et, $valPtr, $xlatPtr, $read) = @_;
     if ($$xlatPtr eq 'bad') {
-        $exifTool->Warn('Some IPTC characters not converted (unsupported CodedCharacterSet)');
+        $et->Warn('Some IPTC characters not converted (unsupported CodedCharacterSet)');
         undef $$xlatPtr;
     } elsif (not $read) {
-        $$valPtr = $exifTool->Decode($$valPtr, undef, undef, $$xlatPtr);
+        $$valPtr = $et->Decode($$valPtr, undef, undef, $$xlatPtr);
     } elsif ($$valPtr !~ /[\x14\x15\x1b]/) {
-        $$valPtr = $exifTool->Decode($$valPtr, $$xlatPtr);
+        $$valPtr = $et->Decode($$valPtr, $$xlatPtr);
     } else {
         # don't yet support reading ISO 2022 shifted character sets
-        $exifTool->WarnOnce('Some IPTC characters not converted (ISO 2022 shifting not supported)');
+        $et->WarnOnce('Some IPTC characters not converted (ISO 2022 shifting not supported)');
     }
 }
 
@@ -996,7 +1019,7 @@ sub TranslateCodedString($$$$)
 sub IsStandardIPTC($)
 {
     my $path = shift;
-    return $path =~ /^(JPEG-APP13-Photoshop-IPTC|TIFF-IFD0-IPTC|PSD-IPTC|MIE-IPTC)$/
+    return $isStandardIPTC{$path};
 }
 
 #------------------------------------------------------------------------------
@@ -1006,20 +1029,20 @@ sub IsStandardIPTC($)
 # Returns: 1 on success, 0 otherwise
 sub ProcessIPTC($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $pos = $$dirInfo{DirStart} || 0;
     my $dirLen = $$dirInfo{DirLen} || 0;
     my $dirEnd = $pos + $dirLen;
-    my $verbose = $exifTool->Options('Verbose');
+    my $verbose = $et->Options('Verbose');
     my $success = 0;
     my ($lastRec, $recordPtr, $recordName);
 
-    $verbose and $dirInfo and $exifTool->VerboseDir('IPTC', 0, $$dirInfo{DirLen});
+    $verbose and $dirInfo and $et->VerboseDir('IPTC', 0, $$dirInfo{DirLen});
 
     if ($tagTablePtr eq \%Image::ExifTool::IPTC::Main) {
         # calculate MD5 if Digest::MD5 is available (for standard IPTC only)
-        my $path = $exifTool->MetadataPath();
+        my $path = $et->MetadataPath();
         if (IsStandardIPTC($path)) {
             my $md5;
             if (eval 'require Digest::MD5') {
@@ -1032,32 +1055,42 @@ sub ProcessIPTC($$$)
                 # a zero digest indicates IPTC exists but we don't have Digest::MD5
                 $md5 = "\0" x 16;
             }
-            $exifTool->FoundTag('CurrentIPTCDigest', $md5);
-        } elsif ($Image::ExifTool::MWG::strict and $$exifTool{FILE_TYPE} =~ /^(JPEG|TIFF|PSD)$/) {
+            $et->FoundTag('CurrentIPTCDigest', $md5);
+        } elsif ($Image::ExifTool::MWG::strict and $$et{FILE_TYPE} =~ /^(JPEG|TIFF|PSD)$/) {
             # ignore non-standard IPTC while in strict MWG compatibility mode
-            $exifTool->Warn("Ignored non-standard IPTC at $path");
+            $et->Warn("Ignored non-standard IPTC at $path");
             return 1;
         }
         # set family 1 group name if multiple IPTC directories
-        my $dirCount = ($exifTool->{DIR_COUNT}->{IPTC} || 0) + 1;
-        $exifTool->{DIR_COUNT}->{IPTC} = $dirCount;
-        $exifTool->{SET_GROUP1} = '+' . $dirCount if $dirCount > 1;
+        my $dirCount = ($$et{DIR_COUNT}{IPTC} || 0) + 1;
+        $$et{DIR_COUNT}{IPTC} = $dirCount;
+        $$et{SET_GROUP1} = '+' . $dirCount if $dirCount > 1;
     }
     # begin by assuming default IPTC encoding
-    my $xlat = $exifTool->Options('CharsetIPTC');
-    undef $xlat if $xlat eq $exifTool->Options('Charset');
+    my $xlat = $et->Options('CharsetIPTC');
+    undef $xlat if $xlat eq $et->Options('Charset');
 
     # quick check for improperly byte-swapped IPTC
     if ($dirLen >= 4 and substr($$dataPt, $pos, 1) ne "\x1c" and
                          substr($$dataPt, $pos + 3, 1) eq "\x1c")
     {
-        $exifTool->Warn('IPTC data was improperly byte-swapped');
+        $et->Warn('IPTC data was improperly byte-swapped');
         my $newData = pack('N*', unpack('V*', substr($$dataPt, $pos, $dirLen) . "\0\0\0"));
         $dataPt = \$newData;
         $pos = 0;
         $dirEnd = $pos + $dirLen;
         # NOTE: MUST NOT access $dirInfo DataPt, DirStart or DataLen after this!
     }
+    # extract IPTC as a block if specified
+    if ($$et{REQ_TAG_LOOKUP}{iptc} or ($$et{TAGS_FROM_FILE} and
+        not $$et{EXCL_TAG_LOOKUP}{iptc}))
+    {
+        if ($pos or $dirLen != length($$dataPt)) {
+            $et->FoundTag('IPTC', substr($$dataPt, $pos, $dirLen));
+        } else {
+            $et->FoundTag('IPTC', $$dataPt);
+        }
+    }
     while ($pos + 5 <= $dirEnd) {
         my $buff = substr($$dataPt, $pos, 5);
         my ($id, $rec, $tag, $len) = unpack("CCCn", $buff);
@@ -1068,31 +1101,15 @@ sub ProcessIPTC($$$)
                 my $remaining = substr($$dataPt, $pos, $dirEnd - $pos);
                 last unless $remaining =~ /[^\0]/;
             }
-            $exifTool->Warn(sprintf('Bad IPTC data tag (marker 0x%x)',$id));
+            $et->Warn(sprintf('Bad IPTC data tag (marker 0x%x)',$id));
             last;
         }
-        if (not defined $lastRec or $lastRec != $rec) {
-            my $tableInfo = $tagTablePtr->{$rec};
-            unless ($tableInfo) {
-                $exifTool->Warn("Unrecognized IPTC record $rec, subsequent records ignored");
-                last;   # stop now because we're probably reading garbage
-            }
-            my $tableName = $tableInfo->{SubDirectory}->{TagTable};
-            unless ($tableName) {
-                $exifTool->Warn("No table for IPTC record $rec!");
-                last;   # this shouldn't happen
-            }
-            $recordName = $$tableInfo{Name};
-            $recordPtr = Image::ExifTool::GetTagTable($tableName);
-            $exifTool->VPrint(0,$$exifTool{INDENT},"-- $recordName record --\n");
-            $lastRec = $rec;
-        }
         $pos += 5;      # step to after field header
         # handle extended IPTC entry if necessary
         if ($len & 0x8000) {
             my $n = $len & 0x7fff; # get num bytes in length field
             if ($pos + $n > $dirEnd or $n > 8) {
-                $exifTool->VPrint(0, "Invalid extended IPTC entry (tag $tag)\n");
+                $et->VPrint(0, "Invalid extended IPTC entry (dataset $rec:$tag, len $len)\n");
                 $success = 0;
                 last;
             }
@@ -1102,20 +1119,37 @@ sub ProcessIPTC($$$)
             }
         }
         if ($pos + $len > $dirEnd) {
-            $exifTool->VPrint(0, "Invalid IPTC entry (tag $tag, len $len)\n");
+            $et->VPrint(0, "Invalid IPTC entry (dataset $rec:$tag, len $len)\n");
             $success = 0;
             last;
         }
+        if (not defined $lastRec or $lastRec != $rec) {
+            my $tableInfo = $tagTablePtr->{$rec};
+            unless ($tableInfo) {
+                $et->WarnOnce("Unrecognized IPTC record $rec (ignored)");
+                $pos += $len;
+                next;   # ignore this entry
+            }
+            my $tableName = $tableInfo->{SubDirectory}->{TagTable};
+            unless ($tableName) {
+                $et->Warn("No table for IPTC record $rec!");
+                last;   # this shouldn't happen
+            }
+            $recordName = $$tableInfo{Name};
+            $recordPtr = Image::ExifTool::GetTagTable($tableName);
+            $et->VPrint(0,$$et{INDENT},"-- $recordName record --\n");
+            $lastRec = $rec;
+        }
         my $val = substr($$dataPt, $pos, $len);
 
         # add tagInfo for all unknown tags:
         unless ($$recordPtr{$tag}) {
             # - no Format so format is auto-detected
             # - no Name so name is generated automatically with decimal tag number
-            Image::ExifTool::AddTagToTable($recordPtr, $tag, { Unknown => 1 });
+            AddTagToTable($recordPtr, $tag, { Unknown => 1 });
         }
 
-        my $tagInfo = $exifTool->GetTagInfo($recordPtr, $tag);
+        my $tagInfo = $et->GetTagInfo($recordPtr, $tag);
         my $format;
         # (could use $$recordPtr{FORMAT} if no Format below, but don't do this to
         #  be backward compatible with improperly written PhotoMechanic tags)
@@ -1137,11 +1171,11 @@ sub ProcessIPTC($$$)
                 $val =~ s/\0+$//;   # some braindead softwares add null terminators
                 if ($rec == 1) {
                     # handle CodedCharacterSet tag
-                    $xlat = HandleCodedCharset($exifTool, $val) if $tag == 90;
+                    $xlat = HandleCodedCharset($et, $val) if $tag == 90;
                 # translate characters if necessary and special characters exist
                 } elsif ($xlat and $rec < 7 and $val =~ /[\x80-\xff]/) {
                     # translate to specified character set
-                    TranslateCodedString($exifTool, \$val, \$xlat, 1);
+                    TranslateCodedString($et, \$val, \$xlat, 1);
                 }
             } elsif ($format =~ /^digits/) {
                 $val =~ s/\0+$//;
@@ -1149,7 +1183,7 @@ sub ProcessIPTC($$$)
                 warn("Invalid IPTC format: $format");
             }
         }
-        $verbose and $exifTool->VerboseInfo($tag, $tagInfo,
+        $verbose and $et->VerboseInfo($tag, $tagInfo,
             Table   => $tagTablePtr,
             Value   => $val,
             DataPt  => $dataPt,
@@ -1159,12 +1193,12 @@ sub ProcessIPTC($$$)
             Extra   => ", $recordName record",
             Format  => $format,
         );
-        $exifTool->FoundTag($tagInfo, $val) if $tagInfo;
+        $et->FoundTag($tagInfo, $val) if $tagInfo;
         $success = 1;
 
         $pos += $len;   # increment to next field
     }
-    delete $exifTool->{SET_GROUP1};
+    delete $$et{SET_GROUP1};
     return $success;
 }
 
@@ -1189,7 +1223,7 @@ image files.
 
 =head1 AUTHOR
 
-Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
