@@ -11,9 +11,8 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.13';
+$VERSION = '1.20';
 
-sub ProcessScalado($$$);
 sub ProcessOcad($$$);
 
 # (this main JPEG table is for documentation purposes only)
@@ -56,6 +55,10 @@ sub ProcessOcad($$$);
         Name => 'QVCI',
         Condition => '$$valPt =~ /^QVCI\0/',
         SubDirectory => { TagTable => 'Image::ExifTool::Casio::QVCI' },
+      }, {
+        Name => 'FLIR',
+        Condition => '$$valPt =~ /^FLIR\0/',
+        SubDirectory => { TagTable => 'Image::ExifTool::FLIR::FFF' },
     }],
     APP2 => [{
         Name => 'ICC_Profile',
@@ -71,8 +74,8 @@ sub ProcessOcad($$$);
         SubDirectory => { TagTable => 'Image::ExifTool::MPF::Main' },
       }, {
         Name => 'PreviewImage',
-        Condition => '$$valPt =~ /^\xff\xd8\xff\xdb/',
-        Notes => 'Samsung large preview',
+        Condition => '$$valPt =~ /^(|QVGA\0|BGTH)\xff\xd8\xff\xdb/',
+        Notes => 'Samsung APP2 preview image', # (Samsung="", BenQ="QVGA\0", Digilife="BGTH")
     }],
     APP3 => [{
         Name => 'Meta',
@@ -85,25 +88,28 @@ sub ProcessOcad($$$);
       }, {
         Name => 'PreviewImage', # (written by HP R837 and Samsung S1060)
         Condition => '$$valPt =~ /^\xff\xd8\xff\xdb/',
-        Notes => 'Hewlett-Packard or Samsung preview image',
+        Notes => 'Samsung/HP preview image', # (Samsung, HP, BenQ)
     }],
     APP4 => [{
         Name => 'Scalado',
         Condition => '$$valPt =~ /^SCALADO\0/',
-        SubDirectory => { TagTable => 'Image::ExifTool::JPEG::Scalado' },
+        SubDirectory => { TagTable => 'Image::ExifTool::Scalado::Main' },
       }, {
         Name => 'FPXR', # (non-standard location written by some HP models)
         Condition => '$$valPt =~ /^FPXR\0/',
         SubDirectory => { TagTable => 'Image::ExifTool::FlashPix::Main' },
       }, {
-        Name => 'PreviewImage', # (written by S1060)
-        Notes => 'Continued Samsung preview from APP3',
+        Name => 'PreviewImage', # (ie. Samsung S1060)
+        Notes => 'continued from APP3',
     }],
-    APP5 => {
+    APP5 => [{
         Name => 'RMETA',
         Condition => '$$valPt =~ /^RMETA\0/',
         SubDirectory => { TagTable => 'Image::ExifTool::Ricoh::RMETA' },
-    },
+      }, {
+        Name => 'PreviewImage', # (ie. BenQ DC E1050)
+        Notes => 'continued from APP4',
+    }],
     APP6 => [{
         Name => 'EPPIM',
         Condition => '$$valPt =~ /^EPPIM\0/',
@@ -117,6 +123,15 @@ sub ProcessOcad($$$);
         Condition => '$$valPt =~ /^TDHD\x01\0\0\0/',
         SubDirectory => { TagTable => 'Image::ExifTool::HP::TDHD' },
     }],
+    APP7 => [{
+        Name => 'Pentax',
+        Condition => '$$valPt =~ /^PENTAX \0/',
+        SubDirectory => { TagTable => 'Image::ExifTool::Pentax::Main' },
+      }, {
+        Name => 'Qualcomm',
+        Condition => '$$valPt =~ /^\x1aQualcomm Camera Attributes/',
+        SubDirectory => { TagTable => 'Image::ExifTool::Qualcomm::Main' },
+    }],
     APP8 => {
         Name => 'SPIFF',
         Condition => '$$valPt =~ /^SPIFF\0/',
@@ -148,6 +163,7 @@ sub ProcessOcad($$$);
     APP14 => {
         Name => 'Adobe',
         Condition => '$$valPt =~ /^Adobe/',
+        Writable => 1,  # (for docs only)
         SubDirectory => { TagTable => 'Image::ExifTool::JPEG::Adobe' },
     },
     APP15 => {
@@ -155,6 +171,7 @@ sub ProcessOcad($$$);
         Condition => '$$valPt =~ /^Q\s*(\d+)/',
         SubDirectory => { TagTable => 'Image::ExifTool::JPEG::GraphConv' },
     },
+    # APP15 - Also unknown "TEXT\0" segment stored by Casio/FujiFilm
     COM => {
         Name => 'Comment',
         # note: flag as writable for documentation, but it won't show up
@@ -323,7 +340,12 @@ sub ProcessOcad($$$);
 %Image::ExifTool::JPEG::Adobe = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     GROUPS => { 0 => 'APP14', 1 => 'Adobe', 2 => 'Image' },
-    NOTES => 'The "Adobe" APP14 segment stores image encoding information for DCT filters.',
+    NOTES => q{
+        The "Adobe" APP14 segment stores image encoding information for DCT filters.
+        This segment may be copied or deleted as a block using the Extra "Adobe"
+        tag, but note that it is not deleted by default when deleting all metadata
+        because it may affect the appearance of the image.
+    },
     FORMAT => 'int16u',
     0 => 'DCTEncodeVersion',
     1 => {
@@ -445,92 +467,23 @@ sub ProcessOcad($$$);
     },
 );
 
-# information written by Scalado software (PhotoFusion maybe?)
-%Image::ExifTool::JPEG::Scalado = (
-    GROUPS => { 0 => 'APP4', 1 => 'Scalado', 2 => 'Image' },
-    PROCESS_PROC => \&ProcessScalado,
-    TAG_PREFIX => 'Scalado',
-    FORMAT => 'int32s',
-    # I presume this was written by 
-    NOTES => q{
-        Tags extracted from the JPEG APP4 "SCALADO" segment (presumably written by
-        Scalado mobile software, L<http://www.scalado.com/>).
-    },
-    SPMO => {
-        Name => 'DataLength',
-        Unkown => 1,
-    },
-    WDTH => {
-        Name => 'PreviewImageWidth',
-        ValueConv => '$val ? abs($val) : undef',
-    },
-    HGHT => {
-        Name => 'PreviewImageHeight',
-        ValueConv => '$val ? abs($val) : undef',
-    },
-    QUAL => {
-        Name => 'PreviewQuality',
-        ValueConv => '$val ? abs($val) : undef',
-    },
-    # tags not yet decoded with observed values:
-    # CHKH: 0, -9010
-    # CHKL: -2664, -12852
-    # CLEN: -1024
-    # CSPC: -2232593
-    # DATA: (+ve data length)
-    # HDEC: 0
-    # MAIN: 0
-    # SCI0: (+ve data length)
-    # SCX1: (+ve data length)
-    # WDEC: 0
-);
-
 #------------------------------------------------------------------------------
 # Extract information from the JPEG APP0 Ocad segment
 # Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
 # Returns: 1 on success
 sub ProcessOcad($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
-    $exifTool->VerboseDir('APP0 Ocad', undef, length $$dataPt);
+    $et->VerboseDir('APP0 Ocad', undef, length $$dataPt);
     for (;;) {
         last unless $$dataPt =~ /\$(\w+):([^\0\$]+)/g;
         my ($tag, $val) = ($1, $2);
-        $val =~ s/(^\s+|\s+$)//g;   # remove leading/trailing spaces
+        $val =~ s/^\s+//; $val =~ s/\s+$//;     # remove leading/trailing spaces
         unless ($$tagTablePtr{$tag}) {
-            Image::ExifTool::AddTagToTable($tagTablePtr, $tag, { Name => "Ocad_$tag" });
+            AddTagToTable($tagTablePtr, $tag, { Name => "Ocad_$tag" });
         }
-        $exifTool->HandleTag($tagTablePtr, $tag, $val);
-    }
-    return 1;
-}
-
-#------------------------------------------------------------------------------
-# Extract information from the JPEG APP4 SCALADO segment
-# Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
-# Returns: 1 on success
-sub ProcessScalado($$$)
-{
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
-    my $dataPt = $$dirInfo{DataPt};
-    my $pos = 0;
-    my $end = length $$dataPt;
-    SetByteOrder('MM');
-    $exifTool->VerboseDir('APP4 SCALADO', undef, $end);
-    for (;;) {
-        last if $pos + 12 > $end;
-        my $tag = substr($$dataPt, $pos, 4);
-        my $unk = Get32u($dataPt, $pos + 4); # (what is this?)
-        $exifTool->HandleTag($tagTablePtr, $tag, undef,
-            DataPt  => $dataPt,
-            Start   => $pos + 8,
-            Size    => 4,
-            Extra   => ", unk $unk",
-        );
-        # shorten directory size by length of SPMO
-        $end -= Get32u($dataPt, $pos + 8) if $tag eq 'SPMO';
-        $pos += 12;
+        $et->HandleTag($tagTablePtr, $tag, $val);
     }
     return 1;
 }
@@ -555,7 +508,7 @@ segments are included in the Image::ExifTool module itself.
 
 =head1 AUTHOR
 
-Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
