@@ -18,7 +18,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.13';
+$VERSION = '1.15';
 
 %Image::ExifTool::MPEG::Audio = (
     GROUPS => { 2 => 'Audio' },
@@ -441,7 +441,7 @@ Image::ExifTool::AddCompositeTags('Image::ExifTool::MPEG');
 # Inputs: 0) ExifTool object ref, 1) tag table ref, 2-N) list of 32-bit data words
 sub ProcessFrameHeader($$@)
 {
-    my ($exifTool, $tagTablePtr, @data) = @_;
+    my ($et, $tagTablePtr, @data) = @_;
     my $tag;
     foreach $tag (sort keys %$tagTablePtr) {
         next unless $tag =~ /^Bit(\d{2})-?(\d{2})?/;
@@ -453,19 +453,20 @@ sub ProcessFrameHeader($$@)
             $mask += (1 << $_);
         }
         my $val = ($word >> (31 + 32*$index - $b2)) & $mask;
-        $exifTool->HandleTag($tagTablePtr, $tag, $val);
+        $et->HandleTag($tagTablePtr, $tag, $val);
     }
 }
 
 #------------------------------------------------------------------------------
 # Read MPEG audio frame header
 # Inputs: 0) ExifTool object reference, 1) Reference to audio data
+#         2) flag set if we are trying to recognized MP3 file only
 # Returns: 1 on success, 0 if no audio header was found
-sub ParseMPEGAudio($$)
+sub ParseMPEGAudio($$;$)
 {
-    my ($exifTool, $buffPt) = @_;
+    my ($et, $buffPt, $mp3) = @_;
     my ($word, $pos);
-    my $ext = $$exifTool{FILE_EXT} || '';
+    my $ext = $$et{FILE_EXT} || '';
 
     for (;;) {
         # find frame sync
@@ -481,7 +482,8 @@ sub ParseMPEGAudio($$)
             ($word & 0x00f000) == 0x000000 or   # 0000 is the "free" bitrate index
             ($word & 0x00f000) == 0x00f000 or   # 1111 is a bad bitrate index
             ($word & 0x000c00) == 0x000c00 or   # 11 is a reserved sampling frequency
-            ($word & 0x000003) == 0x000002)     # 10 is a reserved emphasis
+            ($word & 0x000003) == 0x000002 or   # 10 is a reserved emphasis
+            (($mp3 and ($word & 0x060000) != 0x020000))) # must be layer 3 for MP3
         {
             # give up easily unless this really should be an MP3 file
             return 0 unless $ext eq 'MP3';
@@ -492,13 +494,13 @@ sub ParseMPEGAudio($$)
         last;
     }
     # set file type if not done already
-    $exifTool->SetFileType();
+    $et->SetFileType();
 
     my $tagTablePtr = GetTagTable('Image::ExifTool::MPEG::Audio');
-    ProcessFrameHeader($exifTool, $tagTablePtr, $word);
+    ProcessFrameHeader($et, $tagTablePtr, $word);
 
     # extract the VBR information (ref MP3::Info)
-    my ($v, $m) = ($$exifTool{MPEG_Vers}, $$exifTool{MPEG_Mode});
+    my ($v, $m) = ($$et{MPEG_Vers}, $$et{MPEG_Mode});
     while (defined $v and defined $m) {
         my $len = length $$buffPt;
         $pos += $v == 3 ? ($m == 3 ? 17 : 32) : ($m == 3 ?  9 : 17);
@@ -512,12 +514,12 @@ sub ParseMPEGAudio($$)
         $pos += 8;
         if ($flags & 0x01) {    # VBRFrames
             last if $pos + 4 > $len;
-            $exifTool->HandleTag($xingTable, 1, unpack("x${pos}N", $$buffPt)) if $isVBR;
+            $et->HandleTag($xingTable, 1, unpack("x${pos}N", $$buffPt)) if $isVBR;
             $pos += 4;
         }
         if ($flags & 0x02) {    # VBRBytes
             last if $pos + 4 > $len;
-            $exifTool->HandleTag($xingTable, 2, unpack("x${pos}N", $$buffPt)) if $isVBR;
+            $et->HandleTag($xingTable, 2, unpack("x${pos}N", $$buffPt)) if $isVBR;
             $pos += 4;
         }
         if ($flags & 0x04) {    # VBR_TOC
@@ -528,7 +530,7 @@ sub ParseMPEGAudio($$)
         if ($flags & 0x08) {    # VBRScale
             last if $pos + 4 > $len;
             $vbrScale = unpack("x${pos}N", $$buffPt);
-            $exifTool->HandleTag($xingTable, 3, $vbrScale) if $isVBR;
+            $et->HandleTag($xingTable, 3, $vbrScale) if $isVBR;
             $pos += 4;
         }
         # process Lame header (ref 5)
@@ -550,7 +552,7 @@ sub ParseMPEGAudio($$)
                 } else {
                     last;
                 }
-                $exifTool->HandleTag($xingTable, 4, $lib);
+                $et->HandleTag($xingTable, 4, $lib);
                 last;
             }
         }
@@ -558,10 +560,10 @@ sub ParseMPEGAudio($$)
         last if $lameLen < 9;
         my $enc = substr($$buffPt, $pos, 9);
         if ($enc ge 'LAME3.90') {
-            $exifTool->HandleTag($xingTable, 4, $enc);
+            $et->HandleTag($xingTable, 4, $enc);
             if ($vbrScale <= 100) {
-                $exifTool->HandleTag($xingTable, 5, int((100 - $vbrScale) / 10));
-                $exifTool->HandleTag($xingTable, 6, (100 - $vbrScale) % 10);
+                $et->HandleTag($xingTable, 5, int((100 - $vbrScale) / 10));
+                $et->HandleTag($xingTable, 6, (100 - $vbrScale) % 10);
             }
             my %dirInfo = (
                 DataPt   => $buffPt,
@@ -569,12 +571,12 @@ sub ParseMPEGAudio($$)
                 DirLen   => length($$buffPt) - $pos,
             );
             my $subTablePtr = GetTagTable('Image::ExifTool::MPEG::Lame');
-            $exifTool->ProcessDirectory(\%dirInfo, $subTablePtr);
+            $et->ProcessDirectory(\%dirInfo, $subTablePtr);
         } else {
-            $exifTool->HandleTag($xingTable, 4, substr($$buffPt, $pos, 20));
+            $et->HandleTag($xingTable, 4, substr($$buffPt, $pos, 20));
         }
         last;   # (didn't want to loop anyway)
-	}
+    }
 
     return 1;
 }
@@ -585,7 +587,7 @@ sub ParseMPEGAudio($$)
 # Returns: 1 on success, 0 if no video header was found
 sub ProcessMPEGVideo($$)
 {
-    my ($exifTool, $buffPt) = @_;
+    my ($et, $buffPt) = @_;
 
     return 0 unless length $$buffPt >= 4;
     my ($w1, $w2) = unpack('N2', $$buffPt);
@@ -598,10 +600,10 @@ sub ProcessMPEGVideo($$)
         return 0;
     }
     # set file type if not done already
-    $exifTool->SetFileType('MPEG') unless $exifTool->{VALUE}->{FileType};
+    $et->SetFileType('MPEG') unless $$et{VALUE}{FileType};
 
     my $tagTablePtr = GetTagTable('Image::ExifTool::MPEG::Video');
-    ProcessFrameHeader($exifTool, $tagTablePtr, $w1, $w2);
+    ProcessFrameHeader($et, $tagTablePtr, $w1, $w2);
     return 1;
 }
 
@@ -625,13 +627,13 @@ sub ProcessMPEGVideo($$)
 #   0xe0-0xef - video stream
 sub ParseMPEGAudioVideo($$)
 {
-    my ($exifTool, $buffPt) = @_;
+    my ($et, $buffPt) = @_;
     my (%found, $didHdr);
     my $rtnVal = 0;
     my %proc = ( audio => \&ParseMPEGAudio, video => \&ProcessMPEGVideo );
 
-    delete $exifTool->{AudioBitrate};
-    delete $exifTool->{VideoBitrate};
+    delete $$et{AudioBitrate};
+    delete $$et{VideoBitrate};
 
     while ($$buffPt =~ /\0\0\x01(\xb3|\xc0)/g) {
         my $type = $1 eq "\xb3" ? 'video' : 'audio';
@@ -640,7 +642,7 @@ sub ParseMPEGAudioVideo($$)
             # (the last byte of the 4-byte MP3 audio frame header word may be zero,
             # but the 2nd last must be non-zero, so we need only check to pos-3)
             my $buff = substr($$buffPt, 0, pos($$buffPt) - 3);
-            $found{audio} = 1 if ParseMPEGAudio($exifTool, \$buff);
+            $found{audio} = 1 if ParseMPEGAudio($et, \$buff);
             $didHdr = 1;
         }
         next if $found{$type};
@@ -649,7 +651,7 @@ sub ParseMPEGAudioVideo($$)
         $len > 256 and $len = 256;
         my $dat = substr($$buffPt, pos($$buffPt), $len);
         # process MPEG audio or video
-        if (&{$proc{$type}}($exifTool, \$dat)) {
+        if (&{$proc{$type}}($et, \$dat)) {
             $rtnVal = 1;
             $found{$type} = 1;
             # done if we found audio and video
@@ -665,18 +667,18 @@ sub ParseMPEGAudioVideo($$)
 # Returns: 1 on success, 0 if this wasn't a valid MPEG file
 sub ProcessMPEG($$)
 {
-    my ($exifTool, $dirInfo) = @_;
+    my ($et, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
     my $buff;
 
     $raf->Read($buff, 4) == 4 or return 0;
     return 0 unless $buff =~ /^\0\0\x01[\xb0-\xbf]/;
-    $exifTool->SetFileType();
+    $et->SetFileType();
 
     $raf->Seek(0,0);
     $raf->Read($buff, 65536*4) or return 0;
 
-    return ParseMPEGAudioVideo($exifTool, \$buff);
+    return ParseMPEGAudioVideo($et, \$buff);
 }
 
 1;  # end
@@ -703,7 +705,7 @@ based on unofficial sources which may be incomplete, inaccurate or outdated.
 
 =head1 AUTHOR
 
-Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
