@@ -14,46 +14,53 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
+use Image::ExifTool::XMP;
 
-$VERSION = '1.03';
-
-# enable MWG strict mode by default
-# (causes non-standard EXIF, IPTC and XMP to be ignored)
-$Image::ExifTool::MWG::strict = 1 unless defined $Image::ExifTool::MWG::strict;
+$VERSION = '1.14';
 
 sub RecoverTruncatedIPTC($$$);
 sub ListToString($);
 sub StringToList($$);
 sub OverwriteStringList($$$$);
 
+my $mwgLoaded;  # flag set if we alreaded Load()ed the MWG tags
+
 # MWG Composite tags
 %Image::ExifTool::MWG::Composite = (
     GROUPS => { 0 => 'Composite', 1 => 'MWG', 2 => 'Image' },
     VARS => { NO_ID => 1 },
     NOTES => q{
-        This table lists special Composite tags which are used to access other tags
-        based on the Metadata Working Group 1.01 recommendations.  These tags are
-        only accessible when the MWG module is loaded (via C<-use MWG> on the
-        command line, or C<use Image::ExifTool::MWG> in the API).  When reading, the
-        value of each MWG tag is B<Derived From> the specified tags based on the MWG
-        guidelines.  When writing, the value of the IPTCDigest tag is updated
-        automatically when the IPTC is changed if either the IPTCDigest tag didn't
-        exist beforehand or its value agreed with the original IPTC digest
-        (indicating that the XMP is synchronized with the IPTC).  IPTC information
-        is written only if the original file contained IPTC.  For more information
-        about the MWG recommendation, see L<http://www.metadataworkinggroup.org/>.
+        The table below lists special Composite tags which are used to access other
+        tags based on the MWG 2.0 recommendations.  These tags are only accessible
+        when explicitly loaded, but this is done automatically by the exiftool
+        application if MWG is specified as a group for any tag on the command line,
+        or manually with the C<-use MWG> option.  Via the API, the MWG Composite
+        tags are loaded by calling "C<Image::ExifTool::MWG::Load()>".
+
+        When reading, the value of each MWG tag is B<Derived From> the specified
+        tags based on the MWG guidelines.  When writing, the appropriate associated
+        tags are written.  The value of the IPTCDigest tag is updated automatically
+        when the IPTC is changed if either the IPTCDigest tag didn't exist
+        beforehand or its value agreed with the original IPTC digest (indicating
+        that the XMP is synchronized with the IPTC).  IPTC information is written
+        only if the original file contained IPTC.
 
         Loading the MWG module activates "strict MWG conformance mode", which has
         the effect of causing EXIF, IPTC and XMP in non-standard locations to be
         ignored when reading, as per the MWG recommendations.  Instead, a "Warning"
         tag is generated when non-standard metadata is encountered.  This feature
         may be disabled by setting C<$Image::ExifTool::MWG::strict = 0> in the
-        ExifTool config file (or from your Perl script when using the API).  Note
+        L<ExifTool config file|../config.html> (or from your Perl script when using the API).  Note
         that the behaviour when writing is not changed:  ExifTool always creates new
         records only in the standard location, but writes new tags to any
         EXIF/IPTC/XMP records that exist.
 
-        A complication of the specification is that although the MWG:Creator
+        Contrary to the EXIF specification, the MWG recommends that EXIF "ASCII"
+        string values be stored as UTF-8.  To honour this, the exiftool application
+        sets the default internal EXIF string encoding to "UTF8" when the MWG module
+        is loaded (but this setting does not change automatically via the API).
+
+        A complication of the MWG specification is that although the MWG:Creator
         property may consist of multiple values, the associated EXIF tag
         (EXIF:Artist) is only a simple string.  To resolve this discrepancy the MWG
         recommends a technique which allows a list of values to be stored in a
@@ -69,7 +76,7 @@ sub OverwriteStringList($$$$);
             2 => 'CurrentIPTCDigest',
             3 => 'IPTCDigest',
         },
-        ValueConv => q{
+        RawConv => q{
             return $val[1] if not defined $val[2] or (defined $val[1] and
                              (not defined $val[3] or $val[2] eq $val[3]));
             return Image::ExifTool::MWG::RecoverTruncatedIPTC($val[0], $val[1], 64);
@@ -91,8 +98,8 @@ sub OverwriteStringList($$$$);
             3 => 'CurrentIPTCDigest',
             4 => 'IPTCDigest',
         },
-        ValueConv => q{
-            return $val[0] if defined $val[0];
+        RawConv => q{
+            return $val[0] if defined $val[0] and $val[0] !~ /^ *$/;
             return $val[2] if not defined $val[3] or (defined $val[2] and
                              (not defined $val[4] or $val[3] eq $val[4]));
             return Image::ExifTool::MWG::RecoverTruncatedIPTC($val[1], $val[2], 2000);
@@ -119,8 +126,13 @@ sub OverwriteStringList($$$$);
             5 => 'CurrentIPTCDigest',
             6 => 'IPTCDigest',
         },
+        # must check for validity in RawConv to avoid hiding a same-named tag,
+        # but IPTC dates use a ValueConv so we need to derive the value there
+        RawConv => '(defined $val[0] or defined $val[2] or defined $val[4]) ? $val : undef',
         ValueConv => q{
-            return $val[1] ? "$val[0].$val[1]" : $val[0] if defined $val[0];
+            if (defined $val[0] and $val[0] !~ /^[: ]*$/) {
+                return ($val[1] and $val[1] !~ /^ *$/) ? "$val[0].$val[1]" : $val[0];
+            }
             return $val[4] if not defined $val[5] or (defined $val[4] and
                              (not defined $val[6] or $val[5] eq $val[6]));
             return $val[3] ? "$val[2] $val[3]" : $val[2] if $val[2];
@@ -153,8 +165,11 @@ sub OverwriteStringList($$$$);
             5 => 'CurrentIPTCDigest',
             6 => 'IPTCDigest',
         },
+        RawConv => '(defined $val[0] or defined $val[2] or defined $val[4]) ? $val : undef',
         ValueConv => q{
-            return $val[1] ? "$val[0].$val[1]" : $val[0] if defined $val[0];
+            if (defined $val[0] and $val[0] !~ /^[: ]*$/) {
+                return ($val[1] and $val[1] !~ /^ *$/) ? "$val[0].$val[1]" : $val[0];
+            }
             return $val[4] if not defined $val[5] or (defined $val[4] and
                              (not defined $val[6] or $val[5] eq $val[6]));
             return $val[3] ? "$val[2] $val[3]" : $val[2] if $val[2];
@@ -183,8 +198,10 @@ sub OverwriteStringList($$$$);
             3 => 'CurrentIPTCDigest',
             4 => 'IPTCDigest',
         },
-        ValueConv => q{
-            return $val[1] ? "$val[0].$val[1]" : $val[0] if defined $val[0];
+        RawConv => q{
+            if (defined $val[0] and $val[0] !~ /^[: ]*$/) {
+                return ($val[1] and $val[1] !~ /^ *$/) ? "$val[0].$val[1]" : $val[0];
+            }
             return $val[2] if not defined $val[3] or not defined $val[4] or $val[3] eq $val[4];
             return undef;
         },
@@ -231,8 +248,8 @@ sub OverwriteStringList($$$$);
             3 => 'CurrentIPTCDigest',
             4 => 'IPTCDigest',
         },
-        ValueConv => q{
-            return $val[0] if defined $val[0];
+        RawConv => q{
+            return $val[0] if defined $val[0] and $val[0] !~ /^ *$/;
             return $val[2] if not defined $val[3] or (defined $val[2] and
                              (not defined $val[4] or $val[3] eq $val[4]));
             return Image::ExifTool::MWG::RecoverTruncatedIPTC($val[1], $val[2], 128);
@@ -255,8 +272,8 @@ sub OverwriteStringList($$$$);
             3 => 'CurrentIPTCDigest',
             4 => 'IPTCDigest',
         },
-        ValueConv => q{
-            return $val[0] if defined $val[0];
+        RawConv => q{
+            return $val[0] if defined $val[0] and $val[0] !~ /^ *$/;
             return $val[2] if not defined $val[3] or (defined $val[2] and
                              (not defined $val[4] or $val[3] eq $val[4]));
             return Image::ExifTool::MWG::RecoverTruncatedIPTC($val[1], $val[2], 32);
@@ -275,19 +292,22 @@ sub OverwriteStringList($$$$);
         Desire => {
             0 => 'IPTC:Country-PrimaryLocationName', # (64-character limit)
             1 => 'XMP-photoshop:Country',
-            2 => 'CurrentIPTCDigest',
-            3 => 'IPTCDigest',
+            2 => 'XMP-iptcExt:LocationShownCountryName',
+            3 => 'CurrentIPTCDigest',
+            4 => 'IPTCDigest',
         },
-        ValueConv => q{
-            return $val[1] if not defined $val[2] or (defined $val[1] and
-                             (not defined $val[3] or $val[2] eq $val[3]));
-            return Image::ExifTool::MWG::RecoverTruncatedIPTC($val[0], $val[1], 64);
+        RawConv => q{
+            my $xmpVal = $val[2] || $val[1];
+            return $xmpVal if not defined $val[3] or (defined $xmpVal and
+                             (not defined $val[4] or $val[3] eq $val[4]));
+            return Image::ExifTool::MWG::RecoverTruncatedIPTC($val[0], $xmpVal, 64);
         },
         DelCheck   => 'Image::ExifTool::MWG::ReconcileIPTCDigest($self)',
         WriteCheck => 'Image::ExifTool::MWG::ReconcileIPTCDigest($self)',
         WriteAlso  => {
             'IPTC:Country-PrimaryLocationName' => '$opts{EditGroup} = 1; $val',
-            'XMP-photoshop:Country'            => '$val',
+            'XMP-photoshop:Country'            => '$val', # (legacy)
+            'XMP-iptcExt:LocationShownCountryName' => '$val',
         },
     },
     State => {
@@ -296,19 +316,22 @@ sub OverwriteStringList($$$$);
         Desire => {
             0 => 'IPTC:Province-State', # (32-character limit)
             1 => 'XMP-photoshop:State',
-            2 => 'CurrentIPTCDigest',
-            3 => 'IPTCDigest',
+            2 => 'XMP-iptcExt:LocationShownProvinceState',
+            3 => 'CurrentIPTCDigest',
+            4 => 'IPTCDigest',
         },
-        ValueConv => q{
-            return $val[1] if not defined $val[2] or (defined $val[1] and
-                             (not defined $val[3] or $val[2] eq $val[3]));
-            return Image::ExifTool::MWG::RecoverTruncatedIPTC($val[0], $val[1], 32);
+        RawConv => q{
+            my $xmpVal = $val[2] || $val[1];
+            return $xmpVal if not defined $val[3] or (defined $xmpVal and
+                             (not defined $val[4] or $val[3] eq $val[4]));
+            return Image::ExifTool::MWG::RecoverTruncatedIPTC($val[0], $xmpVal, 32);
         },
         DelCheck   => 'Image::ExifTool::MWG::ReconcileIPTCDigest($self)',
         WriteCheck => 'Image::ExifTool::MWG::ReconcileIPTCDigest($self)',
         WriteAlso  => {
             'IPTC:Province-State' => '$opts{EditGroup} = 1; $val',
-            'XMP-photoshop:State' => '$val',
+            'XMP-photoshop:State' => '$val', # (legacy)
+            'XMP-iptcExt:LocationShownProvinceState' => '$val',
         },
     },
     City => {
@@ -317,19 +340,22 @@ sub OverwriteStringList($$$$);
         Desire => {
             0 => 'IPTC:City', # (32-character limit)
             1 => 'XMP-photoshop:City',
-            2 => 'CurrentIPTCDigest',
-            3 => 'IPTCDigest',
+            2 => 'XMP-iptcExt:LocationShownCity',
+            3 => 'CurrentIPTCDigest',
+            4 => 'IPTCDigest',
         },
-        ValueConv => q{
-            return $val[1] if not defined $val[2] or (defined $val[1] and
-                             (not defined $val[3] or $val[2] eq $val[3]));
-            return Image::ExifTool::MWG::RecoverTruncatedIPTC($val[0], $val[1], 32);
+        RawConv => q{
+            my $xmpVal = $val[2] || $val[1];
+            return $xmpVal if not defined $val[3] or (defined $xmpVal and
+                             (not defined $val[4] or $val[3] eq $val[4]));
+            return Image::ExifTool::MWG::RecoverTruncatedIPTC($val[0], $xmpVal, 32);
         },
         DelCheck   => 'Image::ExifTool::MWG::ReconcileIPTCDigest($self)',
         WriteCheck => 'Image::ExifTool::MWG::ReconcileIPTCDigest($self)',
         WriteAlso  => {
             'IPTC:City'          => '$opts{EditGroup} = 1; $val',
-            'XMP-photoshop:City' => '$val',
+            'XMP-photoshop:City' => '$val', # (legacy)
+            'XMP-iptcExt:LocationShownCity' => '$val',
         },
     },
     Location => {
@@ -338,38 +364,190 @@ sub OverwriteStringList($$$$);
         Desire => {
             0 => 'IPTC:Sub-location', # (32-character limit)
             1 => 'XMP-iptcCore:Location',
-            2 => 'CurrentIPTCDigest',
-            3 => 'IPTCDigest',
+            2 => 'XMP-iptcExt:LocationShownSublocation',
+            3 => 'CurrentIPTCDigest',
+            4 => 'IPTCDigest',
         },
-        ValueConv => q{
-            return $val[1] if not defined $val[2] or (defined $val[1] and
-                             (not defined $val[3] or $val[2] eq $val[3]));
-            return Image::ExifTool::MWG::RecoverTruncatedIPTC($val[0], $val[1], 32);
+        RawConv => q{
+            my $xmpVal = $val[2] || $val[1];
+            return $xmpVal if not defined $val[3] or (defined $xmpVal and
+                             (not defined $val[4] or $val[3] eq $val[4]));
+            return Image::ExifTool::MWG::RecoverTruncatedIPTC($val[0], $xmpVal, 32);
         },
         DelCheck   => 'Image::ExifTool::MWG::ReconcileIPTCDigest($self)',
         WriteCheck => 'Image::ExifTool::MWG::ReconcileIPTCDigest($self)',
         WriteAlso  => {
             'IPTC:Sub-location'     => '$opts{EditGroup} = 1; $val',
-            'XMP-iptcCore:Location' => '$val',
+            'XMP-iptcCore:Location' => '$val', # (legacy)
+            'XMP-iptcExt:LocationShownSublocation' => '$val',
         },
     },
 );
 
-unless ($Image::ExifTool::documentOnly) {
+# MWG XMP structures
+my %sExtensions = (
+    STRUCT_NAME => 'MWG Extensions',
+    NAMESPACE   => undef, # variable namespace
+    NOTES => q{
+        This structure may contain any top-level XMP tags, but none have been
+        pre-defined in ExifTool.  Since no flattened tags have been pre-defined,
+        RegionExtensions is writable only as a structure (ie.
+        C<{xmp-dc:creator=me,rating=5}>).  Fields for this structure are identified
+        using the standard ExifTool tag name (with optional leading group name,
+        and/or trailing language code, and/or trailing C<#> symbol to disable print
+        conversion).
+    },
+);
+my %sRegionStruct = (
+    STRUCT_NAME => 'MWG RegionStruct',
+    NAMESPACE   => 'mwg-rs',
+    Area => { Struct => \%Image::ExifTool::XMP::sArea },
+    Type => {
+        PrintConv => {
+            Face => 'Face',
+            Pet => 'Pet',
+            Focus => 'Focus',
+            BarCode => 'BarCode',
+        },
+    },
+    Name        => { },
+    Description => { },
+    FocusUsage  => {
+        PrintConv => {
+            EvaluatedUsed => 'Evaluated, Used',
+            EvaluatedNotUsed => 'Evaluated, Not Used',
+            NotEvaluatedNotUsed => 'Not Evaluated, Not Used',
+        },
+    },
+    BarCodeValue=> { },
+    Extensions  => { Struct => \%sExtensions },
+    seeAlso => { Namespace => 'rdfs', Resource => 1 },
+);
+my %sKeywordStruct;
+%sKeywordStruct = (
+    STRUCT_NAME => 'MWG KeywordStruct',
+    NAMESPACE   => 'mwg-kw',
+    Keyword   => { },
+    Applied   => { Writable => 'boolean' },
+    Children  => { Struct => \%sKeywordStruct, List => 'Bag' },
+);
+
+# MWG 2.0 XMP region namespace tags
+%Image::ExifTool::MWG::Regions = (
+    %Image::ExifTool::XMP::xmpTableDefaults,
+    GROUPS => { 0 => 'XMP', 1 => 'XMP-mwg-rs', 2 => 'Image' },
+    NAMESPACE => 'mwg-rs',
+    NOTES => q{
+        Image region metadata defined by the MWG 2.0 specification.  These tags
+        belong to the ExifTool XMP-mwg-rs group, and as such they may be accessed
+        without the need to load the MWG Composite tags above.  See
+        L<http://www.metadataworkinggroup.org/> for the official specification.
+    },
+    Regions => {
+        Name => 'RegionInfo',
+        FlatName => 'Region',
+        Struct => {
+            STRUCT_NAME => 'MWG RegionInfo',
+            NAMESPACE   => 'mwg-rs',
+            RegionList => {
+                FlatName => 'Region',
+                Struct => \%sRegionStruct,
+                List => 'Bag',
+            },
+            AppliedToDimensions => { Struct => \%Image::ExifTool::XMP::sDimensions },
+        },
+    },
+    RegionsRegionList => { Flat => 1, Name => 'RegionList' },
+);
+
+# MWG 2.0 XMP hierarchical keyword namespace tags
+%Image::ExifTool::MWG::Keywords = (
+    %Image::ExifTool::XMP::xmpTableDefaults,
+    GROUPS => { 0 => 'XMP', 1 => 'XMP-mwg-kw', 2 => 'Image' },
+    NAMESPACE => 'mwg-kw',
+    NOTES => q{
+        Hierarchical keywords metadata defined by the MWG 2.0 specification.  These
+        tags belong to the ExifTool XMP-mwg-kw group. ExifTool unrolls keyword
+        structures to an arbitrary depth of 6 to allow individual levels to be
+        accessed with different tag names, and to avoid infinite recursion.  See
+        L<http://www.metadataworkinggroup.org/> for the official specification.
+    },
+    # arbitrarily define only the first 6 levels of the keyword hierarchy
+    Keywords => {
+        Name => 'KeywordInfo',
+        Struct => {
+            STRUCT_NAME => 'MWG KeywordInfo',
+            NAMESPACE   => 'mwg-kw',
+            Hierarchy => { Struct => \%sKeywordStruct, List => 'Bag' },
+        },
+    },
+    KeywordsHierarchy => { Name => 'HierarchicalKeywords', Flat => 1 },
+    KeywordsHierarchyKeyword  => { Name => 'HierarchicalKeywords1', Flat => 1 },
+    KeywordsHierarchyApplied  => { Name => 'HierarchicalKeywords1Applied', Flat => 1 },
+    KeywordsHierarchyChildren => { Name => 'HierarchicalKeywords1Children', Flat => 1 },
+    KeywordsHierarchyChildrenKeyword  => { Name => 'HierarchicalKeywords2', Flat => 1 },
+    KeywordsHierarchyChildrenApplied  => { Name => 'HierarchicalKeywords2Applied', Flat => 1 },
+    KeywordsHierarchyChildrenChildren => { Name => 'HierarchicalKeywords2Children', Flat => 1 },
+    KeywordsHierarchyChildrenChildrenKeyword  => { Name => 'HierarchicalKeywords3', Flat => 1 },
+    KeywordsHierarchyChildrenChildrenApplied  => { Name => 'HierarchicalKeywords3Applied', Flat => 1 },
+    KeywordsHierarchyChildrenChildrenChildren => { Name => 'HierarchicalKeywords3Children', Flat => 1 },
+    KeywordsHierarchyChildrenChildrenChildrenKeyword  => { Name => 'HierarchicalKeywords4', Flat => 1 },
+    KeywordsHierarchyChildrenChildrenChildrenApplied  => { Name => 'HierarchicalKeywords4Applied', Flat => 1 },
+    KeywordsHierarchyChildrenChildrenChildrenChildren => { Name => 'HierarchicalKeywords4Children', Flat => 1 },
+    KeywordsHierarchyChildrenChildrenChildrenChildrenKeyword  => { Name => 'HierarchicalKeywords5', Flat => 1 },
+    KeywordsHierarchyChildrenChildrenChildrenChildrenApplied  => { Name => 'HierarchicalKeywords5Applied', Flat => 1 },
+    KeywordsHierarchyChildrenChildrenChildrenChildrenChildren => { Name => 'HierarchicalKeywords5Children', Flat => 1, NoSubStruct => 1 }, # break infinite recursion
+    KeywordsHierarchyChildrenChildrenChildrenChildrenChildrenKeyword => { Name => 'HierarchicalKeywords6', Flat => 1 },
+    KeywordsHierarchyChildrenChildrenChildrenChildrenChildrenApplied => { Name => 'HierarchicalKeywords6Applied', Flat => 1 },
+);
+
+# MWG 2.0 XMP collections namespace tags
+%Image::ExifTool::MWG::Collections = (
+    %Image::ExifTool::XMP::xmpTableDefaults,
+    GROUPS => { 0 => 'XMP', 1 => 'XMP-mwg-coll', 2 => 'Image' },
+    NAMESPACE => 'mwg-coll',
+    NOTES => q{
+        Collections metadata defined by the MWG 2.0 specification.  These tags
+        belong to the ExifTool XMP-mwg-coll group.  See
+        L<http://www.metadataworkinggroup.org/> for the official specification.
+    },
+    Collections => {
+        FlatName => '',
+        List => 'Bag',
+        Struct => {
+            STRUCT_NAME => 'MWG CollectionInfo',
+            NAMESPACE   => 'mwg-coll',
+            CollectionName => { },
+            CollectionURI  => { },
+        },
+    },
+);
+
+
+#------------------------------------------------------------------------------
+# Load the MWG Composite tags
+sub Load()
+{
+    return if $mwgLoaded;
+
     # add our composite tags
     Image::ExifTool::AddCompositeTags('Image::ExifTool::MWG');
     # must also add to lookup so we can write them
     # (since MWG tags aren't in the tag lookup by default)
     Image::ExifTool::AddTagsToLookup(\%Image::ExifTool::MWG::Composite,
                                      'Image::ExifTool::Composite');
-}
 
-# modify EXIF:Artist to behave as a List-type tag
-{
+    # modify EXIF:Artist to behave as a list-type tag
     my $artist = $Image::ExifTool::Exif::Main{0x13b};
     $$artist{List} = 1;
     $$artist{IsOverwriting} = \&OverwriteStringList;
     $$artist{RawConv} = \&StringToList;
+
+    # enable MWG strict mode if not set already
+    # (causes non-standard EXIF, IPTC and XMP to be ignored)
+    $Image::ExifTool::MWG::strict = 1 unless defined $Image::ExifTool::MWG::strict;
+
+    $mwgLoaded = 1;
 }
 
 #------------------------------------------------------------------------------
@@ -397,7 +575,7 @@ sub ListToString($)
 # Notes: Sets Warning tag on error
 sub StringToList($$)
 {
-    my ($str, $exifTool) = @_;
+    my ($str, $et) = @_;
     my (@vals, $inQuotes);
     my @t = split '; ', $str, -1;
     foreach (@t) {
@@ -416,34 +594,43 @@ sub StringToList($$)
             push @vals, $_;
         }
     }
-    $exifTool->Warn('Incorrectly quoted MWG string-list value') if $inQuotes;
+    $et->Warn('Incorrectly quoted MWG string-list value') if $inQuotes;
     return @vals > 1 ? \@vals : $vals[0];
 }
 
 #------------------------------------------------------------------------------
 # Handle logic for overwriting EXIF string-type list tag
-# Inputs: 0) new value hash ref, 1) old string value (or undef if it didn't exist),
-#         2) reference to new value
+# Inputs: 0) new value hash ref, 1) new value hash ref,
+#         2) old string value (or undef if it didn't exist), 3) new value ref
 # Returns: 1 and sets the new value for the tag
 sub OverwriteStringList($$$$)
 {
     local $_;
-    my ($exifTool, $nvHash, $val, $newValuePt) = @_;
-    my @new;
+    my ($et, $nvHash, $val, $newValuePt) = @_;
+    my (@new, $delIndex);
     if ($$nvHash{DelValue} and defined $val) {
         # preserve specified old values
-        my $old = StringToList($val, $exifTool);
+        my $old = StringToList($val, $et);
         my @old = ref $old eq 'ARRAY' ? @$old : $old;
         if (@{$$nvHash{DelValue}}) {
             my %del;
             $del{$_} = 1 foreach @{$$nvHash{DelValue}};
-            $del{$_} or push(@new, $_) foreach @old;
+            foreach (@old) {
+                $del{$_} or push(@new, $_), next;
+                $delIndex or $delIndex = scalar @new;
+            }
         } else {
             push @new, @old;
         }
     }
-    # add new values
-    push @new, @{$$nvHash{Value}} if $$nvHash{Value};
+    # add new values (at location of deleted values, if any)
+    if ($$nvHash{Value}) {
+        if (defined $delIndex) {
+            splice @new, $delIndex, 0, @{$$nvHash{Value}};
+        } else {
+            push @new, @{$$nvHash{Value}};
+        }
+    }
     if (@new) {
         # convert back to string format
         $$newValuePt = ListToString(\@new);
@@ -459,17 +646,17 @@ sub OverwriteStringList($$$$)
 # Returns: empty string
 sub ReconcileIPTCDigest($)
 {
-    my $exifTool = shift;
+    my $et = shift;
 
     # set new value for IPTCDigest if not done already
     unless ($Image::ExifTool::Photoshop::iptcDigestInfo and
-            $exifTool->{NEW_VALUE}{$Image::ExifTool::Photoshop::iptcDigestInfo})
+            $$et{NEW_VALUE}{$Image::ExifTool::Photoshop::iptcDigestInfo})
     {
         # write new IPTCDigest only if it doesn't exist or
         # is the same as the digest of the original IPTC
         my @a; # (capture warning messages)
-        @a = $exifTool->SetNewValue('Photoshop:IPTCDigest', 'old', Protected => 1, DelValue => 1);
-        @a = $exifTool->SetNewValue('Photoshop:IPTCDigest', 'new', Protected => 1);
+        @a = $et->SetNewValue('Photoshop:IPTCDigest', 'old', Protected => 1, DelValue => 1);
+        @a = $et->SetNewValue('Photoshop:IPTCDigest', 'new', Protected => 1);
     }
     return '';
 }
@@ -507,31 +694,40 @@ Image::ExifTool::MWG - Metadata Working Group support
 
 =head1 SYNOPSIS
 
-    # enable MWG tags (strict mode enabled by default)
+    # enable MWG Composite tags
     use Image::ExifTool::MWG;
+    Image::ExifTool::MWG::Load();
+
+    # enable MWG strict mode
+    $Image::ExifTool::MWG::strict = 1;
 
     # disable MWG strict mode
     $Image::ExifTool::MWG::strict = 0;
 
 =head1 DESCRIPTION
 
-The MWG module contains tag definitions which are designed to simplify
-implementation of the Metadata Working Group guidelines.  These special MWG
-composite tags are enabled simply by loading this module:
+The MWG module contains Composite tag definitions which are designed to
+simplify implementation of the Metadata Working Group guidelines.  These
+special MWG Composite tags are enabled by calling the Load() method:
 
     use Image::ExifTool::MWG;
+    Image::ExifTool::MWG::Load();
 
-When the MWG module is loaded, "strict MWG conformance" is enabled by
-default.  In this mode, ExifTool will generate a Warning tag instead of
-extracting EXIF, IPTC and XMP from non-standard locations.  The strict mode
-may be disabled by setting the MWG "strict" flag to zero (either before or
-after loading the MWG module):
+By default, loading the MWG Composite tags enables "strict MWG conformance"
+unless previously enabled or disabled by the user.  In this mode, ExifTool
+will generate a Warning instead of extracting EXIF, IPTC and XMP from
+non-standard locations.  The strict mode may be disabled or enabled at any
+time by setting the MWG "strict" flag to 0 or 1.  ie)
 
     $Image::ExifTool::MWG::strict = 0;
 
+This module also contains the MWG XMP tags which are loaded automatically by
+ExifTool as required, and are independent of the MWG Composite tags which
+must be loaded explicitly as described above.
+
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
