@@ -20,7 +20,7 @@ sub ProcessGE2($$$);
 sub WriteUnknownOrPreview($$$);
 sub FixLeicaBase($$;$);
 
-$VERSION = '1.67';
+$VERSION = '1.87';
 
 my $debug;          # set to 1 to enable debugging code
 
@@ -32,6 +32,29 @@ my $debug;          # set to 1 to enable debugging code
 # - Put these in alphabetical order to make TagNames documentation nicer.
 @Image::ExifTool::MakerNotes::Main = (
     # decide which MakerNotes to use (based on camera make/model)
+    {
+        Name => 'MakerNoteApple',
+        Condition => '$$valPt =~ /^Apple iOS\0/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Apple::Main',
+            ByteOrder => 'Unknown',
+            Start => '$valuePtr + 14',
+            Base => '$start - 14',
+        },
+    },
+    {
+        # this maker notes starts with a standard TIFF header at offset 0x0a
+        # (must check Nikon signature first because Nikon Capture NX can generate
+        #  NEF images containing Nikon maker notes from JPEG images of any camera model)
+        Name => 'MakerNoteNikon',
+        Condition => '$$valPt=~/^Nikon\x00\x02/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Nikon::Main',
+            Start => '$valuePtr + 18',
+            Base => '$start - 8',
+            ByteOrder => 'Unknown',
+        },
+    },
     {
         Name => 'MakerNoteCanon',
         # (starts with an IFD)
@@ -65,6 +88,16 @@ my $debug;          # set to 1 to enable debugging code
             FixBase => 1, # necessary for AVI and MOV videos
         },
     },
+    {
+        Name => 'MakerNoteFLIR',
+        # (starts with IFD, Make is 'FLIR Systems AB' or 'FLIR Systems')
+        Condition => '$$self{Make} =~ /^FLIR Systems/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::FLIR::Main',
+            Start => '$valuePtr',
+            ByteOrder => 'Unknown',
+        },
+    },
     {
         # The Fuji maker notes use a structure similar to a self-contained
         # TIFF file, but with "FUJIFILM" instead of the standard TIFF header
@@ -108,6 +141,16 @@ my $debug;          # set to 1 to enable debugging code
             FixOffsets => '$valuePtr -= 210 if $tagID >= 0x1303',
        },
     },
+    {
+        Name => 'MakerNoteHasselblad',
+        Condition => '$$self{Make} eq "Hasselblad"',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Unknown::Main',
+            ByteOrder => 'Unknown',
+            Start => '$valuePtr',
+            Base => 0, # (avoids warnings since maker notes are not self-contained)
+        },
+    },
     # (the GE X5 has really messed up EXIF-like maker notes starting with
     #  "GENIC\x0c\0" --> currently not decoded)
     {
@@ -402,21 +445,10 @@ my $debug;          # set to 1 to enable debugging code
         Binary => 1,
         Notes => 'not EXIF-based',
     },
-    {
-        # this maker notes starts with a standard TIFF header at offset 0x0a
-        Name => 'MakerNoteNikon',
-        Condition => '$$self{Make}=~/^NIKON/i and $$valPt=~/^Nikon\x00\x02/',
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Nikon::Main',
-            Start => '$valuePtr + 18',
-            Base => '$start - 8',
-            ByteOrder => 'Unknown',
-        },
-    },
     {
         # older Nikon maker notes
         Name => 'MakerNoteNikon2',
-        Condition => '$$self{Make}=~/^NIKON/ and $$valPt=~/^Nikon\x00\x01/',
+        Condition => '$$valPt=~/^Nikon\x00\x01/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Nikon::Type2',
             Start => '$valuePtr + 8',
@@ -484,7 +516,10 @@ my $debug;          # set to 1 to enable debugging code
     {
         Name => 'MakerNoteLeica3', # used by the R8 and R9
         # (starts with IFD)
-        Condition => '$$self{Make} =~ /^Leica Camera AG/ and $$valPt !~ /^LEICA/ and $$self{Model} ne "S2"',
+        Condition => q{
+            $$self{Make} =~ /^Leica Camera AG/ and $$valPt !~ /^LEICA/ and
+            $$self{Model} ne "S2" and $$self{Model} ne "LEICA M (Typ 240)"
+        },
         SubDirectory => {
             TagTable => 'Image::ExifTool::Panasonic::Leica3',
             Start => '$valuePtr',
@@ -492,8 +527,8 @@ my $debug;          # set to 1 to enable debugging code
         },
     },
     {
-        Name => 'MakerNoteLeica4', # used by the M9
-        # (M9 starts with "LEICA0\x03\0")
+        Name => 'MakerNoteLeica4', # used by the M9/M-Monochrom
+        # (M9 and M Monochrom start with "LEICA0\x03\0")
         Condition => '$$self{Make} =~ /^Leica Camera AG/ and $$valPt =~ /^LEICA0/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Panasonic::Leica4',
@@ -503,9 +538,11 @@ my $debug;          # set to 1 to enable debugging code
         },
     },
     {
-        Name => 'MakerNoteLeica5', # used by the X1
+        Name => 'MakerNoteLeica5', # used by the X1/X2
         # (X1 starts with "LEICA\0\x01\0", Make is "LEICA CAMERA AG")
-        Condition => '$$valPt =~ /^LEICA\0\x01\0/',
+        # (X2 starts with "LEICA\0\x05\0", Make is "LEICA CAMERA AG")
+        # (X VARIO starts with "LEICA\0\x04\0", Make is "LEICA CAMERA AG")
+        Condition => '$$valPt =~ /^LEICA\0[\x01\x04\x05]\0/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Panasonic::Leica5',
             Start => '$valuePtr + 8',
@@ -514,11 +551,12 @@ my $debug;          # set to 1 to enable debugging code
         },
     },
     {
-        Name => 'MakerNoteLeica6', # used by the S2 (CAUTION: this tag name is special cased in the code)
-        # (S2 starts with "LEICA\0\x02\xff", Make is "LEICA CAMERA AG",
+        Name => 'MakerNoteLeica6', # used by the S2 and M (Typ 240)
+        # (starts with "LEICA\0\x02\xff", Make is "Leica Camera AG",
         #  but maker notes aren't loaded at the time this is tested)
-        Condition => '$$self{Model} eq "S2"',
+        Condition => '$$self{Model} eq "S2" or $$self{Model} eq "LEICA M (Typ 240)"',
         DataTag => 'LeicaTrailer',  # (generates fixup name for this tag)
+        LeicaTrailer => 1, # flag to special-case this tag in the Exif code
         SubDirectory => {
             TagTable => 'Image::ExifTool::Panasonic::Leica6',
             Start => '$valuePtr + 8',
@@ -526,7 +564,7 @@ my $debug;          # set to 1 to enable debugging code
             # NOTE: Leica uses absolute file offsets when this maker note is stored
             # as a JPEG trailer -- this case is handled by ProcessLeicaTrailer in
             # Panasonic.pm, and any "Base" defined here is ignored for this case.
-            # ExifTool may also create S2 maker notes inside the APP1 segment when
+            # ExifTool may also create S2/M maker notes inside the APP1 segment when
             # copying from other files, and for this the normal EXIF offsets are used,
             # Base should not be defined!
         },
@@ -608,6 +646,7 @@ my $debug;          # set to 1 to enable debugging code
     {
         Name => 'MakerNotePentax5',
         # (starts with "PENTAX \0")
+        # used by cameras such as the Q, Optio  S1, RS1500 and WG-1
         Condition => '$$valPt=~/^PENTAX \0/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Pentax::Main',
@@ -637,13 +676,15 @@ my $debug;          # set to 1 to enable debugging code
             return 1;
         },
         NotIFD => 1,
-        Binary => 1,
+        SubDirectory => { TagTable => 'Image::ExifTool::PhaseOne::Main' },
         PutFirst => 1, # place immediately after TIFF header
-        Notes => 'the raw image data in PhaseOne IIQ images',
     },
     {
         Name => 'MakerNoteReconyx',
-        Condition => '$$valPt =~ /^\x01\xf1[\x02\x03]\x00/',
+        Condition => q{
+            $$valPt =~ /^\x01\xf1([\x02\x03]\x00)?/ and
+            ($1 or $$self{Make} eq "RECONYX")
+        },
         SubDirectory => {
             TagTable => 'Image::ExifTool::Reconyx::Main',
             ByteOrder => 'Little-endian',
@@ -652,7 +693,9 @@ my $debug;          # set to 1 to enable debugging code
     {
         Name => 'MakerNoteRicoh',
         # (my test R50 image starts with "      \x02\x01" - PH)
-        Condition => '$$self{Make}=~/^RICOH/ and $$valPt=~/^(Ricoh|      )/i',
+        # (the HZ15 starts with "MM\0\x2a" but an extra 2 bytes of padding after
+        #  the IFD entry count prevents these from being processed as a standard IFD)
+        Condition => '$$self{Make}=~/^(PENTAX )?RICOH/ and $$valPt=~/^(Ricoh|      |MM\0\x2a|II\x2a\0)/i',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Ricoh::Main',
             Start => '$valuePtr + 8',
@@ -748,7 +791,8 @@ my $debug;          # set to 1 to enable debugging code
     {
         Name => 'MakerNoteSony',
         # (starts with "SONY DSC \0" or "SONY CAM \0")
-        Condition => '$$self{Make}=~/^SONY/ and $$valPt=~/^SONY (DSC|CAM)/',
+        # (TF1 starts with "\0\0SONY PIC\0")
+        Condition => '$$valPt=~/^SONY (DSC|CAM)/ or $$valPt=~/^\0\0SONY PIC\0/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Sony::Main',
             Start => '$valuePtr + 12',
@@ -758,7 +802,7 @@ my $debug;          # set to 1 to enable debugging code
     {
         Name => 'MakerNoteSony2',
         # (starts with "SONY PI\0" -- DSC-S650/S700/S750)
-        Condition => '$$self{Make}=~/^SONY/ and $$valPt=~/^SONY PI\0/ and $$self{OlympusCAMER}=1',
+        Condition => '$$valPt=~/^SONY PI\0/ and $$self{OlympusCAMER}=1',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Olympus::Main',
             Start => '$valuePtr + 12',
@@ -768,7 +812,7 @@ my $debug;          # set to 1 to enable debugging code
     {
         Name => 'MakerNoteSony3',
         # (starts with "PREMI\0" -- DSC-S45/S500)
-        Condition => '$$self{Make}=~/^SONY/ and $$valPt=~/^(PREMI)\0/ and $$self{OlympusCAMER}=1',
+        Condition => '$$valPt=~/^(PREMI)\0/ and $$self{OlympusCAMER}=1',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Olympus::Main',
             Start => '$valuePtr + 8',
@@ -776,7 +820,13 @@ my $debug;          # set to 1 to enable debugging code
         },
     },
     {
-        Name => 'MakerNoteSony4', # used in SR2 and ARW images
+        Name => 'MakerNoteSony4',
+        # (starts with "SONY PIC\0" -- DSC-H200/J20/W370/W510, MHS-TS20)
+        Condition => '$$valPt=~/^SONY PIC\0/',
+        SubDirectory => { TagTable => 'Image::ExifTool::Sony::PIC' },
+    },    
+    {
+        Name => 'MakerNoteSony5', # used in SR2 and ARW images
         Condition => '$$self{Make}=~/^SONY/ and $$valPt!~/^\x01\x00/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Sony::Main',
@@ -811,6 +861,13 @@ my $debug;          # set to 1 to enable debugging code
         ValueConv => 'length($val) > 64 ? \$val : $val',
         ValueConvInv => '$val',
     },
+    {
+        Name => 'MakerNoteUnknownBinary',
+        # "LSI1\0" - SilverFast
+        Condition => '$$valPt =~ /^LSI1\0/',
+        Notes => 'unknown binary maker notes',
+        Binary => 1,
+    },
     {
         Name => 'MakerNoteUnknown',
         PossiblePreview => 1,
@@ -828,6 +885,7 @@ my $debug;          # set to 1 to enable debugging code
 my $tagInfo;
 foreach $tagInfo (@Image::ExifTool::MakerNotes::Main) {
     $$tagInfo{Writable} = 'undef';
+    $$tagInfo{Format} = 'undef', # (make sure we don't convert this when reading)
     $$tagInfo{WriteGroup} = 'ExifIFD';
     $$tagInfo{Groups} = { 1 => 'MakerNotes' };
     next unless $$tagInfo{SubDirectory};
@@ -845,10 +903,10 @@ foreach $tagInfo (@Image::ExifTool::MakerNotes::Main) {
 # Notes: Directory size should be validated before calling this routine
 sub GetMakerNoteOffset($)
 {
-    my $exifTool = shift;
+    my $et = shift;
     # figure out where we expect the value data based on camera type
-    my $make = $exifTool->{Make};
-    my $model = $exifTool->{Model};
+    my $make = $$et{Make};
+    my $model = $$et{Model};
     my ($relative, @offsets);
 
     # normally value data starts 4 bytes after end of directory, so this is the default.
@@ -865,7 +923,7 @@ sub GetMakerNoteOffset($)
         # Casio AVI and MOV images use no padding, and their JPEG's use 4,
         # except some models like the EX-S770,Z65,Z70,Z75 and Z700 which use 16,
         # and the EX-Z35 which uses 2 (grrrr...)
-        push @offsets, $$exifTool{FILE_TYPE} =~ /^(RIFF|MOV)$/ ? 0 : (4, 16, 2);
+        push @offsets, $$et{FILE_TYPE} =~ /^(RIFF|MOV)$/ ? 0 : (4, 16, 2);
     } elsif ($make =~ /^(General Imaging Co.|GEDSC IMAGING CORP.)/i) {
         push @offsets, 0;
     } elsif ($make =~ /^KYOCERA/) {
@@ -873,7 +931,9 @@ sub GetMakerNoteOffset($)
     } elsif ($make =~ /^Leica Camera AG/) {
         if ($model eq 'S2') {
             # lots of empty space before first value in S2 images
-            push @offsets, 4, ($$exifTool{FILE_TYPE} eq 'JPEG' ? 286 : 274);
+            push @offsets, 4, ($$et{FILE_TYPE} eq 'JPEG' ? 286 : 274);
+        } elsif ($model eq 'LEICA M (Typ 240)') {
+            push @offsets, 4, 118;
         } elsif ($model =~ /^(R8|R9|M8)\b/) {
             push @offsets, 6;
         } else {
@@ -889,8 +949,10 @@ sub GetMakerNoteOffset($)
     } elsif ($make =~ /^(Panasonic|JVC)\b/) {
         push @offsets, 0;
     } elsif ($make =~ /^SONY/) {
-        # DSLR and "PREMI" models use an offset of 4
-        if ($model =~ /DSLR/ or $$exifTool{OlympusCAMER}) {
+        # earlier DSLR and "PREMI" models use an offset of 4
+        if ($model =~ /^(DSLR-.*|SLT-A(33|35|55V)|NEX-(3|5|C3|VG10E))$/ or
+            $$et{OlympusCAMER})
+        {
             push @offsets, 4;
         } else {
             push @offsets, 0;
@@ -971,7 +1033,7 @@ sub GetValueBlocks($$;$)
 sub FixBase($$)
 {
     local $_;
-    my ($exifTool, $dirInfo) = @_;
+    my ($et, $dirInfo) = @_;
     # don't fix base if fixing offsets individually or if we don't want to fix them
     return 0 if $$dirInfo{FixOffsets} or $$dirInfo{NoFixBase};
 
@@ -980,7 +1042,7 @@ sub FixBase($$)
     my $dirStart = $$dirInfo{DirStart} || 0;
     my $entryBased = $$dirInfo{EntryBased};
     my $dirName = $$dirInfo{DirName};
-    my $fixBase = $exifTool->Options('FixBase');
+    my $fixBase = $et->Options('FixBase');
     my $setBase = (defined $fixBase and $fixBase ne '') ? 1 : 0;
     my ($fix, $fixedBy, %tagPtr);
 
@@ -992,7 +1054,7 @@ sub FixBase($$)
 #
 # handle special case of Canon maker notes with TIFF footer containing original offset
 #
-    if ($$exifTool{Make} =~ /^Canon/ and $$dirInfo{DirLen} > 8) {
+    if ($$et{Make} =~ /^Canon/ and $$dirInfo{DirLen} > 8) {
         my $footerPos = $dirStart + $$dirInfo{DirLen} - 8;
         my $footer = substr($$dataPt, $footerPos, 8);
         if ($footer =~ /^(II\x2a\0|MM\0\x2a)/ and  # check for TIFF footer
@@ -1014,11 +1076,11 @@ sub FixBase($$)
                 # ignore footer offset only if end difference is exactly correct
                 # (allow for possible padding byte, although I have never seen this)
                 if (not $endDiff or $endDiff == 1) {
-                    $exifTool->Warn('Canon maker note footer may be invalid (ignored)',1);
+                    $et->Warn('Canon maker note footer may be invalid (ignored)',1);
                     return 0;
                 }
             }
-            $exifTool->Warn("Adjusted $dirName base by $fix",1);
+            $et->Warn("Adjusted $dirName base by $fix",1);
             $$dirInfo{FixedBy} = $fix;
             $$dirInfo{Base} += $fix;
             $$dirInfo{DataPos} -= $fix;
@@ -1033,9 +1095,9 @@ sub FixBase($$)
     my $minPt = $$dirInfo{MinOffset} = $valPtrs[0]; # if life were simple, this would be it
     my $ifdLen = 2 + 12 * Get16u($$dirInfo{DataPt}, $dirStart);
     my $ifdEnd = $dirStart + $ifdLen;
-    my ($relative, @offsets) = GetMakerNoteOffset($exifTool);
+    my ($relative, @offsets) = GetMakerNoteOffset($et);
     my $makeDiff = $offsets[0];
-    my $verbose = $exifTool->Options('Verbose');
+    my $verbose = $et->Options('Verbose');
     my ($diff, $shift);
 
     # calculate expected minimum value offset
@@ -1076,14 +1138,14 @@ sub FixBase($$)
         # looks like these offsets are entry-based, so use the offsets
         # which have been correcting for individual entry position
         $entryBased = 1;
-        $verbose and $exifTool->Warn("$dirName offsets are entry-based",1);
+        $verbose and $et->Warn("$dirName offsets are entry-based",1);
     } else {
         # calculate offset difference from end of IFD to first value
         $diff = ($minPt - $dataPos) - $ifdEnd;
         $shift = 0;
-        $countOverlap and $exifTool->Warn("Overlapping $dirName values",1);
+        $countOverlap and $et->Warn("Overlapping $dirName values",1);
         if ($entryBased) {
-            $exifTool->Warn("$dirName offsets do NOT look entry-based",1);
+            $et->Warn("$dirName offsets do NOT look entry-based",1);
             undef $entryBased;
             undef $relative;
         }
@@ -1097,8 +1159,9 @@ sub FixBase($$)
         if ($$dirInfo{FixBase} and $$dirInfo{FixBase} == 2) {
             return 0 if $diff >=0 and $diff <= 24;
         }
-        # (used for testing to extract differences)
-        # $exifTool->FoundTag('Diff', $diff);
+        # ******** (used for testing to extract differences) ********
+        # $et->FoundTag('Diff', $diff);
+        # $et->FoundTag('MakeDiff',$makeDiff);
     }
 #
 # handle entry-based offsets
@@ -1157,13 +1220,13 @@ sub FixBase($$)
     } else {
         # print warning unless difference looks reasonable
         if ($diff < 0 or $diff > 16 or ($diff & 0x01)) {
-            $exifTool->Warn("Possibly incorrect maker notes offsets (fix by $fix?)",1);
+            $et->Warn("Possibly incorrect maker notes offsets (fix by $fix?)",1);
         }
         # don't do the fix (but we already adjusted base if entry-based)
         return $shift;
     }
     if (defined $fixedBy) {
-        $exifTool->Warn("Adjusted $dirName base by $fixedBy",1);
+        $et->Warn("Adjusted $dirName base by $fixedBy",1);
         $$dirInfo{FixedBy} = $fixedBy;
     }
     $$dirInfo{Base} += $fix;
@@ -1181,7 +1244,7 @@ sub FixBase($$)
 # Note: Changes byte ordering!
 sub LocateIFD($$)
 {
-    my ($exifTool, $dirInfo) = @_;
+    my ($et, $dirInfo) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $dirStart = $$dirInfo{DirStart} || 0;
     # (ignore MakerNotes DirLen since sometimes this is incorrect)
@@ -1230,7 +1293,7 @@ sub LocateIFD($$)
                     if ($$subdir{ProcessProc} and $$subdir{ProcessProc} eq \&FixLeicaBase) {
                         my $oldStart = $$dirInfo{DirStart};
                         $$dirInfo{DirStart} = $newStart;
-                        FixLeicaBase($exifTool, $dirInfo);
+                        FixLeicaBase($et, $dirInfo);
                         $$dirInfo{DirStart} = $oldStart;
                     }
                 }
@@ -1308,11 +1371,21 @@ IFD_TRY: for ($offset=$firstTry; $offset<=$lastTry; $offset+=2) {
                 my $entry = $pos + 2 + 12 * $index;
                 my $format = Get16u($dataPt, $entry+2);
                 my $count = Get32u($dataPt, $entry+4);
-                # allow everything to be zero if not first entry
-                # because some manufacturers pad with null entries
-                next unless $format or $count or $index == 0;
-                # patch for Canon EOS 40D firmware 1.0.4 bug: allow zero format for last entry
-                next if $format==0 and $index==$num-1 and $$exifTool{Model}=~/EOS 40D/;
+                unless ($format) {
+                    # patch for buggy Samsung NX200 JPEG MakerNotes entry count
+                    if ($num == 23 and $index == 21 and $$et{Make} eq 'SAMSUNG') {
+                        Set16u(21, $dataPt, $pos);  # really 21 IFD entries!
+                        $et->Warn('Fixed incorrect Makernote entry count', 1);
+                        last;
+                    }
+                    # allow everything to be zero if not first entry
+                    # because some manufacturers pad with null entries
+                    next unless $count or $index == 0;
+                    # patch for Canon EOS 40D firmware 1.0.4 bug: allow zero format for last entry
+                    next if $index==$num-1 and $$et{Model}=~/EOS 40D/;
+                }
+                # patch for Sony cameras like the DSC-P10 that have invalid MakerNote entries
+                next if $num == 12 and $$et{Make} eq 'SONY' and $index >= 8;
                 # (would like to verify tag ID, but some manufactures don't
                 #  sort entries in order of tag ID so we don't have much of
                 #  a handle to verify this field)
@@ -1344,7 +1417,7 @@ IFD_TRY: for ($offset=$firstTry; $offset<=$lastTry; $offset+=2) {
 # Returns: 1 on success, and updates $dirInfo if necessary for new directory
 sub FixLeicaBase($$;$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $dirStart = $$dirInfo{DirStart} || 0;
     # get hash of value block positions
@@ -1361,7 +1434,7 @@ sub FixLeicaBase($$;$)
     }
     my $success = 1;
     if ($tagTablePtr) {
-        $success = Image::ExifTool::Exif::ProcessExif($exifTool, $dirInfo, $tagTablePtr);
+        $success = Image::ExifTool::Exif::ProcessExif($et, $dirInfo, $tagTablePtr);
     }
     return $success;
 }
@@ -1372,10 +1445,10 @@ sub FixLeicaBase($$;$)
 # Returns: 1 on success
 sub ProcessCanon($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     # identify Canon MakerNote footer in HtmlDump
     # (this code moved from FixBase so it also works for Adobe MakN in DNG images)
-    if ($$exifTool{HTML_DUMP} and $$dirInfo{DirLen} > 8) {
+    if ($$et{HTML_DUMP} and $$dirInfo{DirLen} > 8) {
         my $dataPos = $$dirInfo{DataPos};
         my $dirStart = $$dirInfo{DirStart} || 0;
         my $footerPos = $dirStart + $$dirInfo{DirLen} - 8;
@@ -1388,11 +1461,11 @@ sub ProcessCanon($$$)
                 $str .= sprintf("\nCurrent maker note offset: 0x%.4x", $newOffset);
             }
             my $filePos = ($$dirInfo{Base} || 0) + $dataPos + $footerPos;
-            $exifTool->HDump($filePos, 8, '[Canon MakerNotes footer]', $str);
+            $et->HDump($filePos, 8, '[Canon MakerNotes footer]', $str);
         }
     }
     # process as normal
-    return Image::ExifTool::Exif::ProcessExif($exifTool, $dirInfo, $tagTablePtr);
+    return Image::ExifTool::Exif::ProcessExif($et, $dirInfo, $tagTablePtr);
 }
 
 #------------------------------------------------------------------------------
@@ -1401,14 +1474,14 @@ sub ProcessCanon($$$)
 # Returns: 1 on success
 sub ProcessGE2($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt} or return 0;
     my $dirStart = $$dirInfo{DirStart} || 0;
 
     # these maker notes are missing the IFD entry count, but they
     # always have 25 entries, so write the entry count manually
     Set16u(25, $dataPt, $dirStart);
-    return Image::ExifTool::Exif::ProcessExif($exifTool, $dirInfo, $tagTablePtr);
+    return Image::ExifTool::Exif::ProcessExif($et, $dirInfo, $tagTablePtr);
 }
 
 #------------------------------------------------------------------------------
@@ -1417,21 +1490,21 @@ sub ProcessGE2($$$)
 # Returns: 1 on success, and updates $dirInfo if necessary for new directory
 sub ProcessUnknownOrPreview($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $dirStart = $$dirInfo{DirStart};
     my $dirLen = $$dirInfo{DirLen};
     # check to see if this is a preview image
     if ($dirLen > 6 and substr($$dataPt, $dirStart, 3) eq "\xff\xd8\xff") {
-        $exifTool->VerboseDir('PreviewImage');
-        if ($$exifTool{HTML_DUMP}) {
+        $et->VerboseDir('PreviewImage');
+        if ($$et{HTML_DUMP}) {
             my $pos = $$dirInfo{DataPos} + $$dirInfo{Base} + $dirStart;
-            $exifTool->HDump($pos, $dirLen, '(MakerNotes:PreviewImage data)', "Size: $dirLen bytes")
+            $et->HDump($pos, $dirLen, '(MakerNotes:PreviewImage data)', "Size: $dirLen bytes")
         }
-        $exifTool->FoundTag('PreviewImage', substr($$dataPt, $dirStart, $dirLen));
+        $et->FoundTag('PreviewImage', substr($$dataPt, $dirStart, $dirLen));
         return 1;
     }
-    return ProcessUnknown($exifTool, $dirInfo, $tagTablePtr);
+    return ProcessUnknown($et, $dirInfo, $tagTablePtr);
 }
 
 #------------------------------------------------------------------------------
@@ -1440,27 +1513,27 @@ sub ProcessUnknownOrPreview($$$)
 # Returns: directory data, '' to delete, or undef on error
 sub WriteUnknownOrPreview($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $dirStart = $$dirInfo{DirStart};
     my $dirLen = $$dirInfo{DirLen};
     my $newVal;
     # check to see if this is a preview image
     if ($dirLen > 6 and substr($$dataPt, $dirStart, 3) eq "\xff\xd8\xff") {
-        if ($$exifTool{NEW_VALUE}{$Image::ExifTool::Extra{PreviewImage}}) {
+        if ($$et{NEW_VALUE}{$Image::ExifTool::Extra{PreviewImage}}) {
             # write or delete new preview (if deleted, it can't currently be added back again)
-            $newVal = $exifTool->GetNewValues('PreviewImage') || '';
-            if ($exifTool->Options('Verbose') > 1) {
-                $exifTool->VerboseValue("- MakerNotes:PreviewImage", substr($$dataPt, $dirStart, $dirLen));
-                $exifTool->VerboseValue("+ MakerNotes:PreviewImage", $newVal) if $newVal;
+            $newVal = $et->GetNewValues('PreviewImage') || '';
+            if ($et->Options('Verbose') > 1) {
+                $et->VerboseValue("- MakerNotes:PreviewImage", substr($$dataPt, $dirStart, $dirLen));
+                $et->VerboseValue("+ MakerNotes:PreviewImage", $newVal) if $newVal;
             }
-            ++$$exifTool{CHANGED};
+            ++$$et{CHANGED};
         } else {
             $newVal = substr($$dataPt, $dirStart, $dirLen);
         }
     } else {
         # rewrite MakerNote IFD
-        $newVal = Image::ExifTool::Exif::WriteExif($exifTool, $dirInfo, $tagTablePtr);
+        $newVal = Image::ExifTool::Exif::WriteExif($et, $dirInfo, $tagTablePtr);
     }
     return $newVal;
 }
@@ -1471,23 +1544,23 @@ sub WriteUnknownOrPreview($$$)
 # Returns: 1 on success, and updates $dirInfo if necessary for new directory
 sub ProcessUnknown($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $success = 0;
 
-    my $loc = LocateIFD($exifTool, $dirInfo);
+    my $loc = LocateIFD($et, $dirInfo);
     if (defined $loc) {
-        $exifTool->{UnknownByteOrder} = GetByteOrder();
-        if ($exifTool->Options('Verbose') > 1) {
-            my $out = $exifTool->Options('TextOut');
-            my $indent = $exifTool->{INDENT};
+        $$et{UnknownByteOrder} = GetByteOrder();
+        if ($et->Options('Verbose') > 1) {
+            my $out = $et->Options('TextOut');
+            my $indent = $$et{INDENT};
             $indent =~ s/\| $/  /;
             printf $out "${indent}Found IFD at offset 0x%.4x in maker notes:\n",
                     $$dirInfo{DirStart} + $$dirInfo{DataPos} + $$dirInfo{Base};
         }
-        $success = Image::ExifTool::Exif::ProcessExif($exifTool, $dirInfo, $tagTablePtr);
+        $success = Image::ExifTool::Exif::ProcessExif($et, $dirInfo, $tagTablePtr);
     } else {
-        $exifTool->{UnknownByteOrder} = ''; # indicates we tried but didn't set byte order
-        $exifTool->Warn("Unrecognized $$dirInfo{DirName}", 1);
+        $$et{UnknownByteOrder} = ''; # indicates we tried but didn't set byte order
+        $et->Warn("Unrecognized $$dirInfo{DirName}", 1);
     }
     return $success;
 }
@@ -1512,7 +1585,7 @@ maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
