@@ -26,15 +26,16 @@ use strict;
 use vars qw($VERSION $AUTOLOAD);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.24';
+$VERSION = '1.29';
 
 sub ProcessPNG_tEXt($$$);
 sub ProcessPNG_iTXt($$$);
 sub ProcessPNG_Compressed($$$);
 sub CalculateCRC($;$$$);
 sub HexEncode($);
-sub AddChunks($$);
+sub AddChunks($$;@);
 sub Add_iCCP($$);
+sub DoneDir($$$;$);
 sub GetLangInfo($$);
 sub BuildTextChunk($$$$$);
 
@@ -117,11 +118,16 @@ $Image::ExifTool::PNG::colorType = -1;
     },
     iCCP => {
         Name => 'ICC_Profile',
+        Notes => 'this is where ExifTool will write a new ICC_Profile',
         SubDirectory => {
             TagTable => 'Image::ExifTool::ICC_Profile::Main',
             ProcessProc => \&ProcessPNG_Compressed,
         },
     },
+   'iCCP-name' => {
+        Name => 'ProfileName',
+        Notes => 'not a real tag ID, this tag represents the iCCP profile name',
+    },
 #   IDAT
 #   IEND
     IHDR => {
@@ -209,7 +215,8 @@ $Image::ExifTool::PNG::colorType = -1;
     },
     tXMP => {
         Name => 'XMP',
-        Notes => 'obsolete location specified by older XMP draft',
+        Notes => 'obsolete location specified by a September 2001 XMP draft',
+        NonStandard => 1,
         SubDirectory => { TagTable => 'Image::ExifTool::XMP::Main' },
     },
     vpAg => { # private imagemagick chunk
@@ -425,9 +432,9 @@ my %unreg = ( Notes => 'unregistered' );
    'XML:com.adobe.xmp' => {
         Name => 'XMP',
         Notes => q{
-            unregistered, but this is the location according to the XMP specification,
-            and is where ExifTool will add a new XMP chunk if the image didn't already
-            contain XMP
+            unregistered, but this is the location according to the June 2002 or later
+            XMP specification, and is where ExifTool will add a new XMP chunk if the
+            image didn't already contain XMP
         },
         SubDirectory => {
             TagTable => 'Image::ExifTool::XMP::Main',
@@ -438,16 +445,17 @@ my %unreg = ( Notes => 'unregistered' );
             # EXIF table must come first because we key on this in ProcessProfile()
             # (No condition because this is just for BuildTagLookup)
             Name => 'APP1_Profile',
-            %unreg,
+            Notes => 'unregistered.  This is where ExifTool will create new EXIF',
             SubDirectory => {
-                TagTable=>'Image::ExifTool::Exif::Main',
+                TagTable => 'Image::ExifTool::Exif::Main',
                 ProcessProc => \&ProcessProfile,
             },
         },
         {
             Name => 'APP1_Profile',
+            NonStandard => 1,
             SubDirectory => {
-                TagTable=>'Image::ExifTool::XMP::Main',
+                TagTable => 'Image::ExifTool::XMP::Main',
                 ProcessProc => \&ProcessProfile,
             },
         },
@@ -456,7 +464,7 @@ my %unreg = ( Notes => 'unregistered' );
         Name => 'EXIF_Profile',
         %unreg,
         SubDirectory => {
-            TagTable=>'Image::ExifTool::Exif::Main',
+            TagTable => 'Image::ExifTool::Exif::Main',
             ProcessProc => \&ProcessProfile,
         },
     },
@@ -478,7 +486,10 @@ my %unreg = ( Notes => 'unregistered' );
     },
    'Raw profile type iptc' => {
         Name => 'IPTC_Profile',
-        %unreg,
+        Notes => q{
+            unregistered.  May be either IPTC IIM or Photoshop IRB format.  This is
+            where ExifTool will add new IPTC, inside a Photoshop IRB container
+        },
         SubDirectory => {
             TagTable => 'Image::ExifTool::Photoshop::Main',
             ProcessProc => \&ProcessProfile,
@@ -487,11 +498,20 @@ my %unreg = ( Notes => 'unregistered' );
    'Raw profile type xmp' => {
         Name => 'XMP_Profile',
         %unreg,
+        NonStandard => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::XMP::Main',
             ProcessProc => \&ProcessProfile,
         },
     },
+    'Raw profile type 8bim' => {
+        Name => 'Photoshop_Profile',
+        %unreg,
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Photoshop::Main',
+            ProcessProc => \&ProcessProfile,
+        },
+    },
 );
 
 #------------------------------------------------------------------------------
@@ -538,22 +558,22 @@ sub GetLangInfo($$)
 # Returns: 1 on success
 sub FoundPNG($$$$;$$$$)
 {
-    my ($exifTool, $tagTablePtr, $tag, $val, $compressed, $outBuff, $enc, $lang) = @_;
+    my ($et, $tagTablePtr, $tag, $val, $compressed, $outBuff, $enc, $lang) = @_;
     return 0 unless defined $val;
-    my $verbose = $exifTool->Options('Verbose');
+    my $verbose = $et->Options('Verbose');
     my $id = $tag;  # generate tag ID which include language code
     if ($lang) {
         # case of language code must be normalized since they are case insensitive
         $lang = StandardLangCase($lang);
         $id .= '-' . $lang;
     }
-    my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $id) ||
+    my $tagInfo = $et->GetTagInfo($tagTablePtr, $id) ||
                   # (some software forgets to capitalize first letter)
-                  $exifTool->GetTagInfo($tagTablePtr, ucfirst($id));
+                  $et->GetTagInfo($tagTablePtr, ucfirst($id));
     # create alternate language tag if necessary
     if (not $tagInfo and $lang) {
-        $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tag) ||
-                   $exifTool->GetTagInfo($tagTablePtr, ucfirst($tag));
+        $tagInfo = $et->GetTagInfo($tagTablePtr, $tag) ||
+                   $et->GetTagInfo($tagTablePtr, ucfirst($tag));
         $tagInfo = GetLangInfo($tagInfo, $lang) if $tagInfo;
     }
 #
@@ -583,19 +603,19 @@ sub FoundPNG($$$$;$$$$)
             $deflateErr = "Unknown compression method $compressed for $tag";
         }
         if ($compressed and $verbose and $tagInfo and $$tagInfo{SubDirectory}) {
-            $exifTool->VerboseDir("Unable to decompress $$tagInfo{Name}", 0, length($val));
+            $et->VerboseDir("Unable to decompress $$tagInfo{Name}", 0, length($val));
         }
         # issue warning if relevant
         if ($deflateErr and (not $outBuff or
-            ($tagInfo and $$tagInfo{SubDirectory} and $$exifTool{EDIT_DIRS}{$$tagInfo{Name}})))
+            ($tagInfo and $$tagInfo{SubDirectory} and $$et{EDIT_DIRS}{$$tagInfo{Name}})))
         {
-            $exifTool->Warn($deflateErr);
+            $et->Warn($deflateErr);
             $noCompressLib = 1 if $deflateErr =~ /^Install/;
         }
     }
     # translate character encoding if necessary (tEXt/zTXt/iTXt string values only)
     if ($enc and not $compressed and not ($tagInfo and $$tagInfo{SubDirectory})) {
-        $val = $exifTool->Decode($val, $enc);
+        $val = $et->Decode($val, $enc);
     }
 #
 # extract information from subdirectory if available
@@ -605,15 +625,15 @@ sub FoundPNG($$$$;$$$$)
         my $processed;
         if ($$tagInfo{SubDirectory} and not $compressed) {
             my $len = length $val;
-            if ($verbose and $exifTool->{INDENT} ne '  ') {
+            if ($verbose and $$et{INDENT} ne '  ') {
                 if ($wasCompressed and $verbose > 2) {
                     my $name = $tagName;
                     $wasCompressed and $name = "Decompressed $name";
-                    $exifTool->VerboseDir($name, 0, $len);
-                    $exifTool->VerboseDump(\$val);
+                    $et->VerboseDir($name, 0, $len);
+                    $et->VerboseDump(\$val);
                 }
                 # don't indent next directory (since it is really the same data)
-                $exifTool->{INDENT} =~ s/..$//;
+                $$et{INDENT} =~ s/..$//;
             }
             my $subdir = $$tagInfo{SubDirectory};
             my $processProc = $$subdir{ProcessProc};
@@ -634,13 +654,27 @@ sub FoundPNG($$$$;$$$$)
             undef $processProc if $wasCompressed and $processProc eq \&ProcessPNG_Compressed;
             # rewrite this directory if necessary (but always process TextualData normally)
             if ($outBuff and not $processProc and $subTable ne \%Image::ExifTool::PNG::TextualData) {
-                return 1 unless $exifTool->{EDIT_DIRS}->{$tagName};
-                $$outBuff = $exifTool->WriteDirectory(\%subdirInfo, $subTable);
-                # if this was an XMP directory, we must make it read-only
-                $tagName eq 'XMP' and Image::ExifTool::XMP::ValidateXMP(\$outBuff,'r');
-                delete $exifTool->{ADD_DIRS}->{$tagName};
+                return 1 unless $$et{EDIT_DIRS}{$tagName};
+                $$outBuff = $et->WriteDirectory(\%subdirInfo, $subTable);
+                if ($tagName eq 'XMP' and $$outBuff) {
+                    if ($$et{FoundIDAT} and $$et{DEL_GROUP}{XMP}) {
+                        $et->VPrint(0,'  Deleting XMP');
+                        $$outBuff = '';
+                    } else {
+                        # make sure the XMP is marked as read-only
+                        Image::ExifTool::XMP::ValidateXMP($outBuff,'r');
+                    }
+                }
+                DoneDir($et, $tagName, $outBuff, $$tagInfo{NonStandard});
             } else {
-                $processed = $exifTool->ProcessDirectory(\%subdirInfo, $subTable, $processProc);
+                # issue warning for standard XMP after IDAT (PNGEarlyXMP option)
+                if ($tagName eq 'XMP' and not $$tagInfo{NonStandard} and
+                    $$et{FoundIDAT} and $$et{FoundIDAT} == 2)
+                {
+                    $et->Warn('XMP found after PNG IDAT');
+                    $$et{FoundIDAT} = 1;
+                }
+                $processed = $et->ProcessDirectory(\%subdirInfo, $subTable, $processProc);
             }
             $compressed = 1;    # pretend this is compressed since it is binary data
         }
@@ -652,43 +686,45 @@ sub FoundPNG($$$$;$$$$)
             {
                 # write new value for this tag if necessary
                 my $newVal;
-                if ($exifTool->{DEL_GROUP}->{PNG}) {
+                if ($$et{DEL_GROUP}{PNG} or $$et{PNGDoneTag}{$tag} or
+                    $$et{PNGDoneTag}{ucfirst $tag})
+                {
                     # remove this tag now, but keep in ADD_PNG list to add back later
                     $isOverwriting = 1;
                 } else {
                     # remove this from the list of PNG tags to add
-                    delete $exifTool->{ADD_PNG}->{$id};
+                    delete $$et{ADD_PNG}{$id};
                     # (also handle case of tEXt tags written with lowercase first letter)
-                    delete $exifTool->{ADD_PNG}->{ucfirst($id)};
-                    my $nvHash = $exifTool->GetNewValueHash($tagInfo);
-                    $isOverwriting = Image::ExifTool::IsOverwriting($nvHash);
+                    delete $$et{ADD_PNG}{ucfirst($id)};
+                    my $nvHash = $et->GetNewValueHash($tagInfo);
+                    $isOverwriting = $et->IsOverwriting($nvHash);
                     if (defined $deflateErr) {
-                        $newVal = Image::ExifTool::GetNewValues($nvHash);
+                        $newVal = $et->GetNewValues($nvHash);
                         # can only write tag now if always overwriting
                         if ($isOverwriting > 0) {
                             $val = '<deflate error>';
                         } elsif ($isOverwriting) {
                             $isOverwriting = 0; # can't overwrite
-                            $exifTool->Warn($deflateErr) if $deflateErr;
+                            $et->Warn($deflateErr) if $deflateErr;
                         }
                     } else {
                         if ($isOverwriting < 0) {
-                            $isOverwriting = Image::ExifTool::IsOverwriting($nvHash, $val);
+                            $isOverwriting = $et->IsOverwriting($nvHash, $val);
                         }
                         # (must get new value after IsOverwriting() in case it was shifted)
-                        $newVal = Image::ExifTool::GetNewValues($nvHash);
+                        $newVal = $et->GetNewValues($nvHash);
                     }
                 }
                 if ($isOverwriting) {
                     $$outBuff = (defined $newVal) ? $newVal : '';
-                    ++$exifTool->{CHANGED};
-                    $exifTool->VerboseValue("- PNG:$tagName", $val);
-                    $exifTool->VerboseValue("+ PNG:$tagName", $newVal) if defined $newVal;
+                    ++$$et{CHANGED};
+                    $et->VerboseValue("- PNG:$tagName", $val);
+                    $et->VerboseValue("+ PNG:$tagName", $newVal) if defined $newVal;
                 }
             }
             if (defined $$outBuff and length $$outBuff) {
                 if ($enc) { # must be tEXt/zTXt/iTXt if $enc is set
-                    $$outBuff = BuildTextChunk($exifTool, $tag, $tagInfo, $$outBuff, $lang);
+                    $$outBuff = BuildTextChunk($et, $tag, $tagInfo, $$outBuff, $lang);
                 } elsif ($wasCompressed) {
                     # re-compress the output data
                     my $deflate;
@@ -701,7 +737,7 @@ sub FoundPNG($$$$;$$$$)
                             undef $$outBuff;
                         }
                     }
-                    $$outBuff or $exifTool->Warn("PNG:$tagName not written (compress error)");
+                    $$outBuff or $et->Warn("PNG:$tagName not written (compress error)");
                 }
             }
             return 1;
@@ -714,7 +750,7 @@ sub FoundPNG($$$$;$$$$)
         $$tagInfo{LangCode} = $lang if $lang;
         # make unknown profiles binary data type
         $$tagInfo{Binary} = 1 if $tag =~ /^Raw profile type /;
-        Image::ExifTool::AddTagToTable($tagTablePtr, $tag, $tagInfo);
+        AddTagToTable($tagTablePtr, $tag, $tagInfo);
     }
 #
 # store this tag information
@@ -724,7 +760,7 @@ sub FoundPNG($$$$;$$$$)
         # since we aren't decoding it anyway;
         my $subdir = $$tagInfo{SubDirectory};
         delete $$tagInfo{SubDirectory};
-        $exifTool->VerboseInfo($tag, $tagInfo,
+        $et->VerboseInfo($tag, $tagInfo,
             Table  => $tagTablePtr,
             DataPt => \$val,
         );
@@ -736,7 +772,7 @@ sub FoundPNG($$$$;$$$$)
         $$tagInfo{RawConv} = '\$val';
         $delRawConv = 1;
     }
-    $exifTool->FoundTag($tagInfo, $val);
+    $et->FoundTag($tagInfo, $val);
     delete $$tagInfo{RawConv} if $delRawConv;
     return 1;
 }
@@ -747,7 +783,7 @@ sub FoundPNG($$$$;$$$$)
 # Returns: 1 on success
 sub ProcessProfile($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $tagInfo = $$dirInfo{TagInfo};
     my $outBuff = $$dirInfo{OutBuff};
@@ -762,17 +798,17 @@ sub ProcessProfile($$$)
     my $buff = pack('H*', join('',split(' ',$3)));
     my $actualLen = length $buff;
     if ($len ne $actualLen) {
-        $exifTool->Warn("$tagName is wrong size (should be $len bytes but is $actualLen)");
+        $et->Warn("$tagName is wrong size (should be $len bytes but is $actualLen)");
         $len = $actualLen;
     }
-    my $verbose = $exifTool->Options('Verbose');
+    my $verbose = $et->Options('Verbose');
     if ($verbose) {
         if ($verbose > 2) {
-            $exifTool->VerboseDir("Decoded $tagName", 0, $len);
-            $exifTool->VerboseDump(\$buff);
+            $et->VerboseDir("Decoded $tagName", 0, $len);
+            $et->VerboseDump(\$buff);
         }
         # don't indent next directory (since it is really the same data)
-        $exifTool->{INDENT} =~ s/..$//;
+        $$et{INDENT} =~ s/..$//;
     }
     my %dirInfo = (
         Parent   => 'PNG',
@@ -783,22 +819,28 @@ sub ProcessProfile($$$)
         Base     => 0,
         OutFile  => $outBuff,
     );
+    $$et{PROCESSED} = { };    # reset processed directory offsets
     my $processed = 0;
-    my $oldChanged = $exifTool->{CHANGED};
+    my $oldChanged = $$et{CHANGED};
     my $exifTable = GetTagTable('Image::ExifTool::Exif::Main');
-    my $editDirs = $exifTool->{EDIT_DIRS};
-    my $addDirs = $exifTool->{ADD_DIRS};
+    my $editDirs = $$et{EDIT_DIRS};
+
     if ($tagTablePtr ne $exifTable) {
+        # this is unfortunate, but the "IPTC" profile may be stored as either
+        # IPTC IIM or a Photoshop IRB resource, so we must test for this
+        if ($tagName eq 'IPTC_Profile' and $buff =~ /^\x1c/) {
+            $tagTablePtr = GetTagTable('Image::ExifTool::IPTC::Main');
+        }
         # process non-EXIF and non-APP1 profile as-is
         if ($outBuff) {
             # no need to rewrite this if not editing tags in this directory
             my $dir = $tagName;
             $dir =~ s/_Profile// unless $dir =~ /^ICC/;
             return 1 unless $$editDirs{$dir};
-            $$outBuff = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr);
-            delete $$addDirs{$dir};
+            $$outBuff = $et->WriteDirectory(\%dirInfo, $tagTablePtr);
+            DoneDir($et, $dir, $outBuff);
         } else {
-            $processed = $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
+            $processed = $et->ProcessDirectory(\%dirInfo, $tagTablePtr);
         }
     } elsif ($buff =~ /^$Image::ExifTool::exifAPP1hdr/) {
         # APP1 EXIF information
@@ -807,12 +849,12 @@ sub ProcessProfile($$$)
         $dirInfo{DirStart} += $hdrLen;
         $dirInfo{DirLen} -= $hdrLen;
         if ($outBuff) {
-            $$outBuff = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr,
-                                                  \&Image::ExifTool::WriteTIFF);
+            $$outBuff = $et->WriteDirectory(\%dirInfo, $tagTablePtr,
+                                            \&Image::ExifTool::WriteTIFF);
             $$outBuff = $Image::ExifTool::exifAPP1hdr . $$outBuff if $$outBuff;
-            delete $$addDirs{IFD0};
+            DoneDir($et, 'IFD0', $outBuff);
         } else {
-            $processed = $exifTool->ProcessTIFF(\%dirInfo);
+            $processed = $et->ProcessTIFF(\%dirInfo);
         }
     } elsif ($buff =~ /^$Image::ExifTool::xmpAPP1hdr/) {
         # APP1 XMP information
@@ -822,30 +864,30 @@ sub ProcessProfile($$$)
         $dirInfo{DirLen} -= $hdrLen;
         if ($outBuff) {
             return 1 unless $$editDirs{XMP};
-            $$outBuff = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr);
+            $$outBuff = $et->WriteDirectory(\%dirInfo, $tagTablePtr);
             $$outBuff and $$outBuff = $Image::ExifTool::xmpAPP1hdr . $$outBuff;
-            delete $$addDirs{XMP};
+            DoneDir($et, 'XMP', $outBuff, 1);
         } else {
-            $processed = $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
+            $processed = $et->ProcessDirectory(\%dirInfo, $tagTablePtr);
         }
     } elsif ($buff =~ /^(MM\0\x2a|II\x2a\0)/) {
         # TIFF information (haven't seen this, but what the heck...)
         return 1 if $outBuff and not $$editDirs{IFD0};
         if ($outBuff) {
-            $$outBuff = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr,
-                                                  \&Image::ExifTool::WriteTIFF);
+            $$outBuff = $et->WriteDirectory(\%dirInfo, $tagTablePtr,
+                                            \&Image::ExifTool::WriteTIFF);
             $$outBuff = $Image::ExifTool::exifAPP1hdr . $$outBuff if $$outBuff;
-            delete $$addDirs{IFD0};
+            CheckDir($et, 'IFD0', $outBuff);
         } else {
-            $processed = $exifTool->ProcessTIFF(\%dirInfo);
+            $processed = $et->ProcessTIFF(\%dirInfo);
         }
     } else {
         my $profName = $profileType;
         $profName =~ tr/\x00-\x1f\x7f-\xff/./;
-        $exifTool->Warn("Unknown raw profile '$profName'");
+        $et->Warn("Unknown raw profile '$profName'");
     }
     if ($outBuff and defined $$outBuff and length $$outBuff) {
-        if ($exifTool->{CHANGED} != $oldChanged) {
+        if ($$et{CHANGED} != $oldChanged) {
             my $hdr = sprintf("\n%s\n%8d\n", $profileType, length($$outBuff));
             # hex encode the data
             $$outBuff = $hdr . HexEncode($outBuff);
@@ -863,7 +905,7 @@ sub ProcessProfile($$$)
 # Notes: writes new chunk data to ${$$dirInfo{OutBuff}} if writing tag
 sub ProcessPNG_Compressed($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my ($tag, $val) = split /\0/, ${$$dirInfo{DataPt}}, 2;
     return 0 unless defined $val;
     # set compressed to 2 + compression method to decompress the data
@@ -874,12 +916,15 @@ sub ProcessPNG_Compressed($$$)
     my $outBuff = $$dirInfo{OutBuff};
     # use the PNG chunk tag instead of the embedded tag name for iCCP chunks
     if ($$dirInfo{TagInfo} and $$dirInfo{TagInfo}->{Name} eq 'ICC_Profile') {
-        $tag = 'iCCP';
+        $et->VerboseDir('iCCP');
         $tagTablePtr = \%Image::ExifTool::PNG::Main;
-        $success = FoundPNG($exifTool, $tagTablePtr, $tag, $val, $compressed, $outBuff);
+        if (length($tag) and not $outBuff) {
+            FoundPNG($et, $tagTablePtr, 'iCCP-name', $tag);
+        }
+        $success = FoundPNG($et, $tagTablePtr, 'iCCP', $val, $compressed, $outBuff);
         $$outBuff = $hdr . $$outBuff if $outBuff and $$outBuff;
     } else {
-        $success = FoundPNG($exifTool, $tagTablePtr, $tag, $val, $compressed, $outBuff, 'Latin');
+        $success = FoundPNG($et, $tagTablePtr, $tag, $val, $compressed, $outBuff, 'Latin');
     }
     return $success;
 }
@@ -891,10 +936,10 @@ sub ProcessPNG_Compressed($$$)
 # Notes: writes new chunk data to ${$$dirInfo{OutBuff}} if writing tag
 sub ProcessPNG_tEXt($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my ($tag, $val) = split /\0/, ${$$dirInfo{DataPt}}, 2;
     my $outBuff = $$dirInfo{OutBuff};
-    return FoundPNG($exifTool, $tagTablePtr, $tag, $val, undef, $outBuff, 'Latin');
+    return FoundPNG($et, $tagTablePtr, $tag, $val, undef, $outBuff, 'Latin');
 }
 
 #------------------------------------------------------------------------------
@@ -904,7 +949,7 @@ sub ProcessPNG_tEXt($$$)
 # Notes: writes new chunk data to ${$$dirInfo{OutBuff}} if writing tag
 sub ProcessPNG_iTXt($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my ($tag, $dat) = split /\0/, ${$$dirInfo{DataPt}}, 2;
     return 0 unless defined $dat and length($dat) >= 4;
     my ($compressed, $meth) = unpack('CC', $dat);
@@ -912,7 +957,7 @@ sub ProcessPNG_iTXt($$$)
     # set compressed flag so we will decompress it in FoundPNG()
     $compressed and $compressed = 2 + $meth;
     my $outBuff = $$dirInfo{OutBuff};
-    return FoundPNG($exifTool, $tagTablePtr, $tag, $val, $compressed, $outBuff, 'UTF8', $lang);
+    return FoundPNG($et, $tagTablePtr, $tag, $val, $compressed, $outBuff, 'UTF8', $lang);
 }
 
 #------------------------------------------------------------------------------
@@ -921,7 +966,7 @@ sub ProcessPNG_iTXt($$$)
 # Returns: 1 on success, 0 if this wasn't a valid PNG image, or -1 on write error
 sub ProcessPNG($$)
 {
-    my ($exifTool, $dirInfo) = @_;
+    my ($et, $dirInfo) = @_;
     my $outfile = $$dirInfo{OutFile};
     my $raf = $$dirInfo{RAF};
     my $datChunk = '';
@@ -931,34 +976,44 @@ sub ProcessPNG($$)
 
     # check to be sure this is a valid PNG/MNG/JNG image
     return 0 unless $raf->Read($sig,8) == 8 and $pngLookup{$sig};
+
+    my $earlyXMP = $et->Options('PNGEarlyXMP');
     if ($outfile) {
-        delete $$exifTool{TextChunkType};
+        delete $$et{TextChunkType};
         Write($outfile, $sig) or $err = 1 if $outfile;
         # can only add tags in Main and TextualData tables
-        $exifTool->{ADD_PNG} = $exifTool->GetNewTagInfoHash(
+        $$et{ADD_PNG} = $et->GetNewTagInfoHash(
             \%Image::ExifTool::PNG::Main,
             \%Image::ExifTool::PNG::TextualData);
+        # NOTE: PNGDoneTag and PNGDoneDir are used to keep track of metadata added
+        # before the PNG IEND chunk is encountered.  Currently this is implemented
+        # only for XMP (written before IDAT with the PNGEarlyXMP option), but
+        # may be implemented in the future for other types - PH
+        $$et{PNGDoneTag} = { };
+        $$et{PNGDoneDir} = { };
         # initialize with same directories, with PNG tags taking priority
-        $exifTool->InitWriteDirs(\%pngMap,'PNG');
+        $et->InitWriteDirs(\%pngMap,'PNG');
+        # write XMP before IDAT if we would delete it later anyway
+        $earlyXMP = 1 if $$et{DEL_GROUP}{XMP};
     }
     my ($fileType, $hdrChunk, $endChunk) = @{$pngLookup{$sig}};
-    $exifTool->SetFileType($fileType);  # set the FileType tag
+    $et->SetFileType($fileType);  # set the FileType tag
     SetByteOrder('MM'); # PNG files are big-endian
     my $tagTablePtr = GetTagTable('Image::ExifTool::PNG::Main');
     my $mngTablePtr;
     if ($fileType ne 'PNG') {
         $mngTablePtr = GetTagTable('Image::ExifTool::MNG::Main');
     }
-    my $verbose = $exifTool->Options('Verbose');
-    my $out = $exifTool->Options('TextOut');
+    my $verbose = $et->Options('Verbose');
+    my $out = $et->Options('TextOut');
     my ($hbuf, $dbuf, $cbuf, $foundHdr);
 
     # process the PNG/MNG/JNG chunks
     undef $noCompressLib;
     for (;;) {
-        $raf->Read($hbuf,8) == 8 or $exifTool->Warn("Truncated $fileType image"), last;
+        $raf->Read($hbuf,8) == 8 or $et->Warn("Truncated $fileType image"), last;
         my ($len, $chunk) = unpack('Na4',$hbuf);
-        $len > 0x7fffffff and $exifTool->Warn("Invalid $fileType box size"), last;
+        $len > 0x7fffffff and $et->Warn("Invalid $fileType box size"), last;
         if ($verbose) {
             # don't dump image data chunks in verbose mode (only give count instead)
             if ($datCount and $chunk ne $datChunk) {
@@ -976,11 +1031,15 @@ sub ProcessPNG($$)
         if ($outfile) {
             if ($chunk eq 'IEND') {
                 # add any new chunks immediately before the IEND chunk
-                AddChunks($exifTool, $outfile) or $err = 1;
+                AddChunks($et, $outfile) or $err = 1;
             } elsif ($chunk eq 'PLTE' or $chunk eq 'IDAT') {
+                # add XMP before IDAT if specified
+                if ($earlyXMP and $chunk eq 'IDAT') {
+                    AddChunks($et, $outfile, 'XMP') or $err = 1;
+                }
                 # iCCP chunk must come before PLTE and IDAT
                 # (ignore errors -- will add later as text profile if this fails)
-                Add_iCCP($exifTool, $outfile);
+                Add_iCCP($et, $outfile);
             }
         }
         if ($chunk eq $endChunk) {
@@ -995,18 +1054,20 @@ sub ProcessPNG($$)
             $ok = 1;
             last;
         }
+        # set FoundIDAT flag: 1=after IDAT, 2=after IDAT and warn about late XMP
+        $$et{FoundIDAT} = $earlyXMP ? 2 : 1 if $chunk eq 'IDAT';
         # read chunk data and CRC
         unless ($raf->Read($dbuf,$len)==$len and $raf->Read($cbuf, 4)==4) {
-            $exifTool->Warn("Corrupted $fileType image");
+            $et->Warn("Corrupted $fileType image");
             last;
         }
         unless ($foundHdr) {
             if ($chunk eq $hdrChunk) {
                 $foundHdr = 1;
             } elsif ($hdrChunk eq 'IHDR' and $chunk eq 'CgBI') {
-                $exifTool->Warn('Non-standard PNG image (Apple iPhone format)');
+                $et->Warn('Non-standard PNG image (Apple iPhone format)');
             } else {
-                $exifTool->Warn("$fileType image did not start with $hdrChunk");
+                $et->Warn("$fileType image did not start with $hdrChunk");
                 last;
             }
         }
@@ -1014,29 +1075,29 @@ sub ProcessPNG($$)
             # check CRC when in verbose mode (since we don't care about speed)
             my $crc = CalculateCRC(\$hbuf, undef, 4);
             $crc = CalculateCRC(\$dbuf, $crc);
-            $crc == unpack('N',$cbuf) or $exifTool->Warn("Bad CRC for $chunk chunk");
+            $crc == unpack('N',$cbuf) or $et->Warn("Bad CRC for $chunk chunk");
             if ($datChunk) {
                 Write($outfile, $hbuf, $dbuf, $cbuf) or $err = 1 if $outfile;
                 next;
             }
             print $out "$fileType $chunk ($len bytes):\n";
-            $exifTool->VerboseDump(\$dbuf, Addr => $raf->Tell() - $len - 4) if $verbose > 2;
+            $et->VerboseDump(\$dbuf, Addr => $raf->Tell() - $len - 4) if $verbose > 2;
         }
         # only extract information from chunks in our tables
         my ($theBuff, $outBuff);
         $outBuff = \$theBuff if $outfile;
         if ($$tagTablePtr{$chunk}) {
-            FoundPNG($exifTool, $tagTablePtr, $chunk, $dbuf, undef, $outBuff);
+            FoundPNG($et, $tagTablePtr, $chunk, $dbuf, undef, $outBuff);
         } elsif ($mngTablePtr and $$mngTablePtr{$chunk}) {
-            FoundPNG($exifTool, $mngTablePtr, $chunk, $dbuf, undef, $outBuff);
+            FoundPNG($et, $mngTablePtr, $chunk, $dbuf, undef, $outBuff);
         }
         if ($outfile) {
             if (defined $theBuff) {
                 next unless length $theBuff; # empty if we deleted the information
                 # change chunk type if necessary
-                if ($$exifTool{TextChunkType}) {
-                    $chunk = $$exifTool{TextChunkType};
-                    delete $$exifTool{TextChunkType};
+                if ($$et{TextChunkType}) {
+                    $chunk = $$et{TextChunkType};
+                    delete $$et{TextChunkType};
                 }
                 $hbuf = pack('Na4', length($theBuff), $chunk);
                 $dbuf = $theBuff;
@@ -1071,7 +1132,7 @@ and JNG (JPEG Network Graphics) images.
 
 =head1 AUTHOR
 
-Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
