@@ -15,7 +15,7 @@ use Image::ExifTool::Exif;
 use Image::ExifTool::IPTC;
 use Image::ExifTool::XMP;
 
-$VERSION = '1.00';
+$VERSION = '1.04';
 
 sub ProcessPhotoMechanic($$);
 
@@ -49,7 +49,7 @@ my %colorClasses = (
     },
 );
 
-# rawCrop coordinate conversions
+# raw/preview crop coordinate conversions
 my %rawCropConv = (
     ValueConv => '$val / 655.36',
     ValueConvInv => 'int($val * 655.36 + 0.5)',
@@ -64,22 +64,10 @@ my %rawCropConv = (
     CHECK_PROC => \&Image::ExifTool::IPTC::CheckIPTC,
     WRITABLE => 1,
     FORMAT => 'int32s',
-    209 => {
-        Name => 'RawCropLeft',
-        %rawCropConv,
-    },
-    210 => {
-        Name => 'RawCropTop',
-        %rawCropConv,
-    },
-    211 => {
-        Name => 'RawCropRight',
-        %rawCropConv,
-    },
-    212 => {
-        Name => 'RawCropBottom',
-        %rawCropConv,
-    },
+    209 => { Name => 'RawCropLeft',   %rawCropConv },
+    210 => { Name => 'RawCropTop',    %rawCropConv },
+    211 => { Name => 'RawCropRight',  %rawCropConv },
+    212 => { Name => 'RawCropBottom', %rawCropConv },
     213 => 'ConstrainedCropWidth',
     214 => 'ConstrainedCropHeight',
     215 => 'FrameNum',
@@ -105,6 +93,10 @@ my %rawCropConv = (
         PrintConv => \%colorClasses,
     },
     223 => 'Rating',
+    236 => { Name => 'PreviewCropLeft',   %rawCropConv },
+    237 => { Name => 'PreviewCropTop',    %rawCropConv },
+    238 => { Name => 'PreviewCropRight',  %rawCropConv },
+    239 => { Name => 'PreviewCropBottom', %rawCropConv },
 );
 
 # PhotoMechanic XMP properties
@@ -118,9 +110,15 @@ my %rawCropConv = (
         namespace prefix is "photomechanic" but ExifTool shortens this in
         the "XMP-photomech" family 1 group name.
     },
+    ColorClass  => {
+        Writable => 'integer',
+        PrintConv => \%colorClasses,
+    },
     CountryCode => { Avoid => 1, Groups => { 2 => 'Location' } },
     EditStatus  => { },
+    PMVersion   => { },
     Prefs       => {
+        Notes => 'format is "Tagged:0, ColorClass:1, Rating:2, FrameNum:3"',
         PrintConv => q{
             $val =~ s[\s*(\d+):\s*(\d+):\s*(\d+):\s*(\S*)]
                      [Tagged:$1, ColorClass:$2, Rating:$3, FrameNum:$4];
@@ -132,6 +130,7 @@ my %rawCropConv = (
             return $val;
         },
     },
+    Tagged      => { Writable => 'boolean', PrintConv => { False => 'No', True => 'Yes' } },
     TimeCreated => {
         Avoid => 1,
         Groups => { 2 => 'Time' },
@@ -149,12 +148,12 @@ my %rawCropConv = (
 # - updates DirLen to trailer length
 sub ProcessPhotoMechanic($$)
 {
-    my ($exifTool, $dirInfo) = @_;
+    my ($et, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
     my $offset = $$dirInfo{Offset} || 0;
     my $outfile = $$dirInfo{OutFile};
-    my $verbose = $exifTool->Options('Verbose');
-    my $out = $exifTool->Options('TextOut');
+    my $verbose = $et->Options('Verbose');
+    my $out = $et->Options('TextOut');
     my $rtnVal = 0;
     my ($buff, $footer);
 
@@ -163,13 +162,13 @@ sub ProcessPhotoMechanic($$)
         last unless $raf->Seek(-12-$offset, 2) and $raf->Read($footer, 12) == 12;
         last unless $footer =~ /cbipcbbl$/;
         my $size = unpack('N', $footer);
-    
+
         if ($size & 0x80000000 or not $raf->Seek(-$size-12, 1)) {
-            $exifTool->Warn('Bad PhotoMechanic trailer');
+            $et->Warn('Bad PhotoMechanic trailer');
             last;
         }
         unless ($raf->Read($buff, $size) == $size) {
-            $exifTool->Warn('Error reading PhotoMechanic trailer');
+            $et->Warn('Error reading PhotoMechanic trailer');
             last;
         }
         $rtnVal = 1;    # we read the trailer successfully
@@ -177,7 +176,7 @@ sub ProcessPhotoMechanic($$)
         # set variables returned in dirInfo hash
         $$dirInfo{DataPos} = $raf->Tell() - $size;
         $$dirInfo{DirLen} = $size + 12;
-    
+
         my %dirInfo = (
             DataPt => \$buff,
             DataPos => $$dirInfo{DataPos},
@@ -188,20 +187,20 @@ sub ProcessPhotoMechanic($$)
         my $tagTablePtr = GetTagTable('Image::ExifTool::PhotoMechanic::Main');
         if (not $outfile) {
             # extract trailer information
-            $exifTool->DumpTrailer($dirInfo) if $verbose or $exifTool->{HTML_DUMP};
-            $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
-        } elsif ($exifTool->{DEL_GROUP}->{PhotoMechanic}) {
+            $et->DumpTrailer($dirInfo) if $verbose or $$et{HTML_DUMP};
+            $et->ProcessDirectory(\%dirInfo, $tagTablePtr);
+        } elsif ($$et{DEL_GROUP}{PhotoMechanic}) {
             # delete the trailer
             $verbose and printf $out "  Deleting PhotoMechanic trailer\n";
-            ++$exifTool->{CHANGED};
+            ++$$et{CHANGED};
         } else {
             # rewrite the trailer
             my $newPt;
-            my $newBuff = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr);
+            my $newBuff = $et->WriteDirectory(\%dirInfo, $tagTablePtr);
             if (defined $newBuff) {
                 $newPt = \$newBuff; # write out the modified trailer
                 my $pad = 0x800 - length($newBuff);
-                if ($pad > 0 and not $exifTool->Options('Compact')) {
+                if ($pad > 0 and not $et->Options('Compact')) {
                     # pad out to 2kB like PhotoMechanic does
                     $newBuff .= "\0" x $pad;
                 }
@@ -237,7 +236,7 @@ write information written by the Camera Bits Photo Mechanic software.
 
 =head1 AUTHOR
 
-Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
